### ===========================================================================
###
###                        Lehrstuhl fuer Informatik III
###                                Prof. Nagl 
###           RWTH Aachen, Ahornstr. 55, 52074 Aachen, Germany
###
### ===========================================================================
###
### gen_menu.tcl
### Stand 22.01.99
###
### Generierung von Menupunkten fuer Transaktionen/Produktionen aus config.ecu
###
### neue Fetaures:
###   - Einfuegen von Separatoren durch das Schluesselwort 'SEPARATOR'
###       die Schreibweise ist casesensitiv, um Verwechselungen mit Kommandos 
###       zu vermeiden
###   - Einfuegen von Submenus durch das Schluesselwort 'SUBMENU'
###       die Schreibweise ist casesensitiv
###   - beliebige (?) Schachtelungstiefe der Submenus
###   - Leerzeichen zu Zeilenbeginn werden ignoriert
###
### 22.01.99 RS
###
### --------------------------------------------------------------------------
###
### Ausschnittsbeispiel aus config.ecu:
###
###
### MENU AddElements {
###
###  SUBMENU Element {       
###   SUBMENU Primitive {       
###    AddClass AddClass
###    AddPackage AddPackage
###    AddPrimitiveType AddPrimitiveType
###   }
###   SEPARATOR
###   SUBMENU Relation {       
###    AddAssociation AddAssociation
###    SUBMENU Relation2 {       
###     AddFriendRel AddFriendRel
###     AddGeneralization AddGeneralization
###    }
###    AddImport AddImport
###   }
###   SEPARATOR
###   SUBMENU Reference {       
###    AddReference AddReference
###   }
###  }
### 
###  SUBMENU InPackage {       
###   AddClassInPackage AddClassInPackage
###   AddPackageInPackage AddPackageInPackage
###   AddPrimitiveTypeInPackage AddPrimitiveTypeInPackage
###   AddReferenceInPackage AddReferenceInPackage
###  }
###  SUBMENU Role {       
###   AddSourceRole AddSourceRole
###   AddTargetRole AddTargetRole
###   AddSourceRoleToAssoc AddSourceRoleToAssoc
###   AddTargetRoleToAssoc AddTargetRoleToAssoc
###  }
### }
###
### --------------------------------------------------------------------------

#### defines:
set menusymbol MENU
set menusymbollength [string length $menusymbol]

global  GENDIR
global  line
global  menufile
global  submenusymbol
set submenusymbol SUBMENU

#if {[file exist "$GENDIR/config.ecu"] == 0} { return ok }
# das heisst wir sind in einer Interprete sitzung !
# ELSE:
# Abfrage besser:
if { [isProto] == 0 } { return ok }

######### procs die hier local gebraucht werden
proc insert_new_toplevel {menu_id menuname} {
    # Einfuegen eines neuen Hauptmenupunktes
    menubutton .frame1.$menu_id \
    -background {gainsboro} \
    -menu .frame1.$menu_id.m \
    -text $menuname

    menu .frame1.$menu_id.m \
	-background {gainsboro} 
}

proc insert_new_submenu {menu_id submenu_id submenuname} {
    # Einfuegen eines neuen Untermenupunktes
    .frame1.$menu_id  add cascade -label $submenuname -menu .frame1.$menu_id.$submenu_id
    menu .frame1.$menu_id.$submenu_id
}

proc create_submenu {menu_id} {
    # Erstellen eines neuen Submenus
    # die Auswertung erfolgt rekursiv, d.h. eine beliebige (?)
    # Schachtelungstiefe ist moeglich
    global line
    global menufile
    global submenusymbol

    set submenuname [string trimleft  $line $submenusymbol]
    set submenuname [string trimright $submenuname "\{"]
    set submenuname [string trim $submenuname]
    set submenu_id  [string tolower $submenuname]
    
    ### fuege hier neues submenu $submenuname ein
    ### lese dann alle Submenupunkte ein
    insert_new_submenu $menu_id $submenu_id $submenuname
    gets $menufile line 
    set line [string trim $line]
    
    # Zeilen bis zum Submenuende auswerten
    while {[string first "\}" $line] != 0} {
	
	if {[string compare $line ""] ==  0} {
	    # Leerzeile ueberlesen
	    gets $menufile line
	    set line [string trim $line]
	    continue
	}
	if {[string first "#" $line ] != -1} {
	    # Kommentar ueberlesen
	    gets $menufile line
	    set line [string trim $line]
	    continue
	}

	# auf Submenu pruefen
	if {[string first $submenusymbol  $line] != -1} {
	    create_submenu $menu_id.$submenu_id
	    # Submenu abgearbeitet - naechste Zeile einlesen
	    gets $menufile line 
	    set line [string trim $line]
	    continue
	} 

	# Kommando auswerten und eintragen
	set line "$line "
	set command [string range $line 0 [string first " " $line]]
	set command [string trim  $command]
	set commandname [string range $line [string first " " $line] end]
	set commandname [string trim  $commandname]
	if { [string compare $commandname ""] == 0} {
	    set commandname $command
	}
	
	# falls Schluesselwort 'SEPARATOR' gefunden -> Separator einfuegen 
	if {[string compare $command "SEPARATOR"] ==  0} { 
	    # Seperator einfuegen
	    .frame1.$menu_id.$submenu_id add separator
	} else {
	    ### fuege hier in $menuname die neuen Befehle $line ein
	    if { [hasparam $command] != 0 } {
		.frame1.$menu_id.$submenu_id add command \
		    -command "calldialog $command Transactions $command" \
		    -label "$commandname . . ."
	    } else {
		.frame1.$menu_id.$submenu_id add command \
		    -command "calldialog $command Transactions $command" \
		    -label $commandname \
		    -background #90ee90 \
		    -activebackground green
	    }
	}
	
	gets $menufile line 
	set line [string trim $line]
    }
    return ok
}

######
###### Hauptroutine
######

set menufile  [open "$GENDIR/config.ecu"   r]
set line ""

while { [gets $menufile line] != -1 } {
    set line [string trim $line]

    if {[string compare $line ""] ==  0} { 
      # Leerzeile ueberlesen
      continue 
    }

    if {[string first "#" $line ] != -1} { 
      # Kommentar ueberlesen
      continue 
    }

    # Menueintrag vorbereiten
    if {[string first $menusymbol  $line] != -1} {
	set menuname [string trimleft  $line $menusymbol]
	set menuname [string trimright $menuname "\{"]
	set menuname [string trim $menuname]
	set menu_id  [string tolower $menuname]

	### fuege hier neues toplevel $menuname ein
	insert_new_toplevel $menu_id $menuname
	gets $menufile line 
	set line [string trim $line]

	while {[string first "\}" $line] != 0} {

            if {[string compare $line ""] ==  0} {
              # Leerzeile ueberlesen
              gets $menufile line
	      set line [string trim $line]
              continue
            }

            if {[string first "#" $line ] != -1} {
              # Kommentar ueberlesen
              gets $menufile line
	      set line [string trim $line]
              continue
            }

	    # Submenu ?
	    if {[string first $submenusymbol  $line] != -1} {
		create_submenu $menu_id.m
		# Ende des Submenus - naechste Zeile einlesen
		gets $menufile line 
		set line [string trim $line]
		continue
	    } 

	    # Kommando auswerten und eintragen
	    set line "$line "
	    set command [string range $line 0 [string first " " $line]]
	    set command [string trim  $command]
	    set commandname [string range $line [string first " " $line] end]
	    set commandname [string trim  $commandname]
	    if { [string compare $commandname ""] == 0} {
		set commandname $command
	    }
		
	    # falls Schluesselwort 'SEPARATOR' gefunden -> Separator einfuegen 
	    if {[string compare $command "SEPARATOR"] ==  0} { 
		# Seperator einfuegen
		.frame1.$menu_id.m add separator
	    } else {
		### fuege hier in $menuname die neuen befehle $line ein
		# Sinnvoll waere hier, zuerst zu fragen, ob denn der
		# Befehl wirklich existiert, bevor man nach den
		# Parametern fragt.
		# Nichtexistierende Befehle (Schreibfehler in config.ecu)
		# werden mit 0 Parametern eingetragen und schmeissen
		# eine Fehlermeldung. -ajw
		if { [hasparam $command] != 0 } {
		    .frame1.$menu_id.m add command \
			-command "calldialog $command Transactions $command" \
			-label "$commandname . . ."
		} else {
		    .frame1.$menu_id.m add command \
			-command "calldialog $command Transactions $command" \
			-label $commandname \
			-background #90ee90 \
			-activebackground green
		}
	    }
	    gets $menufile line 
	    set line [string trim $line]
	}
	 pack append .frame1 .frame1.$menu_id {left frame center}  
    }
}

close $menufile


