###############################################################################
#
# changeToClass.tcl
# [ansgar] 
# 
# Prozeduren zur Knotendarstellung Box (Label, doubleBox, ...)
#
###############################################################################

global typeArray

# ------------------------------------------------------------
# Procedure: changeToBox
# ndert Knotendarstellung auf Class (Window)

# style: 0:LABEL,  1: BOX,  2: RBOX,  3: DBOX,  4: FBOX

proc changeToBox { item style attrtext icontext {canv "."} } {
    if {$style == "  4"} {
        return [changeToFrameBox $item $attrtext $icontext $canv]
	}

	if {$canv == "." } {
		global symbolicName
		set canv $symbolicName(graphView)
	}
   
	scan [$canv coords $item] {%f %f} x y
	removeItem $canv $item
	#puts "calling changeToClass"
	return [addBoxWindow $canv $style $x $y {green} $attrtext $icontext ]
}


# ------------------------------------------------------------
# procedure: roundrect
# erzeugt abgerundetes Rechteck im bergebenen Canvas
#
proc roundrect {canvas x1 y1 x2 y2 linewidth} {
    set r 10
    set color darkgreen
    $canvas create line [expr $x1+$r] $y1 [expr $x2-$r] $y1\
        -fill $color -width $linewidth
    $canvas create line [expr $x1+$r] $y2 [expr $x2-$r] $y2\
        -fill $color -width $linewidth
    $canvas create line $x1 [expr $y1+$r] $x1 [expr $y2-$r+1]\
        -fill $color -width $linewidth
    $canvas create line $x2 [expr $y1+$r] $x2 [expr $y2-$r+1]\
        -fill $color -width $linewidth
    set r2 [expr $r*2]
    $canvas create arc $x1 [expr $y1+$r2] [expr $x1+$r2] $y1\
        -extent 90 -start 090 -style arc -outline $color -width $linewidth
    $canvas create arc $x2 [expr $y1+$r2] [expr $x2-$r2] $y1\
        -extent 90 -start 000 -style arc -outline $color -width $linewidth
    $canvas create arc $x1 [expr $y2-$r2] [expr $x1+$r2] $y2\
        -extent 90 -start 180 -style arc -outline $color -width $linewidth
    $canvas create arc $x2 [expr $y2-$r2] [expr $x2-$r2] $y2\
        -extent 90 -start 270 -style arc -outline $color -width $linewidth
}


# ------------------------------------------------------------
# Procedure: addBoxWindow
# wandelt Knoten in Class-Darstellung um
# Parameter: Canvas, Postion (x,y), Farbe,
#            Attributtext (Attribute durch NewLine getrennt),
#            Iconattribute (Dateiname, Position, Orientierung)

proc addBoxWindow { canv style x y col attrtext icontext } {
	# Demo-Programm fr Klassendarstellung
	global typeArray

	# -----------------------------------------
	# [ansgar] set font name
	#
	set cFontName Helvetica
	set cFontSize 12         ;# 11 / 12 / 14 / 17
	set cFontState *         ;# Bold / Medium
	set cFontRegular r       ;# R / O
	#
	set FontClass "-*-$cFontName-$cFontState-$cFontRegular-Normal--$cFontSize-*-*"

	# -----------------------------------------
	# [ansgar] calculate width and height of attribute text
	#
	set tmptext $attrtext
	set textWidth 0
	set linecount 0

	while { [string first "\n" $tmptext] > -1} {
		set ind [string first "\n" $tmptext]
		set currentLine [string range $tmptext 0 [expr $ind - 1]]
		set width [font measure $FontClass $currentLine]
		if { $width > $textWidth } then {
			set textWidth $width
		}
		incr ind
		set tmptext [string range $tmptext $ind end]
		incr linecount
	}

	set textHeight [expr [font metrics $FontClass -linespace] * $linecount]

	# -----------------------------------------

	set boxwidth 1

	# [ansgar] the inset denotes the distance of the text from the canvas
	# borders
	set xinset [expr 5 + $boxwidth]
	set yinset [expr 4 + $boxwidth]

	set SubWindowWidth  [expr $textWidth  + 2 * $xinset]
	set SubWindowHeight [expr $textHeight + 2 * $yinset]

	set bg [$canv cget -background]
	set cfTmpTag [$canv create window $x $y\
		-width $SubWindowWidth -height $SubWindowHeight]

	# build widget .frame0
	frame $canv.$cfTmpTag -width $SubWindowWidth -height\
		$SubWindowHeight -background $bg
	# Window Canvas
	canvas $canv.$cfTmpTag.canvas2\
		-closeenough 3.0 -background $bg\
		-insertofftime 600\
		-height $SubWindowHeight -width $SubWindowWidth
 
	pack append $canv.$cfTmpTag $canv.$cfTmpTag.canvas2\
		{top frame center expand fill} 
	frame $canv.$cfTmpTag.canvas2.f1 -bg $bg
 
	# create class canvas
	createBoxCanvas $canv $canv.$cfTmpTag.canvas2.f1 $style\
		$textHeight $textWidth $boxwidth $xinset $yinset $FontClass\
		$attrtext


	pack $canv.$cfTmpTag.canvas2.f1 -side top -expand yes -fill both
	$canv itemconfigure $cfTmpTag -window $canv.$cfTmpTag
	bind $canv <Destroy> exit

	# show arrows
	global isDirected
	set isDirected($canv.$cfTmpTag.canvas2.f1) {YES}

	# responsible for frame and moves
	global dragItem
	#set dragItem($canv.$cfTmpTag.canvas2) [$canv.$cfTmpTag.canvas2 create \
                                           rectangle 0 0 0 0]
	#$canv.$cfTmpTag.canvas2 itemconfigure $dragItem($canv.$cfTmpTag.canvas2)\
	#  -outline [$canv.$cfTmpTag.canvas2 cget -bg]  

	# bindings  
	global currentNode
	global node_B1_press node_B1_shift_press node_B1_motion node_B1_release
	global node_B3_press node_B3_shift_press node_B3_motion node_B3_release
	set currentNode $cfTmpTag

	# selection
	$node_B1_press $canv.$cfTmpTag.canvas2.f1 $cfTmpTag $currentNode
	$node_B1_press $canv.$cfTmpTag.canvas2.f1.canv3 $cfTmpTag $currentNode
	$node_B1_shift_press $canv.$cfTmpTag.canvas2.f1 $cfTmpTag $currentNode
	$node_B1_shift_press $canv.$cfTmpTag.canvas2.f1.canv3 $cfTmpTag $currentNode

	# Bewegen des Fensters
	# beachte: benutze absolute Screen-Koordinaten %X und %Y
	$node_B1_motion $canv.$cfTmpTag.canvas2.f1
	$node_B1_motion $canv.$cfTmpTag.canvas2.f1.canv3

	# Menu anzeigen
	# Menu wird absolut auf dem Screen plaziert - ist kein
	# ChildWindow des Canvas!
	$node_B3_press $canv.$cfTmpTag.canvas2.f1.canv3 $cfTmpTag
	$node_B3_shift_press $canv.$cfTmpTag.canvas2.f1.canv3 $cfTmpTag
	# Menu entfernen
	$node_B3_release $canv.$cfTmpTag.canvas2.f1.canv3
	$node_B1_release $canv.$cfTmpTag.canvas2.f1.canv3
	# Menupunkt auswhlen
	$node_B3_motion $canv.$cfTmpTag.canvas2.f1.canv3 $cfTmpTag

	# Tag hinzufgen - wird fr Bewegen des Fensters etc. bentigt
	$canv addtag classNode withtag $cfTmpTag
	# Bindings des Canvas durchfhren
	addCanvasBindings $canv

	$canv addtag nodetag withtag $cfTmpTag

	# cfTmpTag fr alle neuen Darstellungsformen setzen - sonst gibt es einen
	# Segmentation Fault bei Anklicken eines Knotens in dieser Darstellung!
	set cfTmpTag "$canv.$cfTmpTag.canvas2 $cfTmpTag"
 
	return $cfTmpTag
}


# ------------------------------------------------------------

proc createBoxCanvas { canv c style textHeight textWidth\
		boxwidth xinset yinset FontClass attrtext} {

	set SubWindowWidth  [expr $textWidth + 2 * $xinset]
	set SubWindowHeight [expr $textHeight + 2 * $yinset]

	#set dashStyle ""

	# Window Canvas
	set bg [$canv cget -background]
	canvas $c.canv3\
		-closeenough 3.0 -background $bg\
		-insertofftime 600\
		-height $SubWindowHeight -width $SubWindowWidth\
		-highlightthickness 0
	set canv3 $c.canv3
	pack $canv3 -side top -expand yes -fill both

	# [ansgar] the linewidth lets boxes grow. If the box starts at (1.1),
	#   it can extend to the "outer" area by one pixel (=> linewidth up to 3)
	set x1 1
	set y1 1
	set x2 [expr $textWidth  + 2 * $xinset - 2]
	set y2 [expr $textHeight + 2 * $yinset - 2]

	# [ansgar] the value 8 should be calculated from font metrics
	$canv3 create text\
		[expr $xinset + $textWidth/2]\
		[expr $yinset + $textHeight/2 + 8]\
		-text $attrtext -width $textWidth\
		-font $FontClass -justify center

	# DBOX?
    if {$style == "  3"} {
		$canv3 create rectangle $x1 $x1 $x2 $y2 -width $boxwidth
		set x [expr $x1+2*$boxwidth]
		$canv3 create line $x $y1 $x $y2 -width $boxwidth
		set x [expr $x2-2*$boxwidth]
		$canv3 create line $x $y1 $x $y2 -width $boxwidth
	}

	# RBOX?
    if {$style == "  2"} {
		roundrect $canv3 $x1 $y1 $x2 $y2 $boxwidth
	}

	# BOX?
    if {$style == "  1"} {
		$canv3 create rectangle $x1 $x1 $x2 $y2 -width $boxwidth
	}

	pack $canv3
	return $canv3
}

return ok
