###############################################################################
#
# changeToFrameBox.tcl
# 
# 
# Prozeduren zur Knotendarstellung Frame Box
#
###############################################################################


# Procedure: changeToFrameBox
# aendert Knotendarstellung auf FrameBox (Window)
proc changeToFrameBox { item attrtext icontext {canv "."} } {
    if {$canv == "." } {
        global symbolicName
        set canv $symbolicName(graphView)
    }
    
   # if {[lsearch [$canv gettags $item] windowNode] != -1} {
   #     return ""
   # } {
      scan [$canv coords $item] {%f %f} x y
      removeItem $canv $item
      return [addFrameBoxWindow $canv $x $y {green} $attrtext $icontext]
   # }
}

# Procedure: addFrameBoxWindow
# wandelt Knoten in Boxdarstellung um
# Parameter: Canvas, Postion (x,y), Farbe,
#            Attributtext (Attribute durch NewLine getrennt),
#            Iconattribute (Dateiname, Position, Orientierung)
proc addFrameBoxWindow { canv x y col attrtext icontext} {

  # Fontdaten - eventuell spaeter konfigurierbar
  set cFontName Courier
  set cFontSize 12         ;# 11 / 12 / 14 / 17
  set cFontState *         ;# Bold / Medium
  set cFontRegular r       ;# R / O
  set FontDefault "-*-$cFontName-$cFontState-$cFontRegular-Normal--$cFontSize-*-*"
  set cMaxChars 30     ;# max. erlaubte Zeilenlaenge - default 30

  # berechnete Werte
  set charcount 1      ;# max. Textzeilenlaenge
  set linecount 1      ;# Zeilenanzahl

  # ueberfluessige Leerzeichen loeschen  
  set attrtext [string trim $attrtext]
  set icontext [string trim $icontext]

  # boxinfo pruefen 
  set boxinfo empty
  set tmpListe [split $boxinfo " "]
  if { [llength $tmpListe] != 3 } then {
    #puts "addFrameBoxWindow: Daten unvollstaendig '$boxinfo'"
    set FontData $FontDefault
    set cCharHeight 14.5 ;# 12-Punkt
    set cCharWidth 7     ;# 12-Punkt
  } else {
    # BoxSize
    set tmpWidth [lindex $tmpListe 0]
    if { $tmpWidth > 0 && $tmpWidth < 100 } then {
      set cMaxChars $tmpWidth ;
      incr cMaxChars
    }
    set cFontName [lindex $tmpListe 1]
    set cFontSize [lindex $tmpListe 2]
    # Konstanten zur Fenstergroessenberechnung
    # empirisch ermittelt - Groessenabhaengig
    switch $cFontSize {
	11        { set cCharHeight 13   ; set cCharWidth 7 ; set cFontWidth 70 }
	12        { set cCharHeight 14.5 ; set cCharWidth 7 ; set cFontWidth 70 }
	14        { set cCharHeight 17   ; set cCharWidth 8 ; set cFontWidth 80 }
	17        { set cCharHeight 20   ; set cCharWidth 9 ; set cFontWidth 120 }
	default   { set cCharHeight 14.5 ; set cCharWidth 7 ; set cFontWidth 70 }
    }
    set FontData "-*-$cFontName-$cFontState-$cFontRegular-*-*-$cFontSize-*-*-*-*-$cFontWidth-*-*"
  }

 
  if { $attrtext == "" } then {
    set attrIsVisible false
  } else {
    set attrIsVisible true
  }

  if { $attrIsVisible } {
    # Text bei CarriageReturn umbrechen, Zeichen und Zeilen zaehlen
    set tmptext $attrtext
    while { [string first "\n" $tmptext] > -1} {
      set ind [string first "\n" $tmptext]
      if { $ind > $charcount } then {
        set charcount $ind
      }
      incr ind
      set tmptext [string range $tmptext $ind end]
      incr linecount
    }
    set ind [string length $tmptext]
    if { $ind > $charcount } then {
      set charcount $ind
    }
    incr charcount
    if { $charcount > $cMaxChars } then {
      set charcount $cMaxChars
    }
  } 

  # Test auf gueltige Icon-Parameter und Image-Format
  set iconIsVisible [GetIconVisibility $icontext]

  if { $iconIsVisible } then {
    set tmpListe [split $icontext " "]
    set iconfilename [lindex $tmpListe 0]

  ### Hack, um zuviel uebergebene Blanks zu uebergehen
    # statt untere beiden Zeilen
    #set iconPosition [string tolower [lindex $tmpListe 1]]
    #set iconOrientation [string tolower [lindex $tmpListe 2]]
    set listlength [llength $tmpListe]
    set ind 1
    set iconPosition [lindex $tmpListe 1]
    while { $iconPosition == "" } {
      if { $ind < $listlength } then { 
        incr ind 
        set iconPosition [lindex $tmpListe $ind]
      } else { 
        set iconPosition none
      }
    }
    if { $ind < $listlength } then { 
      incr ind 
      set iconOrientation [string tolower [lindex $tmpListe $ind]]
    } else { 
      set iconOrientation none
    }
    while { $iconOrientation == "" } {
      if { $ind < $listlength } then { 
        incr ind 
        set iconOrientation [string tolower [lindex $tmpListe $ind]]
      } else { 
        set iconOrientation none
      }
    }
    set iconPosition [string tolower $iconPosition]
### Hack-Ende
  } {
    set iconPosition ""
  }

  if { $iconIsVisible } then {
    # Position auf Gueltigkeit pruefen
    if { [lsearch {top bottom left right} $iconPosition] < 0 } then {
      set iconPosition left    ;# Defaultwert
    }    
    # Orientierung auf Gueltigkeit pruefen
    if { [lsearch {first middle last} $iconOrientation] < 0 } then {
      set iconOrientation first    ;# Defaultwert
    }    
    switch $iconOrientation {
          first   { set iconOrientation nw }
          last    { set iconOrientation se }
          middle  { if { ($iconPosition == "top") || ($iconPosition == "bottom") } then { set iconOrientation n } else { set iconOrientation w} }
          default { set iconOrientation nw }
    }

    global ImageArray
  # Hoehe und Weite der Widgets setzen - Toleranzen als Konstanten definieren ?
    set IconHeight [expr [image height $ImageArray($iconfilename)] + 10]
    set IconWidth [expr [image width  $ImageArray($iconfilename)] + 10]
    if { $attrIsVisible } then {
      if { ($iconPosition == "top") || ($iconPosition == "bottom") } then {
      # Icon oberhalb oder unterhalb Attribute
        set SubWindowHeight [expr $linecount * $cCharHeight + $IconHeight]
        if { [expr $charcount * $cCharWidth] < $IconWidth } then {
          set SubWindowWidth $IconWidth
        } else {
          set SubWindowWidth [expr $charcount * $cCharWidth + 6]
        }
      } else {
      # Icon neben Attribute
        set SubWindowWidth [expr $charcount * $cCharWidth + $IconWidth]
        if { [expr $linecount * $cCharHeight] < $IconHeight } then {
          set SubWindowHeight $IconHeight
        } else {  
          set SubWindowHeight [expr $linecount * $cCharHeight + 6]
        }
      }
    } else {
    # nur Icon
      set SubWindowWidth $IconWidth
      set SubWindowHeight $IconHeight
    }
  } else {
    # nur Attribute
    if { $attrIsVisible } then {
      set SubWindowHeight [expr $linecount * $cCharHeight + 6]
      set SubWindowWidth [expr $charcount * $cCharWidth + 6]
    } else {
    # weder Icon noch Attribute
      set SubWindowHeight 10
      set SubWindowWidth 10
    }
  }
    
  # Fehlerkorrektur der uebergebenen x/y-Koordinaten - Grund des Fehlers?
  set x [expr $x + 10]
  set y [expr $y + 10]
  # Fenster erstellen
  set cfTmpTag [$canv create window $x $y -width $SubWindowWidth -height $SubWindowHeight]

  # Icon erstellen
  if { $iconIsVisible } then {
    catch {set boxicon($cfTmpTag) $ImageArray($iconfilename) ; return 0 } error_code
    if { $error_code != 0 } then { 
      # allerletzte Sicherheitspruefung - hier duerfte eigentlich kein Fehler mehr auftreten !!
      global iconfile
      puts stdout "Warning: unknown image format in file $iconfile - unable to display icon"
      set iconIsVisible false
    }     
  }
  
  # Window Border
  frame $canv.$cfTmpTag \
    -bd {3} -bg {lightgrey} -relief {ridge} \
    -width $SubWindowWidth -height $SubWindowHeight
  
  # Window Canvas
  canvas $canv.$cfTmpTag.canvas2 \
    -background {lightgrey} -closeenough {3.0} \
    -insertbackground {black} -insertofftime {600} \
    -relief {sunken} -selectbackground {black} \
    -selectforeground {gainsboro} \
    -height $SubWindowHeight -width $SubWindowWidth 
 
  pack append $canv.$cfTmpTag $canv.$cfTmpTag.canvas2 {top frame center expand fill} 
  frame $canv.$cfTmpTag.canvas2.f1 -bd 1 -relief groove -height 1

  if { $attrIsVisible } then {
    incr charcount -1     ;# sieht besser aus
    catch { label $canv.$cfTmpTag.canvas2.f1.l1 -text $attrtext -justify left -font $FontData -width $charcount -bd {1} ; return 0 } error_code
    if { $error_code != 0 } then { 
      puts "Warning: font '$FontData' not found"
      label $canv.$cfTmpTag.canvas2.f1.l1 -text $attrtext -justify left -font $FontDefault -width $charcount -bd {1}
    }
 
    switch $iconPosition {
          top    { set attrPosition bottom }
          bottom { set attrPosition top }
          left   { set attrPosition right }
          right  { set attrPosition left }
          default { set attrPosition left }
    }
    pack $canv.$cfTmpTag.canvas2.f1.l1 -side $attrPosition -anchor nw
  }
  
  if { $iconIsVisible } then {
    label $canv.$cfTmpTag.canvas2.f1.l2 -image $boxicon($cfTmpTag) -bd {1}
    pack $canv.$cfTmpTag.canvas2.f1.l2 -anchor $iconOrientation -side $iconPosition
  } 

  #if { $attrIsVisible } then {
  #} else {
  #  if { $iconIsVisible } then {
  #  } else {  
  #    puts "Ouch - no icon and no attribute to show!"
  #  }
  #}

  # hier -side top notwendig, damit Labeltext linksbuendig !
  pack $canv.$cfTmpTag.canvas2.f1 -side top -expand yes -fill both
  $canv itemconfigure $cfTmpTag -window $canv.$cfTmpTag
  bind $canv <Destroy> exit
     
  # Pfeilspitzen anzeigen
  global isDirected
  set isDirected($canv.$cfTmpTag.canvas2) {YES}
    
  # verantwortlich fuer Rahmen und Moven
  global dragItem
  set dragItem($canv.$cfTmpTag.canvas2) [$canv.$cfTmpTag.canvas2 create \
                                           rectangle 0 0 0 0]
  $canv.$cfTmpTag.canvas2 itemconfigure $dragItem($canv.$cfTmpTag.canvas2)\
    -outline [$canv.$cfTmpTag.canvas2 cget -bg]  

# Bindings  
  global currentNode
  global node_B1_press node_B1_shift_press node_B1_motion node_B1_release
  global node_B3_press node_B3_shift_press node_B3_motion node_B3_release
  set currentNode $cfTmpTag

#  Events an Frame binden
  # Selektierung - tut so wie es ist - wer weiss wieso
  $node_B1_press $canv.$cfTmpTag.canvas2.f1 $cfTmpTag $currentNode
  $node_B1_shift_press $canv.$cfTmpTag.canvas2.f1 $cfTmpTag $currentNode
  # Bewegen des Fensters
  # beachte: benutze absolute Screen-Koordinaten %X und %Y
  $node_B1_motion $canv.$cfTmpTag.canvas2.f1
  # Popup-Menu anzeigen
  # Menu wird absolut auf dem Screen plaziert -  ist kein ChildWindow des Canvas!
  $node_B3_press $canv.$cfTmpTag.canvas2.f1 $cfTmpTag
  # Node Info in Statusbar anzeigen
  $node_B3_shift_press $canv.$cfTmpTag.canvas2.f1 $cfTmpTag
  # Menu verstecken
  $node_B3_release $canv.$cfTmpTag.canvas2.f1
  $node_B1_release $canv.$cfTmpTag.canvas2.f1
  # Menupunkt auswaehlen
  $node_B3_motion $canv.$cfTmpTag.canvas2.f1 $cfTmpTag

#  Events an Attributtext binden
  if { $attrIsVisible } then {
      $node_B1_press $canv.$cfTmpTag.canvas2.f1.l1 $cfTmpTag $currentNode
      $node_B1_shift_press $canv.$cfTmpTag.canvas2.f1.l1 $cfTmpTag $currentNode
      $node_B1_motion $canv.$cfTmpTag.canvas2.f1.l1
      $node_B3_press $canv.$cfTmpTag.canvas2.f1.l1 $cfTmpTag
      $node_B3_shift_press $canv.$cfTmpTag.canvas2.f1.l1 $cfTmpTag
      $node_B3_release $canv.$cfTmpTag.canvas2.f1.l1
      $node_B1_release $canv.$cfTmpTag.canvas2.f1.l1
      $node_B3_motion $canv.$cfTmpTag.canvas2.f1.l1 $cfTmpTag
  }
#  Events an Icon binden
  if { $iconIsVisible } then {
      $node_B1_press $canv.$cfTmpTag.canvas2.f1.l2 $cfTmpTag $currentNode
      $node_B1_shift_press $canv.$cfTmpTag.canvas2.f1.l2 $cfTmpTag $currentNode
      $node_B1_motion $canv.$cfTmpTag.canvas2.f1.l2
      $node_B3_press $canv.$cfTmpTag.canvas2.f1.l2 $cfTmpTag
      $node_B3_shift_press $canv.$cfTmpTag.canvas2.f1.l2 $cfTmpTag
      $node_B3_release $canv.$cfTmpTag.canvas2.f1.l2
      $node_B1_release $canv.$cfTmpTag.canvas2.f1.l2
      $node_B3_motion $canv.$cfTmpTag.canvas2.f1.l2 $cfTmpTag
  }
  
  $canv addtag windowNode withtag $cfTmpTag  ;# wird fuer Bewegen des Fensters etc. benoetigt
  addCanvasBindings $canv                    ;# Bindings des Canvas durchfuehren

  $canv addtag nodetag withtag $cfTmpTag

  # cfTmpTag fuer alle neuen Darstellungsformen setzen - sonst gibts Segmentation Fault bei Anklicken eines Knotens in dieser Darstellung !!!
  set cfTmpTag "$canv.$cfTmpTag.canvas2 $cfTmpTag"
 
  return $cfTmpTag
}

# Procedure GetIconVisibility
proc GetIconVisibility {icontext} {
  set iconIsVisible true

  # Icon-Pfad gesetzt?
  global IconPath
  global iconfile
  global ImageArray

  if { $IconPath == "@" } then {
      set iconIsVisible false
  } else {
    set tmpListe [split $icontext " "]
    if { [llength $tmpListe] < 3 } then {
      # zu wenige Parameter
      set iconIsVisible false
    } else {
      # Datei-Existenz pruefen
      set tmpListe [split $icontext " "]
      set iconfilename [lindex $tmpListe 0]

      if {[array size ImageArray] != 0 } then {
        # Array ist nicht leer - Liste aller Elemente erstellen
	set ImageList [array names ImageArray]
        # Image auf Existenz im Array pruefen - Index = -1 falls nicht vorhanden
        set ImageIndex [lsearch -exact $ImageList $iconfilename]
      } else {
	# Array ist leer
        set ImageIndex -1
      }

      if { $ImageIndex == -1 } then {
	# Dateiname nicht in Array - Image laden
        #puts "loading image: $iconfilename"
        set iconfile $IconPath/$iconfilename
        if { [file exists $iconfile] == 0 } then {
          # Datei-Existenz pruefen - fuege Endung XBM hinzu
          set iconfile $IconPath/[lindex $tmpListe 0].xbm
          if { [file exists $iconfile] == 0 } then {
            # Datei-Existenz pruefen - fuege Endung GIF hinzu
            set iconfile $IconPath/[lindex $tmpListe 0].gif
            if { [file exists $iconfile] == 0 } then {
              set iconfile $IconPath/[lindex $tmpListe 0]
              set iconIsVisible false
              # puts "Warning: Iconfile $iconfile not found!"
	    }
	  }
        }
        if { $iconIsVisible } then {
          # Test auf gueltiges Image-Format
	  # bei Gueltigkeit wird Icon in Array gespeichert
          set iconIsVisible [BitmapFormatOK $iconfilename]
	}
      }
    }
  }

  return $iconIsVisible
}


# Procedure BitmapFormatOK
# Test auf gueltiges Image-Format - falls gueltig, wird Image in Array gespeichert
proc BitmapFormatOK {iconfilename} {
  set iconIsVisible true
  global iconfile

  catch {set testicon [image create bitmap -file $iconfile] ; return 0 } error_code
  if { $error_code != 0 } then { 
    catch {set testicon [image create photo -file $iconfile] ; return 0 } error_code
    if { $error_code != 0 } then { 
       puts stdout "Warning: unknown image format in file $iconfile - unable to display icon"
       set iconIsVisible false
    }
  }

  if { $iconIsVisible } then {
    # Image wird in assoziatives Array abgelegt
    global ImageArray
    set ImageArray($iconfilename) $testicon
  }

  return $iconIsVisible
}

return ok
