###############################################################################
#
# window.tcl
# 
# 
# Prozeduren fuer die Erzeugung des Haupt-TCL-Fensters
#
###############################################################################


# Procedure ShowWindow ########################################################################################################
proc ShowWindow. {args} {

  # Window manager configurations
  global tk_version
  wm positionfrom . program
  wm sizefrom . program
  wm geometry . 800x550+50+170
  # Warum nicht die Nutzung eines virtuellen Desktops ermoeglichen?
  # Testweise keine Einschraenkung; fvwm bindet Scroll_Lock (F23) an
  # TogglePage, EdgeScroll 5 5, kein EdgeResistance
# wm maxsize . 1152 900
  wm minsize . 0 0
  wm title . {Graph}

  buildMainCanvas
  addCanvasBindings .frame0.canvas2
# buildNodeMenu
  buildEdgeMenu
  buildMainMenu
  buildToolBar
  buildButtonFrame
  buildStatusBar

  # pack widget .
  pack append . \
    .statusbar {bottom frame center fill} \
    .frame0 {bottom frame center expand fill} \
    .frame2 {bottom frame center fill} \
    .frame1 {top frame center fillx} \
    .toolFrame {left frame center expand fillx}

  # put some little help to the window
#  tixBalloon .balloon -statusbar .statusbar.status1 -state status
#  set f .balloon
#  $f bind .frame0.scrollbar1 -statusmsg "vertical scrollbar"
#  $f bind .frame0.scrollbar3 -statusmsg "horizontal scrollbar"
#  $f bind .frame0.canvas2 -statusmsg " "

  ffSetColorsForWin .
}

proc buildStatusBar {} {
    # StatusBar mit zwei Textfeldern
    set w [frame .statusbar -relief raised -bd 1]
    set status1 \
	[label $w.status1 -font -*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*\
	 -relief sunken -bd 1]
    set status2 \
	[label $w.status2 -font -*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*\
	     -relief sunken -bd 1 -text [clock format [clock seconds] -format "%a %b %d, %Y"] -justify center -anchor n]
#    set status3 \
#	[label $w.status3 -font -*-helvetica-medium-r-normal-*-14-*-*-*-*-*-*-*\
#	     -relief sunken -bd 1 -text [clock format [clock seconds] -format "%H:%M"] -justify center -anchor n]

    tixForm $status1 -padx 3 -pady 3 -left %0 -right %75
    tixForm $status2 -padx 3 -pady 3 -left %75 -right %100
#    tixForm $status2 -padx 3 -pady 3 -left %75 -right %93
#    tixForm $status3 -padx 3 -pady 3 -left %93 -right %100
}

# Procedure createButtonRow
proc createButtonRow { frame text befehl {gr 50} {r 10} {l -10} {gl -50}} {
  # build widget .frame2.frame7.frame14.label4
  label $frame.label \
    -background {gainsboro} \
    -padx {2} \
    -text $text

  # build widget .frame2.frame7.frame14.button1
  createButton $frame.button1 { >> }  $befehl $gr

  # build widget .frame2.frame7.frame14.button2
  createButton $frame.button2  { >  } $befehl $r

  # build widget .frame2.frame7.frame14.button3
  createButton $frame.button3  {  < } $befehl $l

  # build widget .frame2.frame7.frame14.button4
  createButton $frame.button4  { << } $befehl $gl

  # pack widget .frame2.frame7.frame14
  pack append $frame \
    $frame.label {left frame center} \
    $frame.button1 {right frame center} \
    $frame.button2 {right frame center} \
    $frame.button3 {right frame center} \
    $frame.button4 {right frame center} 
}

# Procedure ffSetColorsForWin
proc ffSetColorsForWin {w} {
  ##############
  # Procedure  : setColorsForWin
  # Description: sets the colors for the window "w" and all its child-windows
  # Arguments  : w: window
  # Returns    : none
  # Sideeffects: the author is not familiar to any strange sideeffects 
  #              (shit happens)
  # Author     : Achim Dannecker
  ##############
  
  $w configure -bg #dddddd
  set tmplist [winfo children $w]
  while {[llength $tmplist] != 0} {
    switch [winfo class [lindex $tmplist 0]] {
      Listbox {[lindex $tmplist 0] configure -bg #ebebeb\
		 -highlightthickness 0 \
		 -borderwidth 2 -relief ridge -selectmode browse
      }
      Scrollbar {[lindex $tmplist 0] configure -bg gold \
		   -activebackground #c6c6c6 \
		   -highlightthickness 0 \
		   -borderwidth 1 -relief ridge
      }
      Entry {[lindex $tmplist 0] configure -bg #ebebeb \
	       -highlightthickness 0 -bd 1
      }
      Checkbutton {[lindex $tmplist 0] configure -bg #ebebeb \
		     -activebackground #c6c6c6 \
		     -highlightthickness 0 \
		     -relief raised -bd 3 \
		     -padx 5 -pady 1
	bind [lindex $tmplist 0] <1> {
	  %W configure -relief sunken
	}
	bind [lindex $tmplist 0] <ButtonRelease> {
	  %W configure -relief raised
	}
      }
      Button {[lindex $tmplist 0] configure -bg #ebebeb \
		-activebackground #c6c6c6 \
		-highlightthickness 0 -relief raised -bd 2
      }
      Scale {[lindex $tmplist 0] configure -borderwidth 1 -relief ridge \
        -bg #ebebeb -fg #c6c6c6 -activebackground #c6c6c6 -relief raised -bd 1
      }
      default {[lindex $tmplist 0] configure -bg #ebebeb
      }
    }
    set tmplist [concat $tmplist [winfo children [lindex $tmplist 0]]]
    set tmplist [lreplace $tmplist 0 0]
  }
}


# Procedure clearLayoutMenu
proc clearLayoutMenu {} {
  .frame1.menubutton0.m delete 0 last
}

# Procedure addUserMenus
proc addUserMenus {frame} {
    CSets_menubutton $frame ColorSets
}

# Procedure destroyWindow
proc destroyWindow {} {
  if {[string match "*.topA*"    [info commands]]} {
    destroy .topA
  }
  if {[string match "*.topIcon*"    [info commands]]} {
    destroy .topIcon
  }
}

# Procedure buildButtonFrame
proc buildButtonFrame {} {
  # build widget .frame2
  frame .frame2 \
    -background {gainsboro} \
    -borderwidth {2} \
    -relief {raised}
    
  # build widget .frame2.frame8
  frame .frame2.frame8 \
    -background {gainsboro} \
    -borderwidth {2}

  frame .frame2.frame9
  
  label .frame2.frame9.l1 -text "Scale: "
  label .frame2.frame9.l2 -text "Rotate: "
  
  pack append .frame2.frame9 \
    .frame2.frame9.l1 {top} \
    .frame2.frame9.l2 {top}
    
  # build widget .frame2.frame8.scale16
  scale .frame2.frame8.scale16 \
    -width 10 \
    -background {gainsboro} \
    -command {global aktuCommand; $aktuCommand setZWinkel} \
    -orient {horizontal} \
    -relief {raised} \
    -showvalue {0} \
    -from {-360} \
    -to {360}

  .frame2.frame8.scale16 set {0}

  # build widget .frame2.frame8.scale17
  scale .frame2.frame8.scale17 \
    -width 10 \
    -background {gainsboro} \
    -command {global aktuCommand; $aktuCommand zoomScale} \
    -orient {horizontal} \
    -relief {raised} \
    -showvalue {0} \
    -from {40} \
    -to {220}

  .frame2.frame8.scale17 set {100}

  # pack widget .frame2.frame8
  pack append .frame2.frame8 \
    .frame2.frame8.scale17 {top frame center fillx} \
    .frame2.frame8.scale16 {top frame center fillx} 

  # pack widget .frame2
  pack append .frame2 \
    .frame2.frame9 {left frame s} \
    .frame2.frame8 {bottom frame center expand fillx} 
    
}

# Procedure showInfoBox
proc showInfoBox {} {
    toplevel .topA 

    # Window manager configurations
    global tk_version
    wm positionfrom .topA ""
    wm sizefrom .topA ""
    wm maxsize .topA 1000 1000
    wm minsize .topA 10 10
    addInfoBox {.topA}
}
 
# Procedure: addInfoBox
proc addInfoBox {wInfo} {
    global txtPath
    wm title $wInfo {Important Info: }

    set textFile $txtPath/info.txt
    if {[catch "open $textFile r" textInFile]} {
        puts stderr "$textInFile"
        set infoMessage $textInFile
    } else {
        set infoMessage [read $textInFile]
        close $textInFile
    }

    frame $wInfo.bottom
    button $wInfo.bottom.ok -text " Dismiss " -command "destroy $wInfo"
    pack $wInfo.bottom.ok -side left -expand 1 -padx 30

    frame $wInfo.textview
    scrollbar $wInfo.textview.xscroll -command "$wInfo.textview.text xview" -orient horizontal
    scrollbar $wInfo.textview.yscroll -command "$wInfo.textview.text yview"
    text $wInfo.textview.text -relief sunken -bd 2 -xscroll "$wInfo.textview.xscroll set" -yscroll "$wInfo.textview.yscroll set" -wrap none

    pack $wInfo.textview.xscroll -side bottom -fill x
    pack $wInfo.textview.text -side left -fill y 
    pack $wInfo.textview.yscroll -side right -fill y 
    
    pack $wInfo.bottom -side bottom -padx 10 -pady 10
    pack $wInfo.textview -side left -fill y 

    $wInfo.textview.text insert end $infoMessage

    centerWindow $wInfo
}


# Procedure showAboutBox
proc showAboutBox {} {
  toplevel .topA 
  # Window manager configurations
  wm positionfrom .topA ""
  wm sizefrom .topA ""
  wm maxsize .topA 1000 1000
  wm minsize .topA 10 10
  addAboutBox {.topA}
}

# Procedure addAboutBox
proc addAboutBox {fenster} {
  wm title $fenster {About: }

#  set iconIsVisible true
#  global scriptpath
#  if { $scriptpath == "" } {
#      set iconIsVisible false
#  }
#  if { $iconIsVisible } {
#    set iconfile $scriptpath/eculogo.gif
#
#    # Datei-Existenz pruefen
#    if { [file exists $iconfile] == 0 } {
#      # Datei existiert nicht 
#      set iconIsVisible false
#      puts "Warning: Iconfile $iconfile not found!"
#    } {
#      # Test auf gueltiges Image-Format
#      catch {set testicon [image create bitmap -file $iconfile] ; return 0 } error_code
#      if { $error_code != 0 } then { 
#	  catch {set testicon [image create photo -file $iconfile] ; return 0 } error_code
#        if { $error_code != 0 } then { 
#          puts stdout "Warning: unknown image format in file $iconfile - unable to display icon"
#          set iconIsVisible false
#          set testicon ""
#        }
#      }
#    }
#  }

  # build widget $fenster.message1
  message $fenster.message1 -aspect {1331} -borderwidth {1} -justify {center}\
    -padx {1} -pady {1} -relief {raised} -font -*-Times-Bold-R-Normal--18-*-* -text {
  Rheinisch-Westfaelische Technische Hochschule Aachen
  Lehrstuhl fuer Informatik III
  Ahornstr. 55,  52074 Aachen,  Germany
  http://www-i3.informatik.rwth-aachen.de
  }

  # build widget $fenster.message2
  message $fenster.message2 -aspect {1331} -borderwidth {1} -justify {center}\
    -padx {1} -pady {1} -relief {raised} -font -*-Helvetica-Medium-R-Normal--12-*-* -text {
  Rapid Prototyping with PROGRES

  The PROGRES-Team:  Andreas J. Winter, Manfred Muench

  Read the Guided Prototype Generation manual for details.
  In case of trouble contact progres@i3.informatik.rwth-aachen.de
  }

  # build widget $fenster.message3
  message $fenster.message3 -aspect {1331} -borderwidth {1} -justify {center}\
    -padx {1} -pady {1} -relief {raised} -font -*-Helvetica-Medium-R-Normal--12-*-* -text {
  GNU C++  Copyright  by Free  Software  Foundation, Inc.
  FFGRAPH   Copyright  by Carsten Friedrich (Universitaet Passau)
  TCL/TK   Copyright  by University of California, Australian National 
  University, Sun Microsystems, Inc. and Roger E. Critchlow Jr.
  TIX   Copyright  by Ioi Kim Lam and Expert Interface Technologies
  }

  # build widget $fenster.button4
  button $fenster.button4 -command "destroy $fenster"  -text {OK}

  pack append $fenster     \
    $fenster.message1 {frame center fillx} \
    $fenster.message2 {frame center fillx} \
    $fenster.message3 {frame center fillx} \
    $fenster.button4 {bottom frame center fillx}

  centerWindow $fenster
}

# Procedure: centerWindow
# zentriert Fenster auf dem Screen
proc centerWindow { w } {
   update
   set wx [expr ([winfo screenwidth $w] - [winfo width $w]) / 2]
   set wy [expr ([winfo screenheight $w] - [winfo height $w]) / 2]
   wm geometry $w [winfo width $w]x[winfo height $w]+$wx+$wy 
}



# Procedure ShowMessage
proc showMessage {msg} {

    #destroy window if existing
    catch {destroy .msgbox ; return 0 } error_code
    set w .msgbox
    toplevel $w
    wm title $w "System Message"
    wm geometry $w 350x100+400+400
    set top [frame $w.f -bd 1 -relief raised]
    label $w.f.l -bitmap info
    set mess [message $w.f.msg -width 8c -justify center -text $msg]
    set box [tixButtonBox $w.b -bd 1 -relief raised]

    pack $box -side bottom -fill both
    pack $w.f.l $mess -side left -fill both -padx 5
    pack $top -side top -fill both -expand yes

    $box add ok -text Close -underline 0  -command "destroy $w" -width 6
}

# Procedure: resetScales
proc resetScales {} {
  .frame2.frame8.scale16 set {0}
  .frame2.frame8.scale17 set {100}
  recall resetScales
}

# Procedure: test
proc test {} { 
        puts stdout in_test
    #    toplevel .new
    #    button .new.exit -text EXIT -command "destroy .new"
    #    pack .new.exit
    # jk
    global globvar
    puts stdout ************************
#    puts stdout [array names globvar]
#    puts stdout $globvar(task)
    puts stdout ************************

}

# Procedure showUsageBox
proc showUsageBox {} {
  toplevel .topA 

  # Window manager configurations
  global tk_version
  wm positionfrom .topA ""
  wm sizefrom .topA ""
  wm maxsize .topA 1000 1000
  wm minsize .topA 10 10
  addUsageBox {.topA}
}
 
# Procedure: addUsageBox
proc addUsageBox {wInfo} {
    global txtPath
    wm title $wInfo {Usage: }

    set textFile $txtPath/usage.txt
    if {[catch "open $textFile r" textInFile]} {
        puts stderr "$textInFile"
        set infoMessage $textInFile
    } else {
        set infoMessage [read $textInFile]
        close $textInFile
    }

    frame $wInfo.bottom
    button $wInfo.bottom.ok -text " Dismiss " -command "destroy $wInfo"
    pack $wInfo.bottom.ok -side left -expand 1 -padx 30

    frame $wInfo.textview
    scrollbar $wInfo.textview.xscroll -command "$wInfo.textview.text xview" -orient horizontal
    scrollbar $wInfo.textview.yscroll -command "$wInfo.textview.text yview"
    text $wInfo.textview.text -relief sunken -bd 2 -xscroll "$wInfo.textview.xscroll set" -yscroll "$wInfo.textview.yscroll set" -wrap none

    pack $wInfo.textview.xscroll -side bottom -fill x
    pack $wInfo.textview.text -side left -fill y 
    pack $wInfo.textview.yscroll -side right -fill y 
    
    pack $wInfo.bottom -side bottom -padx 10 -pady 10
    pack $wInfo.textview -side left -fill y 

    $wInfo.textview.text insert end $infoMessage

    centerWindow $wInfo
}

# Procedure: addUserUsage
proc addUserUsage {fenster} {
  wm title $fenster {Graph Usage: }

  # build widget $fenster.button4
  button $fenster.button4 -command "destroy $fenster"  -text {OK}

  # build widget $fenster.message3
  message $fenster.message3 -aspect {1331} -borderwidth {4} -justify {left}\
    -padx {5} -pady {2} -relief {raised} -text {
    Shift-XX, Ctrl-XX, Alt-XX etc := press key Shift, Ctrl, ... while doing XX
    Mn := Press Mouse Button n
    
    Add Edgepoint: Select Edge then Shift-Ctrl-M1 on background

    Move Node: M1 on Node then drag
    Move Edge: M1 on Edgepoint then drag
    Move Edgepoint: Shift-M1 on Edgepoint then drag

    Delete Edgepoint: Select Edge, Shift-Ctrl-M2 near point

    Bring up Node Menu: M3 on Node
    Bring up Edge Menu: M3 on Edge

  }

  # pack widget $fenster
  pack append $fenster     \
    $fenster.message3 {top frame center}     \
    $fenster.button4 {bottom frame center fillx}
}

# Procedure buildMainMenu
proc buildMainMenu {} {
  # build widget .frame1
  frame .frame1 \
    -background {gainsboro} \
    -borderwidth {2} \
    -relief {raised}

  # build widget .frame1.menubutton0
  menubutton .frame1.menubutton0 \
    -background {gainsboro} \
    -menu {.frame1.menubutton0.m} \
    -text {Layout} 

  # build widget .frame1.menubutton0.m
  menu .frame1.menubutton0.m \
    -background {gainsboro} 

  # build widget .frame1.menubutton1
  menubutton .frame1.menubutton1 \
    -background {gainsboro} \
    -menu {.frame1.menubutton1.m} \
    -text {File} 
    
  # build widget .frame1.menubutton1.m
  menu .frame1.menubutton1.m \
    -background {gainsboro} 

#@  .frame1.menubutton1.m add command \
#@    -command {recall modPress n} \
#@    -label {New (M-n)} 
#@  .frame1.menubutton1.m add command \
#@    -command {recall modPress l} \
#@    -label {Load (M-l)} 
#@  .frame1.menubutton1.m add command \
#@    -command {recall modPress m} \
#@    -label {Merge (M-m)} 
#  .frame1.menubutton1.m add command \
#    -command {recall modPress s} \
#    -label {Save (M-s)} 
#  .frame1.menubutton1.m add command \
#    -command {recall genEps} \
#    -label {Save Postscript} 
#  .frame1.menubutton1.m add separator

  # Menu items for text graph
  .frame1.menubutton1.m add command \
    -command {showTextGraph} \
    -label {Save Text} 
  .frame1.menubutton1.m add command \
    -command {readTextGraph} \
    -label {Load Text} 
  # Menu item for executing Tcl scripts
  .frame1.menubutton1.m add separator  
  .frame1.menubutton1.m add command \
    -command {ExecScript} \
    -label {Execute script} 
  .frame1.menubutton1.m add separator

  # finished
  #   .frame1.menubutton1.m add command \
      #     -command {recall modPress w} \
      #     -label {Close (M-w)} 

  .frame1.menubutton1.m add command \
    -command {recall modPress q} \
    -label {Quit (M-q)} 

  # build widget .frame1.menubutton4
  menubutton .frame1.menubutton4 \
    -background {gainsboro} \
    -menu {.frame1.menubutton4.m} \
    -text {Window} 

  # build widget .frame1.menubutton4.m
  menu .frame1.menubutton4.m \
    -background {gainsboro} 
  .frame1.menubutton4.m add command \
    -command {findGraph} \
    -label {Find Graph} 
  .frame1.menubutton4.m add command \
    -command {adjWindow} \
    -label {Adjust Window Size} 
  .frame1.menubutton4.m add command \
    -command {resetScales} \
    -label {Reset Window} 

# zum testen einfach einbinden TEST  
#  .frame1.menubutton4.m add command \
#       -command { test}\
#       -label "JK TEST"

#  mein erstes fenster: schnief
#  .frame1.menubutton4.m add command \
#       -command {Visibilitys}\
#       -label "Visibilities"
  
  # build widget .frame1.menubutton2
  menubutton .frame1.menubutton2 \
    -background {gainsboro} \
    -menu {.frame1.menubutton2.m} \
    -text {Help} 

  # build widget .frame1.menubutton2.m
  menu .frame1.menubutton2.m \
    -background {gainsboro} 

  .frame1.menubutton2.m add command \
    -command {if {![string match "*.topA*" [info commands]]}\
                {showAboutBox} } \
    -label {About} \
    -state {active} 
  .frame1.menubutton2.m add separator

#  .frame1.menubutton2.m add command \
#    -command {if {![string match "*.topA*" [info commands]]}\
#		  {showInfoBox} } \
#    -label {Important Info} \
#    -state {active} 

  .frame1.menubutton2.m add command \
    -command {if {![string match "*.topA*" [info commands]]}\
                {showUsageBox} } \
    -label {Usage} \
    -state {active} 

  # .frame1.menubutton2.m add command \
    -command {if {![string match "*.topA*" [info commands]]}\
                {exec /home/private/students/juergenk/jk/bintools/xanim +Ze /home/private/students/juergenk/jk/bintools/rap.mov } } \
    -label {Online Tutorial} \
    -state {active} 

  # build widget .frame1.menubutton3
  menubutton .frame1.menubutton3 \
    -background {gainsboro} \
    -menu {.frame1.menubutton3.m} \
    -text {Options} 

  # build widget .frame1.menubutton3.m
  menu .frame1.menubutton3.m \
    -background {gainsboro} 

  .frame1.menubutton3.m add command \
    -command {recall modPress i} \
    -label {Inspector (M-i)} 

#@  .frame1.menubutton3.m add checkbutton \
#@    -offvalue 0 \
#@    -onvalue 1 \
#@    -variable iconWinFlag \
#@    -selectcolor {gold} \
#@    -command {if {![string match "*.topIcon*" [info commands]]} { \
#@                                                                    showIconBox
#@    } {
#@      global iconWinFlag
#@      if { $iconWinFlag == 0 } {
#@        wm withdraw .topIcon
#@      } {
#@        wm deiconify .topIcon
#@      }
#@    } } \
#@    -label {Icons} \
#@    -state {active} 

#@  .frame1.menubutton3.m add checkbutton \
#@    -offvalue 0 \
#@    -onvalue 1 \
#@    -variable splineFlag \
#@    -selectcolor {gold} \
#@    -command {recall toggleSplineAll} \
#@    -label {Use Splines} 

#@  .frame1.menubutton3.m add checkbutton \
#@    -offvalue 0 \
#@    -onvalue 1 \
#@    -variable showLabelsFlag \
#@    -selectcolor {gold} \
#@    -command {hideallLabels} \
#@    -label {Show Labels} 
#@  .frame1.menubutton3.m add checkbutton \
#@    -offvalue 0 \
#@    -onvalue 1 \
#@    -variable showNodesFlag \
#@    -selectcolor {gold} \
#@    -command {recall showNodes} \
#@    -label {Show Nodes} 
#@  .frame1.menubutton3.m add checkbutton \
#@    -offvalue 0 \
#@    -onvalue 1 \
#@    -variable showEdgesFlag \
#@    -selectcolor {gold} \
#@    -command {recall showEdges} \
#@    -label {Show Edges} 

  if { [isProto] == 1 } {  # ist schon verfuegbar
  .frame1.menubutton3.m add checkbutton \
    -offvalue 0 \
    -onvalue 1 \
    -selectcolor {gold} \
    -command {toggleConstrCheck} \
    -label {Constraint Checking}

  .frame1.menubutton3.m add checkbutton \
    -offvalue 0 \
    -onvalue 1 \
    -selectcolor {gold} \
    -command {toggleRandomChoice} \
    -label {Random Matches}
  }

  .frame1.menubutton3.m add command \
      -command { TCL_config2TCL } \
      -label "General Settings (M-s)"

  # build widget .frame1.menubutton5
#@  menubutton .frame1.menubutton5 \
#@    -background {gainsboro} \
#@    -menu {.frame1.menubutton5.m} \
#@    -text {Edit} 

  # build widget .frame1.menubutton5.m
#@  menu .frame1.menubutton5.m \
#@    -background {gainsboro} 
#@  .frame1.menubutton5.m add command \
#@    -command {recall modPress x} \
#@    -label {Cut (M-x)}
#@  .frame1.menubutton5.m add command \
#@    -command {recall modPress c} \
#@    -label {Copy (M-c)}
#@  .frame1.menubutton5.m add command \
#@    -command {recall modPress v} \
#@    -label {Paste (M-v)}
#@  .frame1.menubutton5.m add command \
#@    -command {recall clear} \
#@    -label {Clear}
#@  .frame1.menubutton5.m add separator
#@  .frame1.menubutton5.m add command \
#@    -command {recall modPress a} \
#@    -label {Select all (M-a)}
#@  .frame1.menubutton5.m add command \
#@    -command {recall revert} \
#@    -label {Revert} 
#@  .frame1.menubutton5.m add command \
#@    -command {recall eliminateParallelEdges} \
#@    -label {Eliminate Parallel Edges} 
 
  # pack widget .frame1
  pack append .frame1 \
      .frame1.menubutton1 {left frame center} \
      .frame1.menubutton4 {left frame center} \
      .frame1.menubutton2 {right frame center} \
      .frame1.menubutton0 {left frame center} \
      .frame1.menubutton3 {left frame center} 
#@      .frame1.menubutton5 {left frame center} \

# Einbinden des Transaktionsmenus
  global scriptpath
  source "$scriptpath/gen_menu.tcl"

  addUserMenus .frame1

#   global tk_version
#   if {$tk_version >= 3.0} {
#     tk_menuBar .frame1 .frame1.menubutton0 .frame1.menubutton1 \
#       .frame1.menubutton4 .frame1.menubutton2 .frame1.menubutton3 \
#       .frame1.menubutton5
#   } {
#     tk_menus . .frame1.menubutton0 .frame1.menubutton1 \
#       .frame1.menubutton4 .frame1.menubutton2 .frame1.menubutton3 \
#       .frame1.menubutton5
#   }
}


# Procedure: addLayoutMenu
proc addLayoutMenu { text} {
  .frame1.menubutton0.m add command \
    -command "recall layout $text" \
    -label $text
}

# Procedure: changeWindowSize
proc changeWindowSize { parent id xoff yoff } {
   set fenster "$parent.$id"
   set neuW [expr [winfo w $fenster] + $xoff]   
   set neuH [expr [winfo h $fenster] + $yoff]
   $parent itemconfigure $id -width $neuW -height $neuH
   $fenster configure -width $neuW -height $neuH
   set neuW [winfo w $fenster]   
   set neuH [winfo h $fenster]
}

# Procedure setBusy
proc setBusy { } {
  .frame0.canvas2 configure -cursor watch
}


# Procedure unsetBusy
proc unsetBusy { } {
  .frame0.canvas2 configure -cursor left_ptr
}

# Procedure setWorkingMode
proc setWorkingMode {mode theEntry} {
  global workingMode
  $theEntry configure -state normal
  $theEntry delete 0 end
  $theEntry insert end $mode
  $theEntry configure -state disabled
  set workingMode $mode
}

# Procedure buildToolBar
proc buildToolBar {} {
  frame .toolFrame
}

# Procedure createButton
proc createButton {b name args} {
  set body1 { global aktuCommand;
    $aktuCommand }
  
  set nase "$body1 $args"
  
  button $b -text $name -command $nase -padx 0 -pady 0
  
  global mousepressed
  bind $b <1> {
    %W configure -relief sunken
    set mousepressed 1
    while {$mousepressed != "0"} {
      %W invoke
      after 1
      update
    }
    break;
  }
  bind $b <ButtonRelease-1> {
    %W configure -relief raised 
    if {$mousepressed == "1"} {
      set mousepressed 0
    }
    break;
  }
}

return ok
