###############################################################################
#
# setNodeType.tcl
# 
# 
# Erzeugung der Notebook-Seite fuer NodeType
#
#
###############################################################################

proc createPage_NodeType { win } {
    global typeList
    global actualType
    global win_path_for_tree
    global win_path_for_types
    global typeArray
    global h1
    global nodeDisplay
    global srcEdgeType
    global trgEdgeType
    global selectedType

    #--------------------------------------------------------------
    # Fenster des Tabs nodeType ermitteln
    #--------------------------------------------------------------
    set w [$win.nb subwidget ntype]

    #--------------------------------------------------------------
    # Container-Rahmen
    #--------------------------------------------------------------
    frame $w.top -border 1
    pack $w.top -side top -fill both -expand yes

    #--------------------------------------------------------------
    # Rahmen fuer die linke Seite
    #--------------------------------------------------------------
    set left [frame $w.top.left]
    $left config -border 1 -relief ridge
    pack $left -side left  -expand no -fill both -padx 1

    #--------------------------------------------------------------
    # Rahmen fuer die rechte Seite
    #--------------------------------------------------------------
    set right [frame $w.top.right]
    $right config -border 1 -relief ridge
    pack $right -side left  -expand yes -fill both -padx 1

    set win_path_for_types $right

}

proc create_node_display {} {

    global win_path_for_types
    global typeArray
    global actualType

    set right $win_path_for_types
    if { [winfo exists $right.frdisp] == 1  } { destroy $right.frdisp }
    
    #--------------------------------------------------------------
    # Rahmen fuer die rechte Seite
    #--------------------------------------------------------------
    frame $right.frdisp -relief raised -bd 1
    pack $right.frdisp -side top -fill both -expand yes -pady 4

    set right $right.frdisp

    #--------------------------------------------------------------
    # Ueberschrift
    #--------------------------------------------------------------
    label $right.label1 -text "NODE Display Specific Options" -fg blue
    pack $right.label1 -side top -padx 2

    #--------------------------------------------------------------
    # zwingende Existenz von Kanten fuer Sichtbarkeit vom Knoten
    #--------------------------------------------------------------
    tixOptionMenu $right.forceEdge -label "Force Existing Edges:" \
        -variable {varForceEdge} \
	-options { label.width  27   label.anchor w   menubutton.width 9 } \
	-state disabled

    pack $right.forceEdge -side top -anchor w -padx 4
    $right.forceEdge add command NO  -label NO
    $right.forceEdge add command YES  -label YES
    tixSetSilent $right.forceEdge NO

    #--------------------------------------------------------------
    # EdgeType Comboboxes
    #--------------------------------------------------------------
    create_etype_combo_box2 $right.srcEdge "Type of Source Edge:" "srcEdgeType:set" srcEdgeType $typeArray($actualType,SRCEDGETYPE)
    create_etype_combo_box2 $right.trgEdge "Type of Target Edge:" "trgEdgeType:set" trgEdgeType $typeArray($actualType,TRGEDGETYPE)

}

proc toggleForceEdge {} {
    #--------------------------------------------------------------
    # Speichern der Einstellung fuer ForceEdge
    # muss noch angebunden werden
    #--------------------------------------------------------------
    global typeArray
    global actualType
    global varForceEdge
#    set typeArray($actualType,FORCEEDGE) $varForceEdge
}

#proc create_edge_display {nodeDisplay} 
proc create_edge_display {} {

    global win_path_for_types
    global actualType
    global typeArray

    set right $win_path_for_types
    if { [winfo exists $right.frdisp] == 1  } { destroy $right.frdisp }
    
    #--------------------------------------------------------------
    # Rahmen fuer die rechte Seite
    #--------------------------------------------------------------
    frame $right.frdisp -relief raised -bd 1
    pack $right.frdisp -side top -fill both -expand yes -pady 4

    set right $right.frdisp

    #--------------------------------------------------------------
    # Ueberschrift
    #--------------------------------------------------------------
#@    label $right.label1 -text "$nodeDisplay Display Specific Options" -fg blue
#@    pack $right.label1 -side top -padx 2

    #--------------------------------------------------------------
    # EdgeType Comboboxes
    #--------------------------------------------------------------
    create_etype_combo_box2 $right.srcEdge "Type of Source Edge:" "srcEdgeType:set" srcEdgeType $typeArray($actualType,SRCEDGETYPE)
    create_etype_combo_box2 $right.trgEdge "Type of Target Edge:" "trgEdgeType:set" trgEdgeType $typeArray($actualType,TRGEDGETYPE)


    #--------------------------------------------------------------
    # Traversierung der Kanten
    #--------------------------------------------------------------
    tixOptionMenu $right.srcTraverse -label "Traverse Source Edge:" \
	-options { label.width  20   label.anchor w   menubutton.width 16 }

    set tmp ""
    foreach opt { FORWARD BACKWARDS } {
	if { [string compare $tmp $opt] != 0 } { $right.srcTraverse add command $opt -label $opt }
    }
    pack $right.srcTraverse -side top -anchor w -padx 4
    $right.srcTraverse config -command srcTraverse:set
    tixSetSilent $right.srcTraverse $typeArray($actualType,SRCTRAVFORWARD)

    tixOptionMenu $right.trgTraverse -label "Traverse Target Edge:" \
	-options { label.width  20   label.anchor w   menubutton.width 16 }

    set tmp ""
    foreach opt { FORWARD BACKWARDS } {
	if { [string compare $tmp $opt] != 0 } { $right.trgTraverse add command $opt -label $opt }
    }
    pack $right.trgTraverse -side top -anchor w -padx 4
    $right.trgTraverse config -command trgTraverse:set
    tixSetSilent $right.trgTraverse $typeArray($actualType,TRGTRAVFORWARD)

}

proc srcTraverse:set {sel} {
    global actualType
    global typeArray

    set typeArray($actualType,SRCTRAVFORWARD) $sel
}

proc trgTraverse:set {sel} {
    global actualType
    global typeArray

    set typeArray($actualType,TRGTRAVFORWARD) $sel
}

proc create_etype_combo_box2 { w wLabel cmd var init} {
    #--------------------------------------------------------------
    # Combobox fuer EdgeType
    #--------------------------------------------------------------
    global actualType
    global win_path_for_types
    global typeArray
    global h1
    global c1
    global $var

    #set w $win_path_for_types

    if { [winfo exists $w] == 1  } { destroy $w }
    
    tixComboBox $w -label $wLabel -dropdown true -editable false  \
	-command $cmd  -variable $var \
	-options { listbox.height 6  entry.width 17  label.width 20   label.anchor w }

    global edgeList
    set edgeList [lsort $edgeList]

    foreach type  $edgeList  { 
	$w insert end $type
    }
    $w insert end NONE

    if { [string compare  "" $init] != 0 } {
	tixSetSilent $w $init
	if { [string compare $h1 "not_yet"] != 0 } {
	    # in var steht der Variablenname ! - Nehme daraus den Inhalt als Parameter...
	    $cmd  [expr $$var]
	}
    } else { 
	$h1 del all
    }
    
#@    pack $w -side top -anchor w -padx 4
}

proc srcEdgeType:set {sel} {
    global actualType
    global typeArray

    set typeArray($actualType,SRCEDGETYPE) $sel
#    puts "srcEdgeType:set $sel"
}

proc trgEdgeType:set {sel} {
    global actualType
    global typeArray

    set typeArray($actualType,TRGEDGETYPE) $sel
}


return ok