
#################################################################################
#
# Procs for nodes 
#
#################################################################################

proc CreateNode {path text tag {bitmap ""}} {
    #
    # Create an new node put it an the canvas and add it to the logical graph
    #
    frame $path.c.node$tag -bd 2 -relief groove -background white
    NodeBindings $path $tag
    
    # Display bitmap
    if {$bitmap!=""} {
	if {[file exists $bitmap]==1} {
	    label $path.c.node$tag.b -bitmap @$bitmap -background white
	    pack $path.c.node$tag.b
	    NodeBindings $path $tag .b
	}
    } 
    # Display message
    message $path.c.node$tag.m -text $text -width 4i -background white
    pack $path.c.node$tag.m -side left -fill x
    NodeBindings $path $tag .m
    # Show node and add to logical graph
    $path.c graph add [$path.c create window 2c 2c -window $path.c.node$tag -tags "node $tag"]
    #	$path.c create text 1c 1c -text $text -fill red -tags "node $tag"
}

proc ChangeNode {path text tag {bitmap ""}} {
    #
    # Update a node on the canvas and the logical graph
    #
    if {$bitmap!=""} {
	if {[file exists $bitmap]==1} {
	    $path.c.node$tag.b configure -bitmap @$bitmap
	}
    } 
    $path.c.node$tag.m configure -text $text
    
}


proc DeleteNode {path tag} {
    #
    # Delete a node logically and remove nodes (which means destroying the widget)
    # and edges from the canvas
    #
    
    global taglist
    
    if {[$path.c find withtag $tag]==""} return
    
    # if node is selected we must remove it from the taglist
    
    set index [lsearch $taglist $tag]
    if {$index>-1} {
	set taglist [lreplace $taglist $index $index]
    }
    
    # store all edge-ids which have the source or target $tag 
    set outedges [$path.c find withtag from$tag]
    set inedges [$path.c find withtag to$tag]
    
    # delete node
    $path.c graph remove $tag
    update
    $path.c delete $tag
    destroy $path.c.node$tag
    
    # and kill edges
    foreach del $outedges {
	update
	$path.c delete $del
    }
    foreach del $inedges {
	update
	$path.c delete $del
    }
}


proc GetNodeId {path tag} {
    return [$path.c find withtag $tag]
}

proc ExtractTag {name} {
    #
    # needed for node bindings: extract the tag from the path of the node 
    # path must be of form .prefix.c.node$tag[.l|m]
    #
    global path
    
    set kill $path.c.node
    set rest [string range $name [string length $kill] [string length $name]]
    set f [string first . $rest]
    if {$f>0} {
	set rest [string range $rest 0 [expr $f-1]]
    }
    return $rest
}

proc NodeSetColor {node list foreground background} {
    # node gives full path to node, list contains a string with the name of the Selection
    # foreground, background gives colors.
    
    global taglist taglistShift taglistCtrl

    if {[string compare $foreground "Black"]==0} {
	# deselect node.
	set tag [ExtractTag $node]
	
	if {[string compare $list "Selection"]==0} {
	    if {([lsearch $taglistShift $tag]>-1) || ([lsearch $taglistCtrl $tag]>-1)} {
		return 
	    }
	}
	
	if {[string compare $list "SelectionShift"]==0} {
	    if {([lsearch $taglist $tag]>-1) || ([lsearch $taglistCtrl $tag]>-1)} {
		return 
	    }
	}
	
	if {[string compare $list "SelectionCtrl"]==0} {
	    if {([lsearch $taglist $tag]>-1) || ([lsearch $taglistShift $tag]>-1)} {
		return 
	    }
	}
    }
    
    $node configure -background $background
    foreach child [winfo children $node] {
	$child configure -background $background 
	$child configure -foreground $foreground
    }
}
    
proc UpdateSelectionVariable {variable selection} {
    # updates selection variable
    global globvar
    
    set templist [list]
    foreach sel $selection {
	set tag [string range $sel 1 end]
	lappend templist $tag
    }
    set globvar($variable) $templist
    unset templist
    
    variabletool setselection $variable $globvar($variable)
}

proc NodeSetSelection {value msgs} {
    global taglist path
    # sets the selection to value 
    
    # delete old selection
    foreach oldTag $taglist {
	# set normal colors.
	NodeSetColor $path.c.node$oldTag Selection Black White
    }
    
    # delete old taglist
    set taglist [list]
    
    set selList $value
    foreach sel $selList {
	# set selection colors
	if {[winfo exists $path.c.nodeN$sel]==1} {
	    lappend taglist N$sel
	    NodeSetColor $path.c.nodeN$sel Selection White Black
	} else {
	    if {[string compare $msgs "true"] == 0} {
		ErrorWin "Node $sel not existent."
	    }
	}
    }
}

proc NodeSetSelectionShift {value msgs} {
    global taglistShift path 
    # sets the selection to value 
    
    # delete old selection
    foreach oldTag $taglistShift {
	# set normal colors.
	NodeSetColor $path.c.node$oldTag SelectionShift Black White
    }
    
    # delete old taglist
    set taglistShift [list]
    
    set selList $value
    foreach sel $selList {
	# set selection colors
	if {[winfo exists $path.c.nodeN$sel]==1} {
	    lappend taglistShift N$sel
	    NodeSetColor $path.c.nodeN$sel SelectionShift White Black
	} else {
	    if {[string compare $msgs "true"] == 0} {
		ErrorWin "Node $sel not existent."
	    }
	}
    }
}

proc NodeSetSelectionCtrl {value msgs} {
    global taglistCtrl path 
    # sets the selection to value 
    
    # delete old selection
    foreach oldTag $taglistCtrl {
	# set normal colors.
	NodeSetColor $path.c.node$oldTag SelectionCtrl Black White
    }
    
    # delete old taglist
    set taglistCtrl [list]
    
    set selList $value
    foreach sel $selList {
	# set selection colors
	if {[winfo exists $path.c.nodeN$sel]==1} {
	    lappend taglistCtrl N$sel
	    NodeSetColor $path.c.nodeN$sel SelectionCtrl White Black
	} else {
	    if {[string compare $msgs "true"] == 0} {
		ErrorWin "Node $sel not existent."
	    }
	}
    }
}


proc NodeBindings {path tag {window ""}} {
    #
    # Bindings for the nodes
    #
    global attr attrcount currentTag taglist taglistShift taglistCtrl motion currX currY motionDist
    
    bind $path.c.node$tag$window <Button-1> {
	set motion ""
	set currX %X
	set currY %Y
        set oldX %x
        set oldY %y
        set currentTag [ExtractTag %W]
    }
    
   bind $path.c.node$tag$window <B1-Motion> {
       if {([expr abs([expr $currX - %X])] > $motionDist) || ([expr abs([expr $currY - %Y])] > $motionDist)} {
	   set motion "motion"
	   set currX %X
	   set currY %Y
       }

       if {[string compare $motion "motion"] == 0} {
	  set curX %x
	  set curY %y
        
	  set dx [expr $curX-$oldX]
	  set dy [expr $curY-$oldY]

	  $path.c move $currentTag $dx $dy
          #behandele alle kanten des/der vorgaenger
          foreach i [$path.c find withtag from$currentTag] {
             set koords [$path.c coords $i]
             $path.c coords $i  [expr [lindex $koords 0] + $dx ] \
                                    [expr [lindex $koords 1] + $dy ] \
                                    [lindex $koords 2] \
                                    [lindex $koords 3] 
          }
          #behandele alle kanten des/der nachfolgers
          foreach i [$path.c find withtag to$currentTag] {
             set koords [$path.c coords $i]
             $path.c coords $i  [lindex $koords 0] \
                                    [lindex $koords 1] \
                                    [expr [lindex $koords 2] + $dx ] \
                                    [expr [lindex $koords 3] + $dy ] 
                              
          }
       }
   }

   bind $path.c.node$tag$window <ButtonRelease-1> {

       if {[string compare $motion "motion"]!=0} {
	   # no motion between button down and up 
	   # start new selection, reset old selection.
	   
	   foreach oldTag $taglist {
	       # set normal colors.
	       NodeSetColor $path.c.node$oldTag Selection Black White
	   }
	   # delete old taglist
	   set taglist [list]
	   
	   #build new selection
	   set tag [ExtractTag %W]
	   lappend taglist $tag
	   NodeSetColor $path.c.node$tag Selection White Black
	   
	   #update variable
	   UpdateSelectionVariable Selection $taglist
       }
   }

   bind $path.c.node$tag$window <Button-2> {
       set motion ""
       set oldX %x
       set oldY %y
       set currentTag [ExtractTag %W]
   }

   bind $path.c.node$tag$window <B2-Motion> {
       if {[string compare $motion "motion"] != 0} {
	   if {([expr abs([expr $oldX - %X])] > $motionDist) || ([expr abs([expr $oldY - %Y])] > $motionDist)} {
	       set motion "motion"
	   }
       }

   }
   
   bind $path.c.node$tag$window <ButtonRelease-2> {

       if {[string compare $motion "motion"]!=0} {
	   if {[string compare $motion "double"]!=0} {
	       # no motion between button down and up
	       
	       # continue selection, switch selection mode 
	       # (selected <--> unselected) of node.
	       
	       set tag [ExtractTag %W]
	       set index [lsearch $taglist $tag]
	       
	       if {$index == -1} {
		   # node not selected.
		   lappend taglist $tag
		   NodeSetColor $path.c.node$tag Selection White Black
		   UpdateSelectionVariable Selection $taglist
	       } else {
		   # node selected.
		   set taglist [lreplace $taglist $index $index]
		   NodeSetColor $path.c.node$tag Selection Black White
		   UpdateSelectionVariable Selection $taglist
	       }
	   }
       }
#       if {[string compare $motion "motion"] == 0} {
#	   #position in Knotenattributen setzen
#	   set koords [$path.c coords $currentTag]
#	   browser setposition [string trim $currentTag N] [lindex $koords 0] [ lindex $koords 1]
#	   browser redisplay
#       }
       set motion "selection"
   }

   bind $path.c.node$tag$window <Double-Button-2> {

       # Doubleclick, show attributes of node 

       if {[string compare $motion "selection"]!=0} {
	   # Selection was extended by single click to node
	   # continue selection, switch selection mode 
	   # (selected <--> unselected) of node.
	   
	   set tag [ExtractTag %W]
	   set index [lsearch $taglist $tag]
	   
	   if {$index == -1} {
	       # node not selected.
	       lappend taglist $tag
	       NodeSetColor $path.c.node$tag Selection White Black
	       UpdateSelectionVariable Selection $taglist
	   } else {
	       # node selected.
	       set taglist [lreplace $taglist $index $index]
	       NodeSetColor $path.c.node$tag Selection Black White
	       UpdateSelectionVariable Selection $taglist
	   }
	   set motion "double"
       }
       
       browser getattribute [string trimleft $tag N]
       ShowAttributes $attrcount
       incr attrcount
   }
   
   bind $path.c.node$tag$window <Button-3> {
       set motion ""
       set currX %X
       set currY %Y
   }

   bind $path.c.node$tag$window <B3-Motion> {
       if {([expr abs([expr $currX - %X])] > $motionDist) || ([expr abs([expr $currY - %Y])] > $motionDist)} {
	   set motion "motion"
	   set currX %X
	   set currY %Y
       }
   }

   bind $path.c.node$tag$window <ButtonRelease-3> {
       
#       if {[string compare $motion "motion"]!=0} {
#	   # no motion between button down and up
#	   
#	   # continue selection, switch selection mode 
#	   # (selected <--> unselected) of node.
#	   
#	   set tag [ExtractTag %W]
#	   set index [lsearch $taglist $tag]
#	   
#	   if {$index == -1} {
#	       # node not selected.
#	       lappend taglist $tag
#	       NodeSetColor $path.c.node$tag Selection White Black
#	       UpdateSelectionVariable Selection $taglist
#	   } else {
#	       # node selected.
#	       set taglist [lreplace $taglist $index $index]
#	       NodeSetColor $path.c.node$tag Selection Black White
#	       UpdateSelectionVariable Selection $taglist
#	   }
#       }
       set motion "selection"
       
       # Start transaction 
       automaticcall
   }

   bind $path.c <ButtonRelease-3> {
       # Start transaction 
       automaticcall
   }

   ##### Selection with Shift

   bind $path.c.node$tag$window <Shift-Button-1> {
       set motion ""
       set currX %X
       set currY %Y
   }

   bind $path.c.node$tag$window <Shift-B1-Motion> {
       if {([expr abs([expr $currX - %X])] > $motionDist) || ([expr abs([expr $currY - %Y])] > $motionDist)} {
	   set motion "motion"
	   set currX %X
	   set currY %Y
       }
   }

   bind $path.c.node$tag$window <Shift-ButtonRelease-1> {

       if {[string compare $motion "motion"]!=0} {
	   # no motion between button down and up 
	   # start new selection, reset old selection.
	   
	   foreach oldTag $taglistShift {
	       # set normal colors.
	       NodeSetColor $path.c.node$oldTag SelectionShift Black White
	   }
	   # delete old taglistShift
	   set taglistShift [list]
	   
	   #build new selection
	   set tag [ExtractTag %W]
	   lappend taglistShift $tag
	   NodeSetColor $path.c.node$tag SelectionShift White Black
	   
	   #update variable
	   UpdateSelectionVariable SelectionShift $taglistShift
       }
   }

   bind $path.c.node$tag$window <Shift-Button-2> {
       set motion ""
       set currX %X
       set currY %Y
   }

   bind $path.c.node$tag$window <Shift-B2-Motion> {
       if {([expr abs([expr $currX - %X])] > $motionDist) || ([expr abs([expr $currY - %Y])] > $motionDist)} {
	   set motion "motion"
	   set currX %X
	   set currY %Y
       }
   }

   bind $path.c.node$tag$window <Shift-ButtonRelease-2> {
       
       if {[string compare $motion "motion"]!=0} {
	   # no motion between button down and up
	   
	   # continue selection, switch selection mode 
	   # (selected <--> unselected) of node.
	   
	   set tag [ExtractTag %W]
	   set index [lsearch $taglistShift $tag]
	   
	   if {$index == -1} {
	       # node not selected.
	       lappend taglistShift $tag
	       NodeSetColor $path.c.node$tag SelectionShift White Black
	       UpdateSelectionVariable SelectionShift $taglistShift
	   } else {
	       # node selected.
	       set taglistShift [lreplace $taglistShift $index $index]
	       NodeSetColor $path.c.node$tag SelectionShift Black White
	       UpdateSelectionVariable SelectionShift $taglistShift
	   }
       }
       set motion "selection"
   }

   bind $path.c.node$tag$window <Shift-Button-3> {
       set motion ""
       set currX %X
       set currY %Y
   }

   bind $path.c.node$tag$window <Shift-B3-Motion> {
       if {([expr abs([expr $currX - %X])] > $motionDist) || ([expr abs([expr $currY - %Y])] > $motionDist)} {
	   set motion "motion"
	   set currX %X
	   set currY %Y
       }
   }

   bind $path.c.node$tag$window <Shift-ButtonRelease-3> {

#      if {[string compare $motion "motion"]!=0} {
#	   # no motion between button down and up
#	   
#	   # continue selection, switch selection mode 
#	   # (selected <--> unselected) of node.
#	   
#	   set tag [ExtractTag %W]
#	   set index [lsearch $taglistShift $tag]
#	   
#	   if {$index == -1} {
#	       # node not selected.
#	       lappend taglistShift $tag
#	       NodeSetColor $path.c.node$tag SelectionShift White Black
#	       UpdateSelectionVariable SelectionShift $taglistShift
#	   } else {
#	       # node selected.
#	       set taglistShift [lreplace $taglistShift $index $index]
#	       NodeSetColor $path.c.node$tag SelectionShift Black White
#	       UpdateSelectionVariable SelectionShift $taglistShift
#	   }
#       }
       set motion "selection"
       
       # Start transaction 
       automaticcall
   }
   
   bind $path.c <Shift-ButtonRelease-3> {
       # Start transaction 
       automaticcall
   }

   #### Selection with Control

   bind $path.c.node$tag$window <Control-Button-1> {
       set motion ""
       set currX %X
       set currY %Y
   }

   bind $path.c.node$tag$window <Control-B1-Motion> {
       if {([expr abs([expr $currX - %X])] > $motionDist) || ([expr abs([expr $currY - %Y])] > $motionDist)} {
	   set motion "motion"
	   set currX %X
	   set currY %Y
       }
   }

   bind $path.c.node$tag$window <Control-ButtonRelease-1> {

       if {[string compare $motion "motion"]!=0} {
	   # no motion between button down and up 
	   # start new selection, reset old selection.
	   
	   foreach oldTag $taglistCtrl {
	       # set normal colors.
	       NodeSetColor $path.c.node$oldTag SelectionCtrl Black White
	   }
	   # delete old taglistCtrl
	   set taglistCtrl [list]
	   
	   #build new selection
	   set tag [ExtractTag %W]
	   lappend taglistCtrl $tag
	   NodeSetColor $path.c.node$tag SelectionCtrl White Black
	   
	   #update variable
	   UpdateSelectionVariable SelectionCtrl $taglistCtrl
       }
   }

   bind $path.c.node$tag$window <Control-Button-2> {
       set motion ""
       set currX %X
       set currY %Y
   }

   bind $path.c.node$tag$window <Control-B2-Motion> {
       if {([expr abs([expr $currX - %X])] > $motionDist) || ([expr abs([expr $currY - %Y])] > $motionDist)} {
	   set motion "motion"
	   set currX %X
	   set currY %Y
       }
   }

   bind $path.c.node$tag$window <Control-ButtonRelease-2> {
       
       if {[string compare $motion "motion"]!=0} {
	   # no motion between button down and up
	   
	   # continue selection, switch selection mode 
	   # (selected <--> unselected) of node.
	   
	   set tag [ExtractTag %W]
	   set index [lsearch $taglistCtrl $tag]
	   
	   if {$index == -1} {
	       # node not selected.
	       lappend taglistCtrl $tag
	       NodeSetColor $path.c.node$tag SelectionCtrl White Black
	       UpdateSelectionVariable SelectionCtrl $taglistCtrl
	   } else {
	       # node selected.
	       set taglistCtrl [lreplace $taglistCtrl $index $index]
	       NodeSetColor $path.c.node$tag SelectionCtrl Black White
	       UpdateSelectionVariable SelectionCtrl $taglistCtrl
	   }
       }
       set motion "selection"
   }

   bind $path.c.node$tag$window <Control-Button-3> {
       set motion ""
       set currX %X
       set currY %Y
   }

   bind $path.c.node$tag$window <Control-B3-Motion> {
       if {([expr abs([expr $currX - %X])] > $motionDist) || ([expr abs([expr $currY - %Y])] > $motionDist)} {
	   set motion "motion"
	   set currX %X
	   set currY %Y
       }
   }

   bind $path.c.node$tag$window <Control-ButtonRelease-3> {
       
#       if {[string compare $motion "motion"]!=0} {
#	   # no motion between button down and up
#	   
#	   # continue selection, switch selection mode 
#	   # (selected <--> unselected) of node.
#	   
#	   set tag [ExtractTag %W]
#	   set index [lsearch $taglistCtrl $tag]
#	   
#	   if {$index == -1} {
#	       # node not selected.
#	       lappend taglistCtrl $tag
#	       NodeSetColor $path.c.node$tag SelectionCtrl White Black
#	       UpdateSelectionVariable SelectionCtrl $taglistCtrl
#	   } else {
#	       # node selected.
#	       set taglistCtrl [lreplace $taglistCtrl $index $index]
#	       NodeSetColor $path.c.node$tag SelectionCtrl Black White
#	       UpdateSelectionVariable SelectionCtrl $taglistCtrl
#	   }
#       }
       set motion "selection"
       
       # Start transaction 
       automaticcall
   }

   bind $path.c <Control-ButtonRelease-3> {
       # Start transaction 
       automaticcall
   }
}

proc ShowAttributes {attrcount} {
    #
    # dialog for extended attributes for a node
    #
    global attr attrname noofattr

    toplevel .attr$attrcount
    wm title .attr$attrcount "Attributes"

    frame .attr$attrcount.top -relief raised -bd 1
    frame .attr$attrcount.bottom  -relief raised -bd 1
    pack .attr$attrcount.top .attr$attrcount.bottom -side top -expand 1 -fill both -pady 0 -ipady 2
    
    frame .attr$attrcount.top.left
    frame .attr$attrcount.top.right
    pack .attr$attrcount.top.left -side left -padx 3m -pady 1m
    pack .attr$attrcount.top.right -side right -padx 3m -pady 1m
    
    for {set i 0} {$i<$noofattr} {incr i} {
	message .attr$attrcount.top.left.m$i -text $attrname($i) -width 4i
	pack .attr$attrcount.top.left.m$i -anchor nw -fill y

	message .attr$attrcount.top.right.l$i -width 4i -text $attr($i)
	pack .attr$attrcount.top.right.l$i -anchor nw
    }
    frame .attr$attrcount.bottom.def -relief sunken -bd 3
    pack .attr$attrcount.bottom.def -pady 5 -padx 2 ; # -ipady 2 -ipadx 2
    button .attr$attrcount.bottom.def.ok -relief raised -bd 2 -text " OK " -command "update; destroy .attr$attrcount"
    pack .attr$attrcount.bottom.def.ok -ipadx 5 -ipady 2 -expand 1
}

################################################################################
#
# Needed globals
#
################################################################################

set motionDist 20
set taglist [list]
set taglistShift [list]
set taglistCtrl [list]
set noofattr 0


return ok