spec
abc (* The specification generates graphs of the form *)
    (* a->b->c, a->a->b->b->c->c, ... , i.e. graphs   *)
    (* of the form a^n -> b^n -> c^n (^ = power).     *)
    (*                                                *)
    (* Its first section introduces a simple schema   *)
    (* for the regarded class of graphs. The next     *)
    (* sections introduce four quite different ways   *)
    (* to generate instances of the regarded class of *)
    (* graphs.                                        *)
    (* @CC_LT Vertical @*)

   declares

      section Schema

         declares

            node_class ANY end;

            edge_type n : ANY -> ANY;

            node_type a : ANY end;

            node_type b : ANY end;

            node_type c : ANY end;

      end;

      section Solution1_with_opt_node (* Not the best solution, but maybe the  *)
                                      (* the most straightforward solution.    *)
                                      (* The optional node `4 in Extend1 could *)
                                      (* be changed into a normal node if an   *)
                                      (* additional production is used for the *)
                                      (* case abc ==> aabbcc, where the node c *)
                                      (* has not yet a successor node.         *)

         declares

            transformation Start1 =
               (<OptPrecondDecl>)
               (* @CC_LG 7008(8,8)(51,61) @*)

               ::=
               begin
                  1' : a;
                  (* @CC_LG 6012(168,208)(97,21) @*)
                  2' : b;
                  (* @CC_LG 6012(192,118)(97,21) @*)
                  3' : c;
                  (* @CC_LG 6012(168,28)(97,21) @*)
                  1' -> 2' : n;
                  (* @CC_LG
                  3004C3
                  168,168 @*)
                  2' -> 3' : n;
                  (* @CC_LG 3004C @*)
                  1' -> 3' : n (* @CC_LG 1002C,TC(0,10)(6,10) @*);
                  (* @CC_LG
                  3004C6
                  144,168
                  144,140
                  120,112
                  120,84 @*)
               end
               (* @CC_LG 7009(8,8)(253,239) @*)
            end;

            transformation Extend1 =

               begin
                  obl_node `1 : a;
                  (* @CC_LG 6007(158,240)(53,21) @*)
                  obl_node `2 : b;
                  (* @CC_LG 6007(206,28)(53,21) @*)
                  obl_node `3 : c;
                  (* @CC_LG 6007(134,150)(53,21) @*)
                  `1 -> `2 : n;
                  (* @CC_LG
                  3004C4
                  180,196
                  204,168 @*)
                  `1 -> `3 : n (* @CC_LG 1002C,TC(0,10)(6,10) @*);
                  (* @CC_LG
                  3004C3
                  156,196 @*)
                  opt_node `4 : c;
                  (* @CC_LG 6009(132,56)(97,21) @*)
                  `3 -> `4 : n;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(315,271) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(173,436)(59,21) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(197,28)(59,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(125,248)(59,21) @*)
                  opt_node 4' = `4;
                  (* @CC_LG 6009(120,56)(97,21) @*)
                  5' : a;
                  (* @CC_LG 6012(168,342)(97,21) @*)
                  6' : b;
                  (* @CC_LG 6012(216,122)(97,21) @*)
                  7' : c;
                  (* @CC_LG 6012(120,154)(97,21) @*)
                  1' -> 5' : n;
                  (* @CC_LG 3004C @*)
                  5' -> 6' : n;
                  (* @CC_LG
                  3004C5
                  192,280
                  192,252
                  216,224 @*)
                  6' -> 2' : n (* @CC_LG 1002C,BC(8,-3)(6,10) @*);
                  (* @CC_LG 3004C @*)
                  3' -> 7' : n;
                  (* @CC_LG 3004C @*)
                  7' -> 4' : n;
                  (* @CC_LG 3004C @*)
                  5' -> 3' : n (* @CC_LG 1002C,BC(39,-10)(6,10) @*);
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(315,467) @*)
            end;

            transformation Stop1 =

               begin
                  obl_node `1 : a;
                  (* @CC_LG 6007(48,28)(53,21) @*)
                  obl_node `2 : b;
                  (* @CC_LG 6007(120,28)(53,21) @*)
                  obl_node `3 : c;
                  (* @CC_LG 6007(192,28)(53,21) @*)
                  `1 -> `2 : n;
                  (* @CC_LG 3004C @*)
                  `1 -> `3 : n (* @CC_LG 1002C,TC(0,10)(6,10) @*);
                  (* @CC_LG
                  3004C4
                  72,56
                  168,56 @*)
               end
               (* @CC_LG 7008(8,8)(243,89) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(48,28)(53,21) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(120,28)(53,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(192,28)(53,21) @*)
                  1' -> 2' : n (* @CC_LG 1002C,BC(0,-10)(6,10) @*);
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(243,61) @*)
            end;

            transformation Loop1 =

                  begin
                     Extend1
                     & Loop1
                  end
               or skip
            end;

            transformation Main1 =

               Start1
               & Loop1
               & Stop1
            end;

      end;

      section Solution2_with_Folding (* This is the most readable solution. *)
                                     (* It uses the transitive, reflexive   *)
                                     (* closure to "jump" from the first b  *)
                                     (* node to the last b node. The first  *)
                                     (* application of production Extend2   *)
                                     (* has the problem that there is only  *)
                                     (* a single b node which is the first  *)
                                     (* and the last b node simultaneously. *)
                                     (* The "folding" construct allows that *)
                                     (* `2 and `3 match the same b node if  *)
                                     (* necessary.                          *)
                                     (* The folding could be avoided if an  *)
                                     (* additional production is introduced *)
                                     (* that generates "aabbcc".            *)

         declares

            transformation Start2 =
               (<OptPrecondDecl>)
               (* @CC_LG 7008(8,8)(51,61) @*)

               ::=
               begin
                  1' : a;
                  (* @CC_LG 6012(144,208)(97,21) @*)
                  2' : b;
                  (* @CC_LG 6012(144,118)(97,21) @*)
                  3' : c;
                  (* @CC_LG 6012(144,28)(97,21) @*)
                  1' -> 2' : n;
                  (* @CC_LG 3004C @*)
                  2' -> 3' : n;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(243,239) @*)
            end;

            transformation Extend2 =

               begin
                  obl_node `1 : a;
                  (* @CC_LG 6007(48,28)(53,21) @*)
                  obl_node `2 : b;
                  (* @CC_LG 6007(120,28)(53,21) @*)
                  obl_node `3 : b;
                  (* @CC_LG 6007(216,28)(53,21) @*)
                  `1 -> `2 : n;
                  (* @CC_LG 3004C @*)
                  obl_node `4 : c;
                  (* @CC_LG 6007(288,28)(53,21) @*)
                  `3 -> `4 : n;
                  (* @CC_LG 3004C @*)
                  `2 => `3 : n *;
                  (* @CC_LG 3006 @*)
               end
               (* @CC_LG 7008(8,8)(339,61) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(108,322)(59,21) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(108,28)(59,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(276,215)(49,21) @*)
                  obl_node 4' = `4;
                  (* @CC_LG 6007(276,28)(59,21) @*)
                  5' : a;
                  (* @CC_LG 6012(108,220)(97,21) @*)
                  6' : b;
                  (* @CC_LG 6012(108,122)(97,21) @*)
                  7' : c;
                  (* @CC_LG 6012(276,122)(97,21) @*)
                  1' -> 5' : n;
                  (* @CC_LG 3004C @*)
                  5' -> 6' : n;
                  (* @CC_LG 3004C @*)
                  6' -> 2' : n;
                  (* @CC_LG 3004C @*)
                  3' -> 7' : n;
                  (* @CC_LG 3004C @*)
                  7' -> 4' : n;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(373,345) @*)
               folding { `2, `3 };
            end;

            transformation Loop2 =

                  begin
                     Extend2
                     & Loop2
                  end
               or skip
            end;

            transformation Main2 =

               Start2
               & Loop2
            end;

      end;

      section Solution3_with_embedding_rule (* This is the ugly solution.    *)
                                            (* It shows that context-free    *)
                                            (* graph grammars (with a single *)
                                            (* node in the left-hand side    *)
                                            (* and with embedding rules) are *)
                                            (* already very powerful.        *)

         declares

            node_type N : ANY end;

            edge_type h : ANY -> ANY;

            transformation Simple3 =
               (<OptPrecondDecl>)
               (* @CC_LG 7008(8,8)(51,61) @*)

               ::=
               begin
                  1' : a;
                  (* @CC_LG 6012(96,252)(97,29) @*)
                  2' : b;
                  (* @CC_LG 6012(96,154)(97,29) @*)
                  3' : c;
                  (* @CC_LG 6012(96,56)(97,29) @*)
                  2' -> 3' : n;
                  (* @CC_LG 3004C @*)
                  1' -> 2' : n;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(243,287) @*)
            end;

            transformation Start3 =
               (<OptPrecondDecl>)
               (* @CC_LG 7008(8,8)(51,61) @*)

               ::=
               begin
                  1' : a;
                  (* @CC_LG 6012(72,350)(97,29) @*)
                  2' : b;
                  (* @CC_LG 6012(96,154)(97,29) @*)
                  3' : c;
                  (* @CC_LG 6012(72,56)(97,29) @*)
                  2' -> 3' : n;
                  (* @CC_LG
                  3004C3
                  72,112 @*)
                  4' : N;
                  (* @CC_LG 6012(72,252)(97,29) @*)
                  1' -> 4' : n;
                  (* @CC_LG 3004C @*)
                  4' -> 2' : n (* @CC_LG 1002C,TC(0,10)(6,10) @*);
                  (* @CC_LG
                  3004C3
                  96,196 @*)
                  4' -> 3' : h;
                  (* @CC_LG
                  3004C6
                  48,196
                  24,168
                  24,140
                  48,112 @*)
               end
               (* @CC_LG 7009(8,8)(243,385) @*)
            end;

            transformation Extend3 =

               begin
                  obl_node `1 : N;
                  (* @CC_LG 6007(48,28)(53,21) @*)
               end
               (* @CC_LG 7008(8,8)(99,61) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(173,150)(59,21) @*)
                  2' : a;
                  (* @CC_LG 6012(168,244)(97,29) @*)
                  3' : b;
                  (* @CC_LG 6012(96,56)(97,29) @*)
                  4' : c;
                  (* @CC_LG 6012(240,56)(97,29) @*)
                  2' -> 1' : n;
                  (* @CC_LG 3004C @*)
                  1' -> 3' : n (* @CC_LG 1002C,TC(0,10)(6,10) @*);
                  (* @CC_LG 3004C @*)
                  1' -> 4' : h;
                  (* @CC_LG
                  3004C3
                  216,112 @*)
               end
               (* @CC_LG 7009(8,8)(301,279) @*)
               embedding redirect <-n- from `1 to 2';
                         redirect -n-> from `1 to 3';
                         redirect -h-> as <-n- from `1 to 4';
            end;

            transformation Stop3 =

               begin
                  obl_node `1 : N;
                  (* @CC_LG 6007(48,28)(53,21) @*)
               end
               (* @CC_LG 7008(8,8)(99,61) @*)
               ::=
               begin
                  2' : a;
                  (* @CC_LG 6012(240,154)(97,29) @*)
                  3' : b;
                  (* @CC_LG 6012(240,56)(97,29) @*)
                  4' : c;
                  (* @CC_LG 6012(96,56)(97,29) @*)
                  2' -> 3' : n;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(301,189) @*)
               embedding redirect <-n- from `1 to 2';
                         redirect -n-> from `1 to 3';
                         redirect -h-> as <-n- from `1 to 4';
            end;

            transformation Loop3 =

                  begin
                     Extend3
                     & Loop3
                  end
               or skip
            end;

            transformation Main3 =

                  Simple3
               or begin
                     Start3
                     & Loop3
                     & Stop3
                  end
            end;

      end;

      section Solution4_with_parallel_rewriting (* This is the tricky solution.  *)
                                                (* It uses a production Extend4  *)
                                                (* which is simulatenaeously     *)
                                                (* applied to all matches of its *)
                                                (* left-hand side in the graph.  *)
                                                (* A variant of Extend4 without  *)
                                                (* the negative nodes duplicates *)
                                                (* all nodes in the graph, i.e.  *)
                                                (* creates the graph language    *)
                                                (*          n   n   n            *)
                                                (*         2   2   2             *)
                                                (*        a   b   c              *)

         declares

            transformation Start4 =
               (<OptPrecondDecl>)
               (* @CC_LG 7008(8,8)(51,61) @*)

               (<OptLeftSideList>)
               (* @CC_LG 7008C(39,33) @*)
               ::=
               begin
                  1' : a;
                  (* @CC_LG 6012(96,252)(97,29) @*)
                  2' : b;
                  (* @CC_LG 6012(96,154)(97,29) @*)
                  3' : c;
                  (* @CC_LG 6012(96,56)(97,29) @*)
                  2' -> 3' : n;
                  (* @CC_LG 3004C @*)
                  1' -> 2' : n;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(243,287) @*)
            end;

            transformation Extend4 + =

               begin
                  obl_node `1 : ANY;
                  (* @CC_LG 6007(152,150)(65,21) @*)
                  opt_node `2 : ANY;
                  (* @CC_LG 6009(240,56)(97,29) @*)
                  `1 -> `2 : n;
                  (* @CC_LG 3004C @*)
                  not_node `3 : `1.type;
                  (* @CC_LG 6013(144,244)(97,29) @*)
                  not_node `4 : `1.type;
                  (* @CC_LG 6013(96,56)(97,29) @*)
                  `3 -> `1 : n;
                  (* @CC_LG 3004C @*)
                  `1 -> `4 : n;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(301,279) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(77,248)(59,21) @*)
                  opt_node 2' = `2;
                  (* @CC_LG 6009(96,56)(97,29) @*)
                  3' : `1.type;
                  (* @CC_LG 6012(96,154)(97,29) @*)
                  1' -> 3' : n;
                  (* @CC_LG 3004C @*)
                  3' -> 2' : n;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(291,279) @*)
            end;

            transformation Loop4 =

                  begin
                     Extend4
                     & Loop4
                  end
               or skip
            end;

            transformation Main4 =

               Start4
               & Loop4
            end;

      end;

      transformation MAIN =

         begin
               Main1
            or Main2
            or Main3
            or Main4
         end
         & fail
      end;

end.
(* @CC_LT Vertical @*)
