spec Colouring (* The specification solves the COLOURING *)
               (* problem for a given map, i.e. a number *)
               (* countries with their adjacency/touches *)
               (* relationships. Any country is assigned *)
               (* colour (out of four different colours) *)
               (* such that two adjacent countries have  *)
               (* different colours.                     *)
               (*                                        *)
               (* The specification contains four well-  *)
               (* known constraint solving algorithms    *)
               (* for solving the problem:               *)
               (* 1) Test and generate                   *)
               (* 2) Depth-first search and Backtracking *)
               (* 3) Forward checking                    *)
               (* 4) Look ahead                          *)
               (* For further details concerning these   *)
               (* algorithms the reader is referred to:  *)
               (*                                        *)
               (* Pascal van Hentenrynck: Constraint     *)
               (* Satisfaction in Logic Programming      *)
               (* MIT Press (1989)                       *)
               (*                                        *)
               (* Please note: for many years it was an  *)
               (* open problem whether four colours are  *)
               (* sufficient to colour any map. Rather   *)
               (* recently, Prof. Haken from University  *)
               (* of Illenois constructed the proof with *)
               (* the help of a computer program.        *)

   declares

      node_class NODE
         intrinsic
            key name : string;
            (* @CC_LG
            3010XC2
            96,0 @*)
      end;
      (* @CC_LG 6007(196,250)(97,29) @*)

      node_type Country : NODE (* @CC_LG 3005C @*)
         intrinsic
            label (* @CC_LG 1002TC(0,8)(30,10) @*) : string := "";
            (* @CC_LG
            3010XC2
            20,-46 @*)
      end;
      (* @CC_LG 6013(124,140)(97,29) @*)

      node_type Colour : NODE
         (<OptRuleList>)
         (* @CC_LG 3005C @*)
      end;
      (* @CC_LG 6013(312,140)(97,29) @*)

      edge_type touches (* @CC_LG 1002C,BC(0,-10)(42,10) @*) : Country -> Country;
      (* @CC_LG
      3004M5
      49,133
      49,102
      92,102 @*)

      path adjacent (* @CC_LG 1002C,TC(0,10)(54,10) @*) : Country -> Country =
             -touches->
         or  <-touches-
      end;
      (* @CC_LG
      3006M5
      96,178
      48,178
      48,145 @*)

      edge_type has : Country -> Colour;
      (* @CC_LG 3004C @*)

      transformation CreateCountry ( N : string) =
         (<OptPrecondDecl>)
         (* @CC_LG 7008(8,8)(36,38) @*)

         (<OptLeftSideList>)
         (* @CC_LG 7008(8,8)(24,24) @*)
         ::=
         begin
            1' : Country;
            (* @CC_LG 6012(96,56)(97,29) @*)
         end
         (* @CC_LG 7009(8,8)(157,91) @*)
         transfer 1'.name := N;
      end;

      transformation CreateColour ( N : string) =
         (<OptPrecondDecl>)
         (* @CC_LG 7008(8,8)(36,38) @*)

         (<OptLeftSideList>)
         (* @CC_LG 7008(8,8)(24,24) @*)
         ::=
         begin
            1' : Colour;
            (* @CC_LG 6012(96,56)(97,29) @*)
         end
         (* @CC_LG 7009(8,8)(157,91) @*)
         transfer 1'.name := N;
      end;

      transformation Touches ( N1, N2 : string)
                             (* Touches is a symmetric relation. *)
                             (* Having directed edges only, we   *)
                             (* simulate the indirected case by  *)
                             (* creating pairs of edges.         *) =

         begin
            obl_node `1 : Country;
            (* @CC_LG 6007(56,20)(89,21) @*)
            obl_node `2 : Country;
            (* @CC_LG 6007(176,20)(89,21) @*)
         end
         (* @CC_LG 7008(8,8)(233,41) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(41,26)(59,21) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(168,26)(59,21) @*)
            1' -> 2' : touches;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(210,47) @*)
         condition `1.name = N1;
                   `2.name = N2;
      end;

      transformation SetupColours ( out Countries : Country [0:n])
                                  (* The production creates the    *)
                                  (* initial situation for all     *)
                                  (* constraint solving algorithms *)
                                  (* any country may still have    *)
                                  (* any existing colour.          *)
                                  (* The set nodes `1 and `2 match *)
                                  (* all countries and colours in  *)
                                  (* the graph. The has edge is    *)
                                  (* created between all matches   *)
                                  (* of `1 and all matches of `2.  *) =

         begin
            obl_set `1 : Country;
            (* @CC_LG 6011(240,56)(97,29) @*)
            obl_set `2 : Colour;
            (* @CC_LG 6011(96,56)(97,29) @*)
         end
         (* @CC_LG 7008(8,8)(306,100) @*)
         ::=
         begin
            obl_set 1' = `1;
            (* @CC_LG 6011(96,158)(97,29) @*)
            obl_set 2' = `2;
            (* @CC_LG 6011(96,56)(97,29) @*)
            1' -> 2' : has;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(162,193) @*)
         return Countries := 1';
      end;

      transformation ComputeLabel ( This : Country) =

         begin
            obl_node `1 = This;
            (* @CC_LG 6007(96,154)(71,21) @*)
            opt_set `2 : Colour;
            (* @CC_LG 6010(96,56)(97,29) @*)
            `1 -> `2 : has;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(162,181) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(96,150)(59,21) @*)
            opt_set 2' = `2;
            (* @CC_LG 6010(96,56)(97,29) @*)
            1' -> 2' : has (* @CC_LG 1002C,BC(0,-10)(18,10) @*);
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(162,181) @*)
         transfer 1'.label := conc ( `1.name & ":", all `2.name );
      end;

      transformation ComputeLabels ( ThisSet : Country [0:n]) =

         for_all This := elem ( ThisSet )
         do
            ComputeLabel ( This )
         end
      end;

      function conc : ( first, second : string) -> string =
         first & " " & second
      end;

      transformation RestrictNeighbours ( This : Country ;
                                           out Restricted : Country [0:n]
                                           (* @CC_LT VertWithIndent @*)  )
                                        (* Restricting the colour set  *)
                                        (* of a country is possible if *)
                                        (* one adjacent country has a  *)
                                        (* single colour. This single  *)
                                        (* colour may then be removed  *)
                                        (* from the colour sets of all *)
                                        (* neighbour countries.        *)
                                        (* The production receives a   *)
                                        (* country node with a maybe   *)
                                        (* singleton set of colours as *)
                                        (* input and returns a set of  *)
                                        (* countries whose colour sets *)
                                        (* have been restricted.       *)  =

         begin
            obl_node `2 = This (* @CC_LG 4005LC(24,0)(36,12) @*);
            (* @CC_LG 6007(215,249)(71,21) @*)
            obl_node `3 : Colour;
            (* @CC_LG 6007(245,57)(83,21) @*)
            not_node `5 : Colour;
            (* @CC_LG 6013(108,28)(97,21) @*)
            `2 -> `3 : has (* @CC_LG 1002C,LC(10,0)(18,10) @*);
            (* @CC_LG
            3004C3
            204,112 @*)
            `2 -> `5 : has (* @CC_LG 1002C,TC(0,10)(18,10) @*);
            (* @CC_LG
            3004C7
            180,196
            180,168
            156,140
            156,112
            132,84 @*)
            opt_set `1 : Country;
            (* @CC_LG 6010(276,155)(97,21) @*)
            `1 -> `3 : has (* @CC_LG 1002C,RC(-10,0)(18,10) @*);
            (* @CC_LG 3004C @*)
            `2 => `1 : adjacent;
            (* @CC_LG 3008C @*)
         end
         (* @CC_LG 7008(8,8)(378,280) @*)
         ::=
         begin
            obl_node 2' = `2;
            (* @CC_LG 6007(221,172)(59,21) @*)
            obl_node 3' = `3;
            (* @CC_LG 6007(221,82)(59,21) @*)
            2' -> 3' : has (* @CC_LG 1002C,LC(10,0)(18,10) @*);
            (* @CC_LG 3004C @*)
            opt_set 1' = `1;
            (* @CC_LG 6010(96,56)(97,29) @*)
         end
         (* @CC_LG 7009(8,8)(263,203) @*)
         return Restricted := 1';
      end;

      transformation SelectColour ( This : Country)
                                  (* The production selects one    *)
                                  (* colour in the colour set of a *)
                                  (* country nondeterministically. *)
                                  (* It preserves the has edge to  *)
                                  (* the selected colour `2 and    *)
                                  (* removes the has edges to all  *)
                                  (* remaining colour nodes `3.    *)
                                  (* Later on another colour may   *)
                                  (* be selected by means of back- *)
                                  (* tracking.                     *) =

         begin
            obl_node `1 = This;
            (* @CC_LG 6007(179,166)(71,21) @*)
            obl_node `2 : Colour;
            (* @CC_LG 6007(233,76)(83,21) @*)
            `1 -> `2 : has;
            (* @CC_LG 3004C @*)
            opt_set `3 : Colour;
            (* @CC_LG 6010(96,56)(97,29) @*)
            `1 -> `3 : has;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(287,197) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(221,166)(59,21) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(221,76)(59,21) @*)
            1' -> 2' : has (* @CC_LG 1002C,BC(0,-10)(18,10) @*);
            (* @CC_LG 3004C @*)
            opt_set 3' = `3;
            (* @CC_LG 6010(96,56)(97,29) @*)
         end
         (* @CC_LG 7009(8,8)(263,197) @*)
      end;

      query DangerousNeighbours( This : Country ;
                                  out Neighbours : Country [0:n]
                                  (* @CC_LT VertWithIndent @*)  )
                               (* The production returns any *)
                               (* neighbour of a person with *)
                               (* the same possible colour   *)
                               (* assignment.                *)   =
         begin
            obl_node `1 = This;
            (* @CC_LG 6007(107,226)(71,21) @*)
            obl_node `3 : Colour;
            (* @CC_LG 6007(113,34)(83,21) @*)
            `1 -> `3 : has (* @CC_LG 1002C,BC(0,-10)(18,10) @*);
            (* @CC_LG
            3004C6
            96,182
            96,154
            72,126
            72,98 @*)
            opt_set `2 : Country;
            (* @CC_LG 6010(144,132)(97,21) @*)
            `2 -> `3 : has (* @CC_LG 1002C,LC(12,0)(18,10) @*);
            (* @CC_LG 3004C @*)
            `1 => `2 : adjacent;
            (* @CC_LG 3008C @*)
         end
         (* @CC_LG 7008(8,8)(254,257) @*)
         return Neighbours := `2;
      end;

      query CountryWithoutColour( This : Country) =
         begin
            obl_node `1 = This;
            (* @CC_LG 6007(94,150)(71,21) @*)
            not_node `2 : Colour;
            (* @CC_LG 6013(96,56)(97,29) @*)
            `1 -> `2 : has;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(157,181) @*)
      end;

      query GetCountries( out Countries : Country [1:n]) =
         begin
            obl_set `1 : Country;
            (* @CC_LG 6011(96,56)(97,29) @*)
         end
         (* @CC_LG 7008(8,8)(162,91) @*)
         return Countries := `1;
      end;

      transformation GenerateTest (* The straightforward solution:  *)
                                  (* Select a single colour for any *)
                                  (* country and check then whether *)
                                  (* adjacent countries received    *)
                                  (* different colours. Use another *)
                                  (* colour assignment if necessary.*)
                                  (* Note that the given solution   *)
                                  (* uses already backtracking!     *)
                                  (* The first for_all loop creates *)
                                  (* one colour assignment for all  *)
                                  (* involved countries. The second *)
                                  (* for_all loop fails for wrong   *)
                                  (* assignments and causes back-   *)
                                  (* tracking into the first loop   *)
                                  (* which computes then another    *)
                                  (* colour assignment.             *)
                                  (* @CC_LT HoriWithIndent @*)         =

            use Countries : Country [0:n];
                Dangerous : Country [0:n]
            do
               GetCountries ( out Countries )
               & for_all C := elem ( Countries )
                 do
                    SelectColour ( C )
                    & ComputeLabel ( C )
                 end
               & for_all C := elem ( Countries )
                 do
                    DangerousNeighbours ( C, out Dangerous )
                    & (card ( Dangerous ) = 0)
                 end
            end
            (* @CC_LT VertWithIndentAndSpace @*)
      end;

      transformation
      NaiveBacktracking (* The backtracking algorithm *)
                        (* makes all necessary checks *)
                        (* as early as possible, i.e. *)
                        (* it does not compute a full *)
                        (* colour assigment for all   *)
                        (* countries if the first two *)
                        (* countries have already the *)
                        (* same colours.              *)
                        (* @CC_LT Vertical @*)           =

            use Countries : Country [0:n];
                Processed : Country [0:n] := nil;
                Dangerous : Country [0:n]
            do
               GetCountries ( out Countries )
               & for_all C := elem ( Countries )
                 do
                    Processed := (Processed or C)
                    & SelectColour ( C )
                    & ComputeLabel ( C )
                    & DangerousNeighbours ( C, out Dangerous )
                    & (card ( Dangerous and Processed ) = 0)
                 end
            end
            (* @CC_LT VertWithIndentAndSpace @*)
      end;

      transformation ForwardChecking (* This is usually the best solution. *)
                                     (* Any colour assignment is followed  *)
                                     (* by a processing of all neighbour   *)
                                     (* countries. The selected colour is  *)
                                     (* removed from their colour sets.    *)
                                     (* Backtracking starts if one of the  *)
                                     (* neigbour colour sets becomes empty.*)
                                     (* @CC_LT HoriWithIndent @*)             =

         use Countries : Country [0:n];
             Restricted : Country [0:n]
         do
            GetCountries ( out Countries )
            & for_all C := elem ( Countries )
              do
                 SelectColour ( C )
                 & ComputeLabel ( C )
                 & RestrictNeighbours ( C, out Restricted )
                 & ComputeLabels ( Restricted )
                 & not CountryWithoutColour ( Restricted )
              end
         end
      end;

      transformation LookAhead (* A theoretically better algorithm than *)
                               (* ForwardChecking. But the additional   *)
                               (* processing overhead is very often not *)
                               (* worthwhile the effort. The algorithm  *)
                               (* processes the colour sets of indirect *)
                               (* neighbours of the selected country,   *)
                               (* too.                                  *)
                               (* @CC_LT HoriWithIndent @*)                =

            use Countries : Country [0:n];
                Restricted : Country [0:n];
                WorkingSet : Country [0:n]
            do
               GetCountries ( out Countries )
               & for_all C := elem ( Countries )
                 do
                    SelectColour ( C )
                    & ComputeLabel ( C )
                    & WorkingSet := set_of ( C )
                    & for_all Elem := elem ( WorkingSet )
                      do
                         choose
                            RestrictNeighbours ( Elem, out Restricted )
                            & ComputeLabels ( Restricted )
                         else
                            Restricted := nil
                         end
                         & not CountryWithoutColour ( Restricted )
                         & WorkingSet := ((WorkingSet but_not Elem) or Restricted (* @CC_LT Vertical @*)
                                                                                  (* @CC_LT VertWithIndent @*))
                      end
                 end
                 (* @CC_LT HoriWithSpace @*)
            end
            (* @CC_LT VertWithIndentAndSpace @*)
      end;

      transformation MAIN =

         CreateCountry ( "Netherlands" )
         & CreateCountry ( "France" )
         & CreateCountry ( "Spain" )
         & CreateCountry ( "Belgium" )
         & CreateCountry ( "Luxembourg" )
         & CreateCountry ( "Germany" )
         & CreateCountry ( "Switzerland" )
         & CreateCountry ( "Austria" )
         & CreateCountry ( "Italy" )
         & Touches ( "Germany", "Austria" )
         & Touches ( "Austria", "Italy" )
         & Touches ( "Switzerland", "Austria" )
         & Touches ( "Belgium", "France" )
         & Touches ( "Belgium", "Germany" )
         & Touches ( "Belgium", "Luxembourg" )
         & Touches ( "Netherlands", "Belgium" )
         & Touches ( "France", "Germany" )
         & Touches ( "France", "Italy" )
         & Touches ( "France", "Luxembourg" )
         & Touches ( "France", "Spain" )
         & Touches ( "France", "Switzerland" )
         & Touches ( "Luxembourg", "Germany" )
         & Touches ( "Netherlands", "Germany" )
         & Touches ( "Germany", "Switzerland" )
         & Touches ( "Switzerland", "Italy" )
         & CreateColour ( "red" )
         & CreateColour ( "green" )
         & CreateColour ( "yellow" )
         & CreateColour ( "blue" )
         & use Countries : Country [0:n]
           do
              SetupColours ( out Countries )
              & ComputeLabels ( Countries )
           end
         & begin
                 GenerateTest
              or NaiveBacktracking
              or ForwardChecking
              or LookAhead
           end
      end;

end.
(* @CC_LG 7012(8,8)(401,303) @*)
