spec FerryMan (* The programmed graph rewriting system is a rule *)
              (* based specification of the following well-known *)
              (* AI problem:                                     *)
              (*                                                 *)
              (* The ferry man's problem:                        *)
              (* ------------------------                        *)
              (*  There are a cabbage, a goat, and a wolf which  *)
              (*  must be ferried over a small river. But the    *)
              (*  ferry is too small to carry more than one of   *)
              (*  the objects at a time. Furthermore, the ferry  *)
              (*  man must not leave the wolf and the goat or    *)
              (*  the goat and the cabbage at one river side     *)
              (*  with himself being on the other side.          *)
              (* @CC_LT HoriWithIndent @*)

   declares section AuxiliaryFunctions

               declares

                  function $ : ( Dollar : integer) -> integer =
                     Dollar * 100
                  end;

                  function insurance : ( Police : integer) -> integer =
                     Police / 10
                  end;

                  function Payment : -> integer =
                     $ 30
                  end;

            end;
            (* Auxiliary Functions *)
            (* @CC_LT HoriWithIndent @*)
            (* @CC_LG 7015H(768,350)(97,29) @*)

   section MainActions

      declares

         transformation SolveFerryMansProblem (* Try&Error solution for the ferry  *)
                                              (* man's problem. We are looking for *)
                                              (* a solution with a minimal number  *)
                                              (* of basic operations.              *) =

            use StepCounter : integer
            do
               CreateProblem
               & StepCounter := 0
               & loop
                    when def ( StillCargoOnThisSide )
                    then
                       OneStep
                       & StepCounter := (StepCounter + 1)
                 end
            end
         end;

         transformation OneStep =

               TransferCargo
            or EmptyMove
         end;

         transformation TransferCargo =

            LoadFerry
            & not DangerousSituation
            & FerryOver
            & UnloadFerry
         end;

         transformation EmptyMove =

            not DangerousSituation
            & FerryOver
         end;

   end;
   (* SolveFerryMansProblem *)
   (* @CC_LT VertWithSpace @*)
   (* @CC_LG 7015H(768,266)(97,29) @*)

   section
   GraphScheme (* Definition of the overall graph scheme *)
               (* for boats, cargos etc.                 *)
               (* @CC_LT Vertical @*)

      declares

         node_class THING
            intrinsic
               Expenditures := $ 0;
               (* @CC_LG
               3010XC2
               -72,-56 @*)
            derived
               Price = $ 0;
               (* @CC_LG
               3012XC2
               -96,0 @*)
               Win = self.Price - self.Expenditures;
               (* @CC_LG
               3012XC2
               96,0 @*)
         end;
         (* @CC_LG 6013(204,84)(251,121) @*)

         node_class RIVER_SIDE end;
         (* @CC_LG 6013H(312,406)(97,29) @*)

         node_class CARGO is_a THING (* @CC_LG 3005C @*) end;
         (* @CC_LG 6013H(528,266)(97,29) @*)

         edge_type On : CARGO [0:n] -> RIVER_SIDE [0:1];
         (* A CARGO is at most at one river side and we have *)
         (* an arbitrary number of cargos on each river side *)
         (* @CC_LG 3004C @*)

         edge_type In : CARGO [0:1] -> BOAT [0:1];
         (* A CARGO is in at most one BOAT at the same time *)
         (* @CC_LG 3004C @*)

         node_class BOAT is_a THING (* @CC_LG 3005C @*)
            intrinsic
               Income := $ 70;
               (* @CC_LG
               3010XC2
               108,0 @*)
            derived
               TransferCosts (* @CC_LG 1002TC(0,8)(78,10) @*) = ($ 20) + insurance ( [ self.<-In-.Price
                                                                                     | $ 0 ]            );
               (* TransferCosts are fixed costs plus insurance which is *)
               (* proportional to the price of the BOAT'S CARGO         *)
               (* @CC_LG
               3012XC2
               0,42 @*)
            redef_derived
               Win = self.Income - self.Expenditures;
         end;
         (* @CC_LG 6013(204,266)(383,145) @*)

         edge_type At : BOAT [0:n] -> RIVER_SIDE [1:1];
         (* @CC_LG 3004C @*)

         node_class BEING is_a CARGO (* @CC_LG 3005C @*) end;
         (* @CC_LG 6013H(528,336)(97,29) @*)

         node_class PLANT is_a BEING (* @CC_LG 3005C @*) end;
         (* @CC_LG 6013H(456,406)(97,29) @*)

         node_class ANIMAL is_a BEING (* @CC_LG 3005C @*) end;
         (* @CC_LG 6013H(612,406)(97,29) @*)

   end;
   (* GlobalScheme *)
   (* @CC_LT VertWithSpace @*)
   (* @CC_LG 7015H(12,70)(97,29) @*)

   section
   SpecialObjectTypes (* Definition of the problem specific *)
                      (* object types Ferry, Wolf etc.      *)
                      (* @CC_LT Vertical @*)

      declares

         node_type ThisSide : RIVER_SIDE end;
         (* @CC_LG 6015H(72,56)(97,29) @*)

         node_type OtherSide : RIVER_SIDE end;
         (* @CC_LG 6015H(192,56)(97,29) @*)

         node_type Ferry : BOAT
            redef_derived
               Price = $ 1000;
         end;
         (* @CC_LG 6015H(312,56)(97,29) @*)

         node_type Cabbage : PLANT
            redef_derived
               Price = $ 40;
         end;
         (* @CC_LG 6015H(312,112)(97,29) @*)

         node_type Goat : ANIMAL
            redef_derived
               Price = $ 100;
         end;
         (* @CC_LG 6015H(192,112)(97,29) @*)

         node_type Wolf : ANIMAL
            redef_derived
               Price = $ 60;
         end;
         (* @CC_LG 6015H(72,112)(97,29) @*)

   end;
   (* SpecialObjectTypes *)
   (* @CC_LT VertWithSpace @*)
   (* @CC_LG 7015(756,70)(387,141) @*)

   section BasicOperations (* Definition of all legal basic actions of *)
                           (* the ferry man - like 'LoadFerry' - and   *)
                           (* some auxiliary graph rewrite rules.      *)
                           (* @CC_LG 1002(56,12)(90,10) @*)

      declares

         transformation
         CreateProblem (* Creates the initial situation with all *)
                       (* objects on this side of the river.     *)
                       (* @CC_LT Vertical @*)                       =
            (<OptPrecondDecl>)
            (* @CC_LG 7008(8,8)(26,28) @*)

            (<OptLeftSideList>)
            (* @CC_LG 7008C(27,33) @*)
            ::=
            begin
               1' : Cabbage;
               (* @CC_LG 6012(408,154)(97,29) @*)
               2' : Wolf;
               (* @CC_LG 6012(552,154)(97,29) @*)
               3' : Goat;
               (* @CC_LG 6012(120,154)(97,29) @*)
               4' : ThisSide;
               (* @CC_LG 6012(384,56)(97,29) @*)
               5' : Ferry;
               (* @CC_LG 6012(264,154)(97,29) @*)
               6' : OtherSide;
               (* @CC_LG 6012(240,56)(97,29) @*)
               2' -> 4' : On;
               (* @CC_LG 3004C @*)
               3' -> 4' : On;
               (* @CC_LG 3004C @*)
               1' -> 4' : On;
               (* @CC_LG
               3004C3
               384,112 @*)
               5' -> 4' : At;
               (* @CC_LG
               3004C3
               336,112 @*)
            end
            (* @CC_LG 7009(8,8)(613,189) @*)
         end;

         transformation LoadFerry (* Puts one cargo into the ferry    *)
                                  (* if the ferry is on the same side *)
                                  (* and empty.                       *) =

            begin
               obl_node `1 : CARGO;
               (* @CC_LG 6007(65,26)(77,21) @*)
               obl_node `2 : RIVER_SIDE;
               (* @CC_LG 6007(65,126)(107,21) @*)
               obl_node `3 : Ferry;
               (* @CC_LG 6007(250,126)(77,21) @*)
               `1 -> `2 : On;
               (* @CC_LG 3004C @*)
               `3 -> `2 : At;
               (* @CC_LG 3004C @*)
               => `3 : WithoutCargo
                       (* @CC_LG 4005LC,RC(-2,0)(72,12) @*);
               (* @CC_LG
               3009XC2
               251,65 @*)
            end
            (* @CC_LG 7008(8,8)(305,157) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(41,40)(59,21) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(41,140)(59,21) @*)
               obl_node 3' = `3;
               (* @CC_LG 6007(226,140)(59,21) @*)
               3' -> 2' : At (* @CC_LG 1002C,BC(0,-9)(12,10) @*);
               (* @CC_LG 3004C @*)
               1' -> 3' : In;
               (* @CC_LG
               3004C3
               226,40 @*)
            end
            (* @CC_LG 7009(8,8)(269,171) @*)
            condition `1.Win > Payment;
            transfer 1'.Expenditures := `1.Expenditures + Payment;
                     3'.Income := `3.Income + Payment;
         end;

         transformation UnloadFerry =

            begin
               obl_node `1 : CARGO;
               (* @CC_LG 6007(232,26)(77,21) @*)
               obl_node `2 : RIVER_SIDE;
               (* @CC_LG 6007(70,126)(107,21) @*)
               obl_node `3 : Ferry;
               (* @CC_LG 6007(228,126)(77,21) @*)
               `3 -> `2 : At;
               (* @CC_LG 3004C @*)
               `1 -> `3 : In;
               (* @CC_LG 3004M @*)
            end
            (* @CC_LG 7008(8,8)(293,155) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(204,28)(59,21) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(48,126)(59,21) @*)
               obl_node 3' = `3;
               (* @CC_LG 6007(204,126)(59,21) @*)
               3' -> 2' : At (* @CC_LG 1002C,BC(0,-10)(12,10) @*);
               (* @CC_LG 3004C @*)
               1' -> 2' : On;
               (* @CC_LG
               3004C3
               48,28 @*)
            end
            (* @CC_LG 7009(8,8)(257,155) @*)
         end;

         transformation FerryOver =

            begin
               obl_node `1 : RIVER_SIDE;
               (* @CC_LG 6007(71,40)(107,21) @*)
               obl_node `2 : Ferry;
               (* @CC_LG 6007(204,40)(77,21) @*)
               obl_node `3 : RIVER_SIDE;
               (* @CC_LG 6007(333,40)(107,21) @*)
               `2 -> `1 : At;
               (* @CC_LG 3004C @*)
            end
            (* @CC_LG 7008(8,8)(401,71) @*)
            ::=
            begin
               obl_node 1' = `1;
               (* @CC_LG 6007(47,40)(59,21) @*)
               obl_node 2' = `2;
               (* @CC_LG 6007(180,40)(59,21) @*)
               obl_node 3' = `3;
               (* @CC_LG 6007(309,40)(59,21) @*)
               2' -> 3' : At;
               (* @CC_LG 3004C @*)
            end
            (* @CC_LG 7009(8,8)(353,71) @*)
            condition `2.Win > `2.TransferCosts;
            transfer 2'.Expenditures := `2.Expenditures + `2.TransferCosts;
         end;

         query StillCargoOnThisSide =
            begin
               obl_node `1 : CARGO;
               (* @CC_LG 6007(64,26)(77,21) @*)
               obl_node `2 : ThisSide;
               (* @CC_LG 6007(64,132)(95,21) @*)
               `1 -> `2 : On;
               (* @CC_LG 3004C @*)
            end
            (* @CC_LG 7008(8,8)(125,157) @*)
         end;

         query DangerousSituation =
            begin
               obl_node `2 : BEING;
               (* @CC_LG 6007(260,40)(77,21) @*)
               obl_node `1 : ANIMAL;
               (* @CC_LG 6007(56,41)(83,21) @*)
               `1 => `2 : NearByFood;
               (* @CC_LG 3006C @*)
            end
            (* @CC_LG 7008(8,8)(315,71) @*)
         end;

   end;
   (* BasicOperations *)
   (* @CC_LT VertWithSpace @*)
   (* @CC_LG 7015H(900,350)(97,29) @*)

   section
   DerivedRelations (* Definition of complex relations and *)
                    (* restrictions.                       *)
                    (* @CC_LT Vertical @*)

      declares

         path NearByFood : ANIMAL -> BEING =
            [ NearBy ( Wolf, Goat )
            | NearBy ( Goat, Cabbage ) ]
         end;

         path NearBy( Subj : type_in ANIMAL ; Obj : type_in BEING) : ANIMAL -> BEING =
               instance_of Subj
            &  -On->
            &  <-On-
            &  instance_of Obj
         end;

         restriction WithoutCargo : BOAT =
            not with <-In-
         end;

   end;
   (* DerivedRelations *)
   (* @CC_LT VertWithSpace @*)
   (* @CC_LG 7015H(900,266)(97,29) @*)

   transformation MAIN =

      SolveFerryMansProblem
   end;
   (* @CC_LT VertWithSpace @*)

end.
(* @CC_LG 7012(8,8)(1162,521) @*)
