spec MIL

   declares

      section IntegerUtilities

         declares

            function max : ( i1, i2 : integer) -> integer =
               [ i1 <= i2 :: i2
               | i1 ]
            end;

            function addSize : ( Val : integer ; Atom : ATOM) -> integer =
               Val + Atom.CodeSize
            end;

      end;
      (* @CC_LG 7013(816,168)(164,42) @*)

      section StringUtilities

         declares

            function find : ( SubStr, InStr : string) -> integer =
               at ( SubStr, InStr, 1 )
            end;
            (* Returns the start position of the first occurrence of *)
            (* a string 'Substr' in another string 'InStr'.          *)

            function at : ( SubStr, InStr : string ; From : integer) -> integer =
               use LS := length ( SubStr );
                   LI := length ( InStr )   ::
                  [ (From + LS - 1) > LI :: 0
                  | SubStr = substr ( InStr, From, From + LS - 1 ) :: From
                  | at ( SubStr, InStr, From + 1 ) ]
               end
            end;
            (* Needed for implementing 'find'. *)

      end;
      (* @CC_LG 7013(816,112)(158,42) @*)

      section VariantProperties (* Defines the data type properties of module variants. *)
                                (* A single property is a pair of (name, value) and it  *)
                                (* is encoded as a string "name:value".                 *)
                                (*                                                      *)
                                (* A 'Variant' may have a set of properties such that:  *)
                                (*                                                      *)
                                (* 1) Property set contains pairs (n, v_1),...,(n, v_n) *)
                                (*    ==> 'Variant' fulfills property 'n' with required *)
                                (*        value 'v_1' or ... or 'v_n'.                  *)
                                (* 2) Property set does not contain a pair (n, ?)       *)
                                (*    ==> 'Variant'  fulfills property 'n' for any      *)
                                (*        required value.                               *)

         declares

            function mkProp : ( PName, PValue : string) -> string =
               PName & ":" & PValue
            end;

            function propName : ( P : string) -> string =
               substr ( P, 1, find ( ":", P ) - 1 )
            end;

            function propValue : ( P : string) -> string =
               substr ( P, find ( ":", P ) + 1, length ( P ) )
            end;

            function selectPSet : ( PName : string
                                            (* @CC_LT HoriWithIndent @*) ;
                                     PSet : string [0:n]
                                     (* @CC_LT VertWithIndent @*))         -> string [0:n]
            =
               use P : string := elem ( PSet ) ::
                  [ propName ( P ) = PName :: P ]
               end
            end;
            (* Returns all ('PName', ?) pairs in a given property set *)
            (* 'PSet' which have 'PName' as their first component.    *)

            function `is_in' : ( P : string ; PSet : string [0:n]) -> boolean =
               [ propName ( P ) in propName ( PSet ) :: P in PSet
               | true ]
            end;
            (* Checks whether 'P' = (N, V) is an element of a   *)
            (* given property set 'PSet' or whether 'PSet' does *)
            (* not contain any (N, ?) pair, i.e. 'PSet' makes   *)
            (* no assumptions about the property with name N.   *)

            function `are_in' : ( PSet1, PSet2 : string [0:n]) -> boolean =
               for_all PName := elem ( propName ( PSet1 ) ) ::
                  exist P1 := elem ( selectPSet ( PName, PSet1 ) ) ::
                     P1 `is_in' PSet2
                  end
               end
            end;
            (* Checks for all specified property names 'PName' in *)
            (* 'PSet1'  whether or not a  pair 'P' = ('PName', ?) *)
            (* exists in 'PSet1' which is guaranteed by 'PSet2'.  *)

            function merge : ( PSet1, PSet2 : string [0:n]) -> string [0:n] =
               use NSet1 := propName ( PSet1 ) but_not propName ( PSet2 );
                   NSet2 := propName ( PSet2 ) but_not propName ( PSet1 )  ::
                     selectPSet ( NSet1, PSet1 )
                     (* @CC_LT SetColumn 3 @*)   or selectPSet ( NSet2, PSet2 ) or
                                                                                (PSet1 and PSet2)
                                                                                (* @CC_LT Vertical @*)
                                                                                (* @CC_LT Vertical @*)
               end
            end;
            (* The result requires any property which is         *)
            (* 1) required in 'PSet1' and unspecified in 'PSet2' *)
            (* 2) required in 'PSet2' and unspecified in 'PSet1' *)
            (* 3) required in 'PSet1' and 'PSet2'.               *)

      end;
      (* @CC_LG 7013(816,224)(170,42) @*)

      section Schema

         declares

            node_class UNIT
               derived
                  CodeSize (* @CC_LG 1002TC(0,8)(24,10) @*) : integer;
                  (* @CC_LG
                  3012XC2
                  108,0 @*)
               intrinsic
                  Name (* @CC_LG 1002TC(0,6)(24,10) @*) : string;
                  (* @CC_LG
                  3010XC2
                  -120,0 @*)
            end;
            (* Root of class hierarchy. *)
            (* @CC_LG 6007(336,420)(97,29) @*)

            node_class SPECIFICATION is_a UNIT (* @CC_LG 3005C @*)
               redef_derived
                  CodeSize = max ( 0, all self.-has->.CodeSize );
               (* The 'Size' of a specification is the maximum of *)
               (* the size of all its realizations, instead of    *)
               (* being the sum (which would reasonable, too).    *)
            end;
            (* A 'SPECIFICATION' is either the complete design of *)
            (* a software system or a design of one its parts.    *)
            (* @CC_LG 6007(252,294)(97,29) @*)

            node_class REALIZATION is_a UNIT (* @CC_LG 3005C @*)
               intrinsic
                  Props (* @CC_LG 1002TC,BC(0,-4)(30,10) @*) : string [0:n]
                                                                      (* @CC_LG 4005LC,RC(-8,0)(72,12) @*);
                  (* A set of guaranteed properties. *)
                  (* @CC_LG
                  3010XC2
                  0,42 @*)
            end;
            (* A 'REALIZATION' is a implementation of a 'SPEC.' *)
            (* which fulfills a given set of properties like    *)
            (* needed hardware platforms, operating system, ... *)
            (* @CC_LG 6007(60,294)(97,29) @*)

            node_class COMPLEX is_a UNIT (* @CC_LG 3005C @*)
               redef_derived
                  CodeSize = addSize ( 0, all self.-ccontains-> );
               (* The 'Size' of a complex unit is the sum of *)
               (* the 'Size' of all its children.            *)
            end;
            (* @CC_LG 6007(420,294)(97,29) @*)

            node_class ATOM is_a UNIT (* @CC_LG 3005C @*)
               derived
                  FileName (* @CC_LG 1002TC(0,8)(24,10) @*) : string = self.Name;
                  (* The 'string' is either the name of an externally *)
                  (* stored file or should be replaced by an imported *)
                  (* externally defined type 'file'.                  *)
                  (* @CC_LG
                  3012XC2
                  0,42 @*)
               redef_derived
                  CodeSize = length ( self.FileName );
               (* 'Size' is the length of the 'File string'.    *)
               (* It should be replaced by an imported function *)
               (* 'wordCount: file -> integer' later on.        *)
            end;
            (* @CC_LG 6007(612,294)(97,29) @*)

            edge_type ccontains : COMPLEX -> ATOM;
            (* @CC_LG 3004C @*)

            node_class SYSTEM is_a SPECIFICATION (* @CC_LG 3005C @*), COMPLEX (* @CC_LG 3005C @*)
               redef_derived
                  CodeSize = addSize ( 0, all self.-ccontains-> );
               (* Resolves inheritance conflict of attribute   *)
               (* definitions in 'SPECIFICATION' and 'COMPLEX' *)
               (* by prefering definition in 'SPECIFICATION'.  *)
            end;
            (* @CC_LG 6007(60,182)(97,29) @*)

            node_class CONFIGURATION is_a REALIZATION (* @CC_LG 3005C @*), COMPLEX (* @CC_LG 3005C @*)
            end;
            (* A 'CONFIGURATION' is a set of variants of module    *)
            (* realizations which fulfill all required properties. *)
            (* @CC_LG 6007(252,182)(97,29) @*)

            node_class MODULE is_a SPECIFICATION (* @CC_LG 3005C @*), ATOM (* @CC_LG 3005C @*)
               redef_derived
                  CodeSize = length ( self.FileName ) + max ( 0, all self.-has->.CodeSize );
               (* Resolves inheritance conflict of attribute    *)
               (* definitions in 'SPECIFICATION' and 'COMPLEX'  *)
               (* by building the sum of both definitions which *)
               (* are in conflict to each other.                *)
            end;
            (* A 'SYSTEM' contains a set of 'MODULES' which have   *)
            (* 'VARIANTS' as their realizations.                   *)
            (* @CC_LG 6007(612,182)(97,29) @*)

            node_class VARIANT is_a REALIZATION (* @CC_LG 3005C @*), ATOM (* @CC_LG 3005C @*)
            end;
            (* @CC_LG 6007(420,182)(97,29) @*)

            edge_type v_uses : VARIANT -> MODULE;
            (* @CC_LG 3004C @*)

            edge_type m_uses (* @CC_LG 1002C,TC(0,10)(36,10) @*) : MODULE -> MODULE;
            (* @CC_LG
            3004C5
            636,224
            660,224
            672,210 @*)

            node_type ADTM : MODULE
               (<OptAttDeclList>)
               (* @CC_LG
               3005C3
               12,56 @*)
               (* @CC_LG
               3005C3
               12,42 @*)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005 @*)
            end;
            (* An Abstract Data Type module. *)
            (* @CC_LG 6013(540,70)(91,35) @*)

            node_type ADOM : MODULE
               (<OptAttDeclList>)
               (* @CC_LG
               3005C3
               0,56 @*)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005C @*)
            end;
            (* An Abstract Data Object module. *)
            (* @CC_LG 6013(696,70)(97,29) @*)

            node_type FM : MODULE
               (<OptAttDeclList>)
               (* @CC_LG 3005C @*)
               (* @CC_LG
               3005C3
               0,42 @*)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005 @*)
            end;
            (* @CC_LG 6013(612,112)(91,35) @*)

            node_type Variant : VARIANT
               (<OptAttDeclList>)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005 @*)
            end;
            (* A 'Variant' is the implementation of a module.  *)
            (* The distinction of different types of variants  *)
            (* is omitted for keeping the specification short. *)
            (* @CC_LG 6013(420,70)(91,35) @*)

            node_type System : SYSTEM
               (<OptAttDeclList>)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005C @*)
            end;
            (* As long as subsystems are not supported the  *)
            (* distinction of different types of systems is *)
            (* not necessary.                               *)
            (* @CC_LG 6013(60,70)(91,35) @*)

            node_type Config : CONFIGURATION
               (<OptAttDeclList>)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005C @*)
               (* @CC_LG 3005 @*)
            end;
            (* A 'Config' is the "implementation" of a system *)
            (* as a consistent configuration with a set of    *)
            (* required properties.                           *)
            (* The configuration contains a variant of the    *)
            (* uniquely defined root module and variants of   *)
            (* all (in-)directly needed modules for realizing *)
            (* the selected root module variant.              *)
            (* All selected variants fulfill at least the     *)
            (* required configuration properties.             *)
            (* @CC_LG 6013(252,70)(97,29) @*)

            edge_type has : SPECIFICATION -> REALIZATION;
            (* @CC_LG 3004C @*)

            restriction name( UnitName : string)
                            (* @CC_LG 4005C,TC(-8,8)(114,12) @*) : UNIT =
               valid (self.Name = UnitName)
            end;
            (* Valid for a 'UNIT' which has the 'Name' *)
            (* 'UnitName'.                             *)
            (* @CC_LG
            3009XC2
            336,462 @*)

            path needs (* @CC_LG 1002C,TC(0,10)(30,10) @*) : ATOM -> MODULE =
               (     (    instance_of VARIANT
                       &
                       (     -v_uses->
                             (* @CC_LT SetColumn 4 @*)
                         or
                         (    <-has-
                           &  instance_of MODULE
                           &  -m_uses->          )
                         (* @CC_LT Vertical @*)        )
                       (* @CC_LT Vertical @*)            )
                 or  (    instance_of MODULE
                       &  -m_uses->          )             )
            end;
            (* Returns for a 'VARIANT'               *)
            (* 1) all its directly used modules and  *)
            (* 2) all used modules of its own module *)
            (* and                                   *)
            (* returns for a 'MODULE'                *)
            (*    all its directly used modules.     *)
            (* @CC_LG
            3006C4
            696,294
            696,182 @*)

            restriction used : MODULE =
               `1 in
               begin
                  obl_node `1 : MODULE;
                  (* @CC_LG 6007(48,28)(83,21) @*)
                  obl_node `2 : ATOM;
                  (* @CC_LG 6007(180,28)(71,21) @*)
                  `2 => `1 : needs;
                  (* @CC_LG 3006C @*)
               end
               (* @CC_LG 7008(8,8)(231,47) @*)
            end;
            (* @CC_LG
            3009XC2
            576,224 @*)

      end;
      (* @CC_LG 7013(36,56)(757,488) @*)

      section EditOperations

         declares

            transformation InitSystem ( SName : string) =

               begin
                  not_node `1 : System;
                  (* @CC_LG 6013(96,56)(97,29) @*)
               end
               (* @CC_LG 7008(8,8)(157,91) @*)
               ::=
               begin
                  1' : System;
                  (* @CC_LG 6012(96,56)(97,29) @*)
               end
               (* @CC_LG 7009(8,8)(157,91) @*)
               transfer 1'.Name := SName;
            end;
            (* Creates the initial 'System' node of a software *)
            (* design. A Module Interconnection Language graph *)
            (* (MIL graph) may not contain more than one       *)
            (* 'System' node.                                  *)

            transformation CreateModule ( MName : string ; MType : type_in MODULE ; out M :    MODULE (* @CC_LT VertWithIndent @*))
               =

               begin
                  obl_node `1 : System;
                  (* @CC_LG 6007(89,150)(83,21) @*)
                  not_node `2 : MODULE;
                  (* @CC_LG 6013(96,56)(97,29) @*)
                  `1 -> `2 : ccontains;
                  (* @CC_LG 3004C @*)
                  => `2 : name ( MName );
                  (* @CC_LG
                  3011XC2
                  218,108 @*)
               end
               (* @CC_LG 7008(8,8)(255,181) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(101,150)(59,21) @*)
                  2' : MType;
                  (* @CC_LG 6012(96,56)(97,29) @*)
                  1' -> 2' : ccontains;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(243,181) @*)
               transfer 2'.Name := MName;
               return M := 2';
            end;
            (* Creates a new module of the given type 'MType' *)
            (* if the system does not already contain another *)
            (* module with the same name 'MName'.             *)

            transformation RenameModule ( Module : MODULE ; NewName : string) =

               begin
                  obl_node `1 : System;
                  (* @CC_LG 6007(232,150)(83,21) @*)
                  not_node `2 : MODULE;
                  (* @CC_LG 6013(60,154)(97,21) @*)
                  `1 -> `2 : ccontains;
                  (* @CC_LG 3004C @*)
                  => `2 : name ( NewName );
                  (* @CC_LG
                  3011XC2
                  182,204 @*)
                  obl_node `3 = Module;
                  (* @CC_LG 6007(232,28)(83,21) @*)
                  `1 -> `3 : ccontains;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(303,223) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(36,42)(59,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(192,42)(59,21) @*)
                  1' -> 3' : ccontains (* @CC_LG 1002C,BC(0,-10)(48,10) @*);
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(231,61) @*)
               transfer 3'.Name := NewName;
            end;
            (* Creates a new module of the given type 'MType' *)
            (* if the system does not already contain another *)
            (* module with the same name 'MName'.             *)

            transformation DeleteModule ( M : MODULE) =

               begin
                  obl_node `1 = M;
                  (* @CC_LG 6007(94,150)(83,21) @*)
                  opt_set `2 : VARIANT;
                  (* @CC_LG 6010(96,56)(97,29) @*)
                  `1 -> `2 : has;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(162,181) @*)
               ::=
               (<OptRightSideList>)
               (* @CC_LG 7009(8,8)(27,33) @*)
            end;
            (* Deletes a module 'MName' with all its 'VARIANTS'. *)

            transformation ChangeModuleType ( M : MODULE ; NewMType : type_in MODULE ; out NewM :    MODULE (* @CC_LT VertWithIndent @*))
               =

               begin
                  obl_node `1 = M;
                  (* @CC_LG 6007(36,28)(53,21) @*)
               end
               (* @CC_LG 7008(8,8)(75,47) @*)
               ::=
               begin
                  1' : NewMType;
                  (* @CC_LG 6012(72,28)(97,21) @*)
               end
               (* @CC_LG 7009(8,8)(135,61) @*)
               embedding
                  redirect -has->, -m_uses->, <-v_uses-
                                              (* @CC_LT Vertical @*),
                           <-m_uses-
                           (* @CC_LT HoriWithSpace @*),
                           <-ccontains-
                           (* @CC_LT HoriWithSpace @*)                from `1 to 1';
                  (* @CC_LT VertWithIndent @*)
               transfer 1'.Name := `1.Name;
               return NewM := 1';
            end;
            (* Changes the type of a module but preserves all *)
            (* its realization variants.                      *)

            transformation CreateVariant ( M : MODULE ; VName : string ; PSet : string [0:n] ; out V : VARIANT)
               =

               begin
                  obl_node `1 = M;
                  (* @CC_LG 6007(60,122)(77,21) @*)
                  not_node `2 : Variant;
                  (* @CC_LG 6013(60,28)(97,21) @*)
                  => `2 : name ( VName );
                  (* @CC_LG
                  3011XC2
                  182,78 @*)
                  `1 -> `2 : has;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(291,153) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(96,150)(59,21) @*)
                  2' : Variant;
                  (* @CC_LG 6012(96,56)(97,29) @*)
                  1' -> 2' : has;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(157,181) @*)
               transfer 2'.Name := VName;
                        2'.Props := PSet;
               return V := 2';
            end;
            (* Creates a new module variant if a variant with *)
            (* the same name 'VName' of the module 'MName' is *)
            (* not already existent.                          *)

            transformation DeleteVariant ( V : VARIANT) =

               begin
                  obl_node `1 : System;
                  (* @CC_LG 6007(60,28)(83,21) @*)
                  not_node `2 : Config;
                  (* @CC_LG 6013(60,98)(97,29) @*)
                  obl_node `3 : MODULE;
                  (* @CC_LG 6007(216,28)(83,21) @*)
                  obl_node `4 = V;
                  (* @CC_LG 6007(216,98)(89,21) @*)
                  `1 -> `3 : ccontains;
                  (* @CC_LG 3004C @*)
                  `2 -> `4 : ccontains;
                  (* @CC_LG 3004C @*)
                  `1 -> `2 : has;
                  (* @CC_LG 3004C @*)
                  `3 -> `4 : has;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(267,117) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(36,28)(59,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(204,28)(59,21) @*)
                  1' -> 3' : ccontains (* @CC_LG 1002C,BC(0,-10)(48,10) @*);
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(243,47) @*)
            end;
            (* Deletes a module variant if this variant is not *)
            (* already part of a system configuration.         *)

            transformation CreateMUse ( ClientM, ServerM : MODULE) =

               begin
                  obl_node `1 = ClientM;
                  (* @CC_LG 6007(48,28)(83,21) @*)
                  obl_node `2 = ServerM;
                  (* @CC_LG 6007(192,28)(83,21) @*)
                  `2 #> `1 : needs *;
                  (* @CC_LG 3007C @*)
               end
               (* @CC_LG 7008(8,8)(243,61) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(36,28)(59,21) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(180,28)(59,21) @*)
                  1' -> 2' : m_uses;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(219,47) @*)
            end;
            (* Creates a new import relationship 'm_uses' from a  *)
            (* module 'ClientMName' (interface) to another module *)
            (* 'ServerMName'. The 'ServerMName' module may not    *)
            (* use the (directly or indirectly) the 'ClientMName' *)
            (* module (cyclic import is not allowed.              *)

            transformation DeleteMUse ( ClientM, ServerM : MODULE) =

               begin
                  obl_node `1 = ClientM;
                  (* @CC_LG 6007(48,28)(83,21) @*)
                  obl_node `2 = ServerM;
                  (* @CC_LG 6007(192,28)(83,21) @*)
                  `1 -> `2 : m_uses;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(243,47) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(36,28)(59,21) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(180,28)(59,21) @*)
               end
               (* @CC_LG 7009(8,8)(219,47) @*)
            end;
            (* Removes a 'm_uses' import relationship. *)

            transformation CreateVUse ( ClientV : VARIANT ; ServerM : MODULE) =

               begin
                  obl_node `1 : MODULE;
                  (* @CC_LG 6007(60,28)(83,21) @*)
                  obl_node `2 = ClientV;
                  (* @CC_LG 6007(60,98)(89,21) @*)
                  obl_node `3 = ServerM;
                  (* @CC_LG 6007(204,28)(89,21) @*)
                  `1 +> `3 : m_uses;
                  (* @CC_LG 3008C @*)
                  `1 -> `2 : has;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(255,117) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(72,28)(59,21) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(72,98)(59,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(216,28)(59,21) @*)
                  2' -> 3' : v_uses;
                  (* @CC_LG 3004C @*)
                  1' -> 2' : has;
                  (* @CC_LG 3004 @*)
               end
               (* @CC_LG 7009(8,8)(255,117) @*)
            end;
            (* Creates an import from an implementation variant   *)
            (* 'ClientVName' of the module 'ClientMName' to the   *)
            (* module 'ServerMName' if a parallel 'm_uses' import *)
            (* relationship from the module (interface) 'Client-  *)
            (* MName' to 'ServerMName' is not already existent.   *)

            transformation DeleteVUse ( ClientV : VARIANT ; ServerM : MODULE) =

               begin
                  obl_node `2 = ClientV;
                  (* @CC_LG 6007(60,28)(89,21) @*)
                  obl_node `3 = ServerM;
                  (* @CC_LG 6007(204,28)(89,21) @*)
                  `2 -> `3 : v_uses;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(267,47) @*)
               ::=
               begin
                  obl_node 2' = `2;
                  (* @CC_LG 6007(36,28)(59,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(180,28)(59,21) @*)
               end
               (* @CC_LG 7009(8,8)(219,47) @*)
            end;
            (* The inverse operation to 'CreateVUse'. *)

      end;
      (* @CC_LG 7013(816,336)(152,42) @*)

      section ConfigOperations

         declares

            transformation CreateConfig ( CName : string ; CProps : string [0:n]) =

               use ReqProps := CProps;
                   NewProps : string [0:n];
                   C : CONFIGURATION
               do
                  InitConfig ( CName, ReqProps, out NewProps, out C )
                  & ReqProps := NewProps
                  & loop
                       ResolveImport ( C, ReqProps, out NewProps )
                       & ReqProps := NewProps
                    end
                  & not UnresolvedImport ( C )
               end
            end;
            (* Creates a new configuration with a required set of *)
            (* properties.                                        *)

      end;
      (* @CC_LG 7013(816,280)(164,42) @*)

      section Configurations

         declares

            transformation InitConfig ( CName : string ; ReqProps : string [0:n] ; out NewProps : string [0:n] ; out C : CONFIGURATION)
               =

               begin
                  obl_node `1 : System;
                  (* @CC_LG 6007(161,320)(83,21) @*)
                  not_node `2 : Config;
                  (* @CC_LG 6013(240,56)(97,29) @*)
                  => `2 : name ( CName );
                  (* @CC_LG
                  3011XC2
                  362,108 @*)
                  obl_node `3 : MODULE;
                  (* @CC_LG 6007(161,230)(83,21) @*)
                  obl_node `4 : VARIANT;
                  (* @CC_LG 6007(164,140)(89,21) @*)
                  `1 -> `3 : ccontains;
                  (* @CC_LG
                  3004C3
                  168,280 @*)
                  => `3 : not used;
                  (* @CC_LG
                  3011XC2
                  175,254 @*)
                  not_node `5 : MODULE;
                  (* @CC_LG 6013(96,56)(97,29) @*)
                  `1 -> `5 : ccontains (* @CC_LG 1002C,TC(0,10)(48,10) @*);
                  (* @CC_LG
                  3004C8
                  192,280
                  192,252
                  240,196
                  192,168
                  168,140
                  96,112 @*)
                  => `5 : not used;
                  (* @CC_LG
                  3011XC2
                  218,108 @*)
                  `1 -> `2 : has;
                  (* @CC_LG
                  3004C6
                  144,280
                  144,252
                  96,196
                  240,112 @*)
                  `3 -> `4 : has;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(303,351) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(185,216)(59,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(257,118)(59,21) @*)
                  obl_node 4' = `4;
                  (* @CC_LG 6007(185,28)(59,21) @*)
                  1' -> 3' : ccontains (* @CC_LG 1002C,BC(0,-10)(48,10) @*);
                  (* @CC_LG 3004C @*)
                  2' : Config;
                  (* @CC_LG 6012(132,122)(97,21) @*)
                  2' -> 4' : ccontains;
                  (* @CC_LG 3004C @*)
                  1' -> 2' : has;
                  (* @CC_LG 3004C @*)
                  3' -> 4' : has;
                  (* @CC_LG
                  3004C3
                  228,70 @*)
               end
               (* @CC_LG 7009(8,8)(359,247) @*)
               condition `4.Props `are_in' ReqProps;
               transfer 2'.Name := CName;
                        2'.Props := ReqProps;
               return NewProps := merge ( ReqProps, 4'.Props );
                      C := 2';
            end;
            (* Creates an intial configuration state which contains *)
            (* 1) a new 'Config' node with 'ReqProps' and 'CName'   *)
            (* 2) a 'Variant' of the uniquely defined root module   *)
            (*    (the only module which is not imported by another *)
            (*     module).                                         *)
            (* The selected 'Variant' has to fulfill 'ReqProps' and *)
            (* the configuration process continues with a may be    *)
            (* more restrictive set of properties 'NewProps'.       *)

            transformation ResolveImport ( C : CONFIGURATION ; ReqProps : string [0:n] ; out NewProps : string [0:n])
               =

               begin
                  obl_node `1 = C;
                  (* @CC_LG 6007(173,340)(83,21) @*)
                  obl_node `2 : VARIANT;
                  (* @CC_LG 6007(200,70)(89,21) @*)
                  obl_node `3 : MODULE;
                  (* @CC_LG 6007(221,160)(83,21) @*)
                  obl_node `4 : VARIANT;
                  (* @CC_LG 6007(224,250)(89,21) @*)
                  `1 +> `2 : ccontains;
                  (* @CC_LG
                  3010C5
                  156,280
                  156,140
                  180,112 @*)
                  not_node `5 : VARIANT;
                  (* @CC_LG 6013(180,28)(97,21) @*)
                  `1 -> `5 : ccontains;
                  (* @CC_LG
                  3004C6
                  132,280
                  132,140
                  156,112
                  156,84 @*)
                  `1 -> `4 : ccontains (* @CC_LG 1002C,TC(0,10)(48,10) @*);
                  (* @CC_LG 3004C @*)
                  `4 => `3 : needs;
                  (* @CC_LG 3008C @*)
                  `3 -> `2 : has;
                  (* @CC_LG
                  3004C3
                  228,112 @*)
                  `3 -> `5 : has (* @CC_LG 1002C,BC(0,-2)(18,10) @*);
                  (* @CC_LG
                  3004C4
                  204,112
                  180,84 @*)
               end
               (* @CC_LG 7008(8,8)(341,371) @*)
               ::=
               begin
                  obl_node 1' = `1;
                  (* @CC_LG 6007(36,98)(59,21) @*)
                  obl_node 2' = `2;
                  (* @CC_LG 6007(192,98)(59,21) @*)
                  obl_node 3' = `3;
                  (* @CC_LG 6007(192,28)(59,21) @*)
                  obl_node 4' = `4;
                  (* @CC_LG 6007(36,28)(59,21) @*)
                  1' -> 4' : ccontains (* @CC_LG 1002C,TC(0,10)(48,10) @*);
                  (* @CC_LG 3004C @*)
                  1' -> 2' : ccontains;
                  (* @CC_LG 3004C @*)
                  3' -> 2' : has;
                  (* @CC_LG 3004 @*)
               end
               (* @CC_LG 7009(8,8)(231,117) @*)
               condition `2.Props `are_in' ReqProps;
               return NewProps := merge ( ReqProps, 2'.Props );
            end;
            (* Selects a module which is used by a 'Variant' in    *)
            (* the configuration if no 'Variant' of this module    *)
            (* is part of the configuration. Furthermore, it tries *)
            (* to add a 'Variant' of the selected module to the    *)
            (* configuration which fulfills 'ReqProps'.            *)

            query UnresolvedImport( C : CONFIGURATION) =
               begin
                  obl_node `1 = C;
                  (* @CC_LG 6007(137,330)(83,21) @*)
                  obl_node `3 : MODULE;
                  (* @CC_LG 6007(161,150)(83,21) @*)
                  obl_node `4 : VARIANT;
                  (* @CC_LG 6007(164,240)(89,21) @*)
                  not_node `5 : VARIANT;
                  (* @CC_LG 6013(144,56)(97,29) @*)
                  `1 -> `5 : ccontains;
                  (* @CC_LG
                  3004C6
                  120,280
                  96,252
                  96,140
                  120,112 @*)
                  `1 -> `4 : ccontains (* @CC_LG 1002C,TC(0,10)(48,10) @*);
                  (* @CC_LG
                  3004C3
                  144,280 @*)
                  `4 => `3 : needs;
                  (* @CC_LG
                  3008C3
                  168,196 @*)
                  `3 -> `5 : has;
                  (* @CC_LG
                  3004C3
                  144,112 @*)
               end
               (* @CC_LG 7008(8,8)(279,361) @*)
            end;
            (* Tests whether all imports of variants in the  *)
            (* configuration 'CName' are resolved, i.e. have *)
            (* modules as their targets which have a variant *)
            (* which is also part of the configuration.      *)

            transformation DeleteConfig ( C : CONFIGURATION) =

               begin
                  obl_node `1 = C;
                  (* @CC_LG 6007(36,27)(53,21) @*)
               end
               (* @CC_LG 7008(8,8)(75,47) @*)
               ::=
               (<OptRightSideList>)
               (* @CC_LG 7009(8,8)(27,33) @*)
            end;

      end;
      (* @CC_LG 7013(816,392)(152,42) @*)

      section Main

         declares

            transformation MAIN =

               use Main, UI, Files : MODULE;
                   V, V_X11_SunOS, V_SunOS, V_Win95 : VARIANT
               do
                  InitSystem ( "Example" )
                  & CreateModule ( "Main", FM, out Main )
                  & CreateModule ( "UI", ADOM, out UI )
                  & CreateModule ( "Files", ADTM, out Files )
                  & CreateMUse ( UI, Files )
                  & CreateVariant ( Main, "Main", nil, out V )
                  & CreateVUse ( V, UI )
                  & CreateVUse ( V, Files )
                  & CreateVariant ( UI, "UI_X11_SunOS", merge ( mkProp ( "WS", "X" ), mkProp ( "OS", "Unix" ) ), out V_X11_SunOS
                                  )
                  & CreateVariant ( Files, "Files_SunOS", mkProp ( "OS", "Unix" ), out V_SunOS )
                  & CreateVariant ( Files, "Files_Win95", mkProp ( "OS", "MSDos" ), out V_Win95 )
               end
            end;

      end;
      (* @CC_LG 7013(816,504)(84,30) @*)

end.
(* @CC_LG 7012(8,8)(1001,556) @*)
