spec MedicalDiagnosis

   declares

      section BasicScheme

         declares

            node_class ITEM
               intrinsic
                  index Name : string := string ( self.type );
               derived
                  index GRAS_BROWSER_LABEL : string = self.Name;
            end;

            node_class LIST end;
            (* @CC_LG 6007(240,336)(97,29) @*)

            node_class LIST_ELEM
               derived
                  No : integer = [ self.<-lnext-.No
                                 | 1 ]             ;
            end;
            (* @CC_LG 6007(228,154)(97,29) @*)

            edge_type lelem : LIST -> LIST_ELEM [1:1];

            edge_type llast : LIST [0:1] -> LIST_ELEM [0:1];

            edge_type lnext : LIST_ELEM [0:1] -> LIST_ELEM [0:1];

            transformation CreateItem ( nodeType : type_in ITEM) =
               (<OptPrecondDecl>)
               (* @CC_LG 7008(8,8)(27,33) @*)

               ::=
               begin
                  1' : nodeType;
                  (* @CC_LG 6012(96,56)(97,29) @*)
               end
               (* @CC_LG 7009(8,8)(157,91) @*)
            end;

            section WorkingData

               declares

                  node_class PRACTICE is_a ITEM end;

                  edge_type selem : PRACTICE -> PERSON;

                  node_class PERSON is_a ITEM end;

                  node_class DOCTOR is_a PERSON end;

                  edge_type does : DOCTOR [1:1] -> TREATMENT;

                  node_class PATIENT is_a PERSON, LIST end;

                  node_class BILL
                     derived
                        CostSum : integer;
                  end;

                  node_class TREATMENT is_a LIST_ELEM, BILL
                     derived
                        Date : integer;
                     redef_derived
                        CostSum = self.toDiagnosis.CostSum + self.toPrescription.CostSum;
                  end;

                  edge_type toDiagnosis : TREATMENT [1:1] -> DIAGNOSIS [1:1];

                  edge_type toPrescription : TREATMENT [1:1] -> PRESCRIPTION [1:1];

                  node_class DIAGNOSIS is_a BILL
                     redef_derived
                        CostSum = AddCosts ( 0, all self.=Clear=> );
                  end;

                  function AddCosts : ( sum : integer ; aNode : BILL_POSITION) -> integer =
                     sum + aNode.Costs
                  end;

                  node_class BILL_POSITION
                     meta
                        Costs : integer := 0;
                  end;

                  edge_type yes : DIAGNOSIS -> EVIDENCE;

                  edge_type no : DIAGNOSIS -> EVIDENCE;

                  node_class PRESCRIPTION is_a BILL end;

                  edge_type gets : PRESCRIPTION -> THERAPY;

                  section WorkingTypes

                     declares

                        node_type LocalPractice : PRACTICE end;

                        node_type Treatment : TREATMENT end;

                        node_type Diagnosis : DIAGNOSIS end;

                        node_type Prescription : PRESCRIPTION end;

                        node_type Doctor : DOCTOR end;

                        node_type Patient : PATIENT end;

                  end;

            end;

            section Knowledge

               declares

                  node_class EVIDENCE is_a ITEM, BILL_POSITION end;
                  (* @CC_LG 6007(120,84)(97,29) @*)

                  node_type Evidence : EVIDENCE end;
                  (* @CC_LG 6013(96,336)(61,43) @*)

                  node_class DISEASE is_a EVIDENCE end;

                  node_type Disease : DISEASE end;

                  node_class SYMPTOM is_a EVIDENCE end;

                  node_type Symptom : SYMPTOM end;

                  node_class EXAMINATION is_a EVIDENCE end;

                  node_type Examination : EXAMINATION end;

                  edge_type pro : EVIDENCE -> EVIDENCE;

                  transformation AddProEdge ( kind, target : string) =

                     begin
                        obl_node `1 : EVIDENCE;
                        (* @CC_LG 6007(60,28)(95,21) @*)
                        obl_node `2 : EVIDENCE;
                        (* @CC_LG 6007(228,28)(95,21) @*)
                     end
                     (* @CC_LG 7008(8,8)(291,47) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(60,28)(95,19) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(228,28)(95,19) @*)
                        1' -> 2' : pro;
                        (* @CC_LG 3004 @*)
                     end
                     (* @CC_LG 7009(8,8)(291,47) @*)
                     condition `1.Name = kind;
                               `2.Name = target;
                  end;

                  edge_type contra : EVIDENCE -> EVIDENCE;

                  transformation AddContraEdge ( kind, target : string) =

                     begin
                        obl_node `1 : EVIDENCE;
                        (* @CC_LG 6007(60,28)(95,21) @*)
                        obl_node `2 : EVIDENCE;
                        (* @CC_LG 6007(228,28)(95,21) @*)
                     end
                     (* @CC_LG 7008(8,8)(291,47) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(60,28)(95,19) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(228,28)(95,19) @*)
                        1' -> 2' : contra;
                        (* @CC_LG 3004 @*)
                     end
                     (* @CC_LG 7009(8,8)(291,47) @*)
                     condition `1.Name = kind;
                               `2.Name = target;
                  end;

                  node_class THERAPY is_a ITEM end;

                  node_type Therapy : THERAPY end;

                  edge_type needs : EVIDENCE -> THERAPY;

                  transformation AddNeedsEdge ( kind, target : string) =

                     begin
                        obl_node `1 : EVIDENCE;
                        (* @CC_LG 6007(60,28)(95,21) @*)
                        obl_node `2 : THERAPY;
                        (* @CC_LG 6007(228,28)(95,21) @*)
                     end
                     (* @CC_LG 7008(8,8)(291,47) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(60,28)(95,19) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(228,28)(95,19) @*)
                        1' -> 2' : needs;
                        (* @CC_LG 3004 @*)
                     end
                     (* @CC_LG 7009(8,8)(291,47) @*)
                     condition `1.Name = kind;
                               `2.Name = target;
                  end;

                  edge_type excludes : EVIDENCE -> THERAPY;

                  transformation AddExcludesEdge ( kind, target : string) =

                     begin
                        obl_node `1 : EVIDENCE;
                        (* @CC_LG 6007(60,28)(95,21) @*)
                        obl_node `2 : THERAPY;
                        (* @CC_LG 6007(228,28)(89,21) @*)
                     end
                     (* @CC_LG 7008(8,8)(291,47) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(60,28)(95,19) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(228,28)(95,19) @*)
                        1' -> 2' : excludes;
                        (* @CC_LG 3004 @*)
                     end
                     (* @CC_LG 7009(8,8)(291,47) @*)
                     condition `1.Name = kind;
                               `2.Name = target;
                  end;

                  section Base

                     declares

                        node_type HasCough : SYMPTOM end;
                        (* @CC_LG 6013(60,364)(61,43) @*)

                        node_type HighAnitHistamenica : SYMPTOM end;
                        (* @CC_LG 6013(120,364)(61,43) @*)

                        node_type IsChild : SYMPTOM end;
                        (* @CC_LG 6013(168,350)(61,43) @*)

                        node_type PseudoCroup : SYMPTOM end;
                        (* @CC_LG 6013(204,364)(61,43) @*)

                        node_type Stomachache : SYMPTOM end;
                        (* @CC_LG 6013(228,322)(61,43) @*)

                        node_type InHypogastrium : SYMPTOM end;

                        node_type InEpigastrum : SYMPTOM end;

                        node_type Diarrhoe : DISEASE end;

                        node_type Catarrh : DISEASE end;

                        node_type Influenza : DISEASE end;

                        node_type AllergicTest : EVIDENCE end;

                        node_type HayFever : DISEASE end;

                        node_type AvoidPollen : THERAPY end;

                        node_type IsSmoker : EVIDENCE end;

                        node_type Alcoholic : DISEASE end;

                        node_type Fibrinolyse : THERAPY end;

                        node_type Makoma : THERAPY end;

                        node_type Histaminika : THERAPY end;

                        node_type NoCalorieDiet : THERAPY end;

                  end;

            end;

      end;

      section Operations

         declares

            section BuildingOperations

               declares

                  section BuildKnowledge

                     declares

                        transformation AddKnowledge =

                           AddTonsilitis
                           & AddMorbusPfeifer
                        end;

                        transformation AddMoreKnowledge =

                           AddCough
                           & AddAsthma
                           & AddCardiacInfarction
                        end;

                        section Tonsilitis_Knowledge

                           declares

                              node_type AntiPyretic : THERAPY end;
                              (* @CC_LG 6013(144,294)(61,43) @*)

                              node_type SwollenTonsils : SYMPTOM end;
                              (* @CC_LG 6013(60,56)(61,43) @*)

                              node_type SoreThroat : SYMPTOM end;
                              (* @CC_LG 6013(144,56)(61,43) @*)

                              node_type Croup : SYMPTOM end;
                              (* @CC_LG 6013(228,56)(61,43) @*)

                              node_type Tonsilitis : SYMPTOM end;
                              (* @CC_LG 6013(60,112)(61,43) @*)

                              node_type Diphteria : SYMPTOM end;

                              node_type HerpesSimplex : SYMPTOM end;

                              node_type Cough : DISEASE end;

                              node_type TakeTemperature : EXAMINATION end;

                              node_type Fever : DISEASE end;

                              node_type Antibiotic : THERAPY end;

                              node_type BloodTest : EXAMINATION end;

                              node_type VariedBloodTest : SYMPTOM end;

                              transformation AddTonsilitis =
                                 (<OptPrecondDecl>)
                                 (* @CC_LG 7008(8,8)(291,33) @*)

                                 ::=
                                 begin
                                    1' : SoreThroat;
                                    (* @CC_LG 6012(336,448)(97,29) @*)
                                    2' : SwollenTonsils;
                                    (* @CC_LG 6012(336,350)(97,29) @*)
                                    1' -> 2' : pro;
                                    (* @CC_LG 3004C @*)
                                    3' : Fever;
                                    (* @CC_LG 6012(192,350)(97,29) @*)
                                    4' : Tonsilitis;
                                    (* @CC_LG 6012(312,252)(97,29) @*)
                                    2' -> 4' : pro;
                                    (* @CC_LG
                                    3004C3
                                    312,308 @*)
                                    3' -> 4' : pro (* @CC_LG 1002(-12,28)(18,10) @*);
                                    (* @CC_LG 3004MC @*)
                                    5' : VariedBloodTest;
                                    (* @CC_LG 6012(480,350)(97,29) @*)
                                    5' -> 4' : contra (* @CC_LG 1002C,LC(30,6)(36,10) @*);
                                    (* @CC_LG 3004C @*)
                                    6' : Croup;
                                    (* @CC_LG 6012(72,252)(97,29) @*)
                                    7' : Cough;
                                    (* @CC_LG 6012(48,350)(97,29) @*)
                                    7' -> 6' : pro;
                                    (* @CC_LG
                                    3004C3
                                    72,308 @*)
                                    8' : Diphteria;
                                    (* @CC_LG 6012(120,154)(97,29) @*)
                                    6' -> 8' : pro;
                                    (* @CC_LG 3004C @*)
                                    4' -> 8' : pro;
                                    (* @CC_LG 3004C @*)
                                    9' : HerpesSimplex;
                                    (* @CC_LG 6012(384,56)(97,29) @*)
                                    4' -> 9' : pro;
                                    (* @CC_LG
                                    3004C5
                                    336,196
                                    336,168
                                    384,112 @*)
                                    10' : Antibiotic;
                                    (* @CC_LG 6012(96,56)(97,29) @*)
                                    8' -> 10' : needs;
                                    (* @CC_LG
                                    3004C3
                                    120,112 @*)
                                    12' : AntiPyretic;
                                    (* @CC_LG 6012(240,56)(97,29) @*)
                                    3' -> 12' : needs;
                                    (* @CC_LG
                                    3004C6
                                    168,308
                                    168,224
                                    216,168
                                    216,112 @*)
                                 end
                                 (* @CC_LG 7009(8,8)(541,483) @*)
                              end;

                        end;
                        (* @CC_LG 7013(24,182)(291,435) @*)

                        section MorbusPfeifer_Knowledge

                           declares

                              node_type SwollenSpleen : SYMPTOM end;

                              node_type SwollenLiver : SYMPTOM end;

                              node_type SwollenLymphNodes : SYMPTOM end;

                              node_type MorbusPfeifer : SYMPTOM end;

                              transformation AddMorbusPfeifer =

                                 begin
                                    obl_node `1 : SwollenTonsils;
                                    (* @CC_LG 6007(84,28)(131,21) @*)
                                    obl_node `2 : VariedBloodTest;
                                    (* @CC_LG 6007(108,84)(155,21) @*)
                                    obl_node `3 : Antibiotic;
                                    (* @CC_LG 6007(144,126)(107,21) @*)
                                    obl_node `4 : Fever;
                                    (* @CC_LG 6007(204,28)(77,21) @*)
                                 end
                                 (* @CC_LG 7008(8,8)(351,145) @*)
                                 ::=
                                 begin
                                    obl_node 1' = `1;
                                    (* @CC_LG 6007(160,214)(57,21) @*)
                                    obl_node 2' = `2;
                                    (* @CC_LG 6007(64,214)(57,21) @*)
                                    obl_node 3' = `3;
                                    (* @CC_LG 6007(352,28)(57,21) @*)
                                    5' : MorbusPfeifer;
                                    (* @CC_LG 6012(372,121)(97,21) @*)
                                    5' -> 3' : excludes;
                                    (* @CC_LG 3004C @*)
                                    2' -> 5' : pro;
                                    (* @CC_LG 3004C @*)
                                    obl_node 4' = `4;
                                    (* @CC_LG 6007(688,214)(57,21) @*)
                                    4' -> 5' : pro;
                                    (* @CC_LG 3004C @*)
                                    1' -> 5' : pro;
                                    (* @CC_LG 3004C @*)
                                    6' : SwollenSpleen;
                                    (* @CC_LG 6012(276,219)(97,21) @*)
                                    6' -> 5' : pro;
                                    (* @CC_LG 3004C @*)
                                    7' : SwollenLiver;
                                    (* @CC_LG 6012(420,219)(97,21) @*)
                                    7' -> 5' : pro (* @CC_LG 1002C,TC(0,10)(18,10) @*);
                                    (* @CC_LG 3004C @*)
                                    8' : SwollenLymphNodes;
                                    (* @CC_LG 6012(564,219)(97,21) @*)
                                    8' -> 5' : pro (* @CC_LG 1002C,TC(-10,10)(18,10) @*);
                                    (* @CC_LG 3004C @*)
                                    9' : SuperSonics;
                                    (* @CC_LG 6012(348,317)(97,21) @*)
                                    9' -> 6' : pro (* @CC_LG 1002C,BC(-21,-1)(18,10) @*);
                                    (* @CC_LG 3004C @*)
                                    9' -> 7' : pro (* @CC_LG 1002C,TC(0,10)(18,10) @*);
                                    (* @CC_LG 3004M @*)
                                 end
                                 (* @CC_LG 7009(8,8)(741,352) @*)
                              end;

                        end;

                        node_type CoughsSputum : SYMPTOM end;

                        node_type InTheMorning : SYMPTOM end;

                        node_type Lasting3Month : SYMPTOM end;

                        node_type Smoker : DISEASE end;

                        node_type InhalesNoxiousStuff : SYMPTOM end;

                        node_type ChronicalBronchitis : DISEASE end;

                        node_type NoSmoking : THERAPY end;

                        node_type SeaClimate : THERAPY end;

                        node_type AvoidNoxiousStuff : THERAPY end;

                        transformation AddCough =

                           begin
                              obl_node `1 : Cough;
                              (* @CC_LG 6007(84,28)(77,21) @*)
                              obl_node `2 : Fever;
                              (* @CC_LG 6007(204,28)(77,21) @*)
                           end
                           (* @CC_LG 7008(8,8)(363,47) @*)
                           ::=
                           begin
                              obl_node 1' = `1;
                              (* @CC_LG 6007(220,443)(57,19) @*)
                              obl_node 2' = `2;
                              (* @CC_LG 6007(484,247)(57,19) @*)
                              3' : Lasting3Month;
                              (* @CC_LG 6012(72,252)(97,29) @*)
                              1' -> 3' : pro;
                              (* @CC_LG
                              3004C4
                              240,392
                              240,336 @*)
                              4' : CoughsSputum;
                              (* @CC_LG 6012(144,350)(97,29) @*)
                              1' -> 4' : pro;
                              (* @CC_LG
                              3004C3
                              192,392 @*)
                              5' : InTheMorning;
                              (* @CC_LG 6012(360,252)(97,29) @*)
                              4' -> 5' : pro;
                              (* @CC_LG 3004C @*)
                              6' : ChronicalBronchitis;
                              (* @CC_LG 6012(216,154)(97,29) @*)
                              5' -> 6' : pro (* @CC_LG 1002C,TC(-4,7)(18,10) @*);
                              (* @CC_LG 3004C @*)
                              7' : Smoker;
                              (* @CC_LG 6012(360,154)(97,29) @*)
                              2' -> 6' : contra (* @CC_LG 1002(-36,28)(36,10) @*);
                              (* @CC_LG 3004C @*)
                              8' : NoSmoking;
                              (* @CC_LG 6012(384,56)(97,29) @*)
                              6' -> 8' : needs (* @CC_LG 1002(23,4)(30,10) @*);
                              (* @CC_LG
                              3004C3
                              312,112 @*)
                              7' -> 8' : needs (* @CC_LG 1002C,TC(0,10)(30,10) @*);
                              (* @CC_LG
                              3004C3
                              360,112 @*)
                              9' : InhalesNoxiousStuff;
                              (* @CC_LG 6012(216,252)(97,29) @*)
                              9' -> 6' : pro;
                              (* @CC_LG 3004C @*)
                              10' : AvoidNoxiousStuff;
                              (* @CC_LG 6012(96,56)(97,29) @*)
                              6' -> 10' : needs (* @CC_LG 1002(-72,14)(30,10) @*);
                              (* @CC_LG 3004C @*)
                              11' : SeaClimate;
                              (* @CC_LG 6012(240,56)(97,29) @*)
                              6' -> 11' : needs (* @CC_LG 1002C,BC(10,-1)(30,10) @*);
                              (* @CC_LG
                              3004C3
                              216,112 @*)
                              3' -> 6' : pro (* @CC_LG 1002C,BC(-57,-10)(18,10) @*);
                              (* @CC_LG 3004C @*)
                           end
                           (* @CC_LG 7009(8,8)(525,473) @*)
                        end;

                        node_type HasAnhelations : SYMPTOM end;

                        node_type InAttacks : SYMPTOM end;

                        node_type LungRattles : SYMPTOM end;

                        node_type CongenitallyAfflicted : SYMPTOM end;

                        node_type IsAcute : SYMPTOM end;

                        node_type CompliansOk : SYMPTOM end;

                        node_type ResistanceOk : SYMPTOM end;

                        node_type LungBreatheStertorously : SYMPTOM end;

                        node_type HasAsthmaAttack : SYMPTOM end;

                        node_type Asthma : DISEASE end;

                        node_type HasAllergies : SYMPTOM end;

                        node_type Theophillin : THERAPY end;

                        node_type Fenotioral : THERAPY end;

                        node_type Cortison : THERAPY end;

                        node_type SecondaryEffects : DISEASE end;

                        transformation AddAsthma =

                           begin
                              obl_node `1 : ChronicalBronchitis;
                              (* @CC_LG 6007(180,28)(161,21) @*)
                           end
                           (* @CC_LG 7008(8,8)(399,47) @*)
                           ::=
                           begin
                              obl_node 1' = `1;
                              (* @CC_LG 6007(388,224)(57,19) @*)
                              2' : HasAnhelations;
                              (* @CC_LG 6012(120,350)(97,29) @*)
                              3' : IsAcute;
                              (* @CC_LG 6012(720,350)(97,29) @*)
                              4' : InAttacks;
                              (* @CC_LG 6012(120,252)(97,29) @*)
                              2' -> 4' : pro;
                              (* @CC_LG 3004C @*)
                              5' : HasAsthmaAttack;
                              (* @CC_LG 6012(696,252)(97,29) @*)
                              3' -> 5' : pro;
                              (* @CC_LG
                              3004C3
                              720,308 @*)
                              6' : Asthma;
                              (* @CC_LG 6012(408,154)(97,29) @*)
                              5' -> 6' : pro (* @CC_LG 1002(-24,4)(18,10) @*);
                              (* @CC_LG 3004MC @*)
                              4' -> 6' : pro (* @CC_LG 1002(5,5)(18,10) @*);
                              (* @CC_LG 3004C @*)
                              7' : HasAllergies;
                              (* @CC_LG 6012(528,252)(97,29) @*)
                              7' -> 6' : pro (* @CC_LG 1002C,BC(-36,-7)(18,10) @*);
                              (* @CC_LG
                              3004MC3
                              480,196 @*)
                              8' : CongenitallyAfflicted;
                              (* @CC_LG 6012(264,252)(97,29) @*)
                              8' -> 6' : pro (* @CC_LG 1002(-24,4)(18,10) @*);
                              (* @CC_LG
                              3004MC3
                              336,196 @*)
                              9' : CompliansOk;
                              (* @CC_LG 6012(336,317)(97,29) @*)
                              9' -> 6' : pro (* @CC_LG 1002(-48,0)(18,10) @*);
                              (* @CC_LG
                              3004C4
                              360,252
                              384,224 @*)
                              9' -> 1' : pro (* @CC_LG 1002(34,0)(18,10) @*);
                              (* @CC_LG 3004C @*)
                              10' : ResistanceOk;
                              (* @CC_LG 6012(192,317)(97,29) @*)
                              10' -> 6' : contra (* @CC_LG 1002C,LC(10,0)(36,10) @*);
                              (* @CC_LG
                              3004C4
                              384,252
                              408,224 @*)
                              10' -> 1' : contra (* @CC_LG 1002C,TC(-12,0)(36,10) @*);
                              (* @CC_LG 3004C @*)
                              11' : LungRattles;
                              (* @CC_LG 6012(480,317)(97,29) @*)
                              11' -> 1' : contra (* @CC_LG 1002(0,18)(36,10) @*);
                              (* @CC_LG 3004C @*)
                              12' : LungBreatheStertorously;
                              (* @CC_LG 6012(624,317)(97,29) @*)
                              12' -> 6' : pro;
                              (* @CC_LG
                              3004C4
                              456,252
                              456,224 @*)
                              12' -> 1' : contra;
                              (* @CC_LG 3004C @*)
                              13' : SecondaryEffects;
                              (* @CC_LG 6012(648,154)(97,29) @*)
                              14' : Cortison;
                              (* @CC_LG 6012(672,56)(97,29) @*)
                              13' -> 14' : excludes (* @CC_LG 1002C,BC(10,50)(48,10) @*);
                              (* @CC_LG
                              3004C3
                              648,112 @*)
                              6' -> 14' : needs (* @CC_LG 1002C,BC(-15,-10)(30,10) @*);
                              (* @CC_LG 3004C @*)
                              15' : Fenotioral;
                              (* @CC_LG 6012(96,56)(97,29) @*)
                              6' -> 15' : needs (* @CC_LG 1002C,BC(2,14)(30,10) @*);
                              (* @CC_LG 3004C @*)
                              16' : SeaClimate;
                              (* @CC_LG 6012(240,56)(97,29) @*)
                              17' : Theophillin;
                              (* @CC_LG 6012(528,56)(97,29) @*)
                              18' : Histaminika;
                              (* @CC_LG 6012(384,56)(97,29) @*)
                              6' -> 16' : needs (* @CC_LG 1002(-12,32)(30,10) @*);
                              (* @CC_LG
                              3004C3
                              312,112 @*)
                              6' -> 18' : needs (* @CC_LG 1002C,BC(-15,8)(30,10) @*);
                              (* @CC_LG
                              3004C3
                              408,112 @*)
                              6' -> 17' : needs (* @CC_LG 1002C,BC(-23,8)(30,10) @*);
                              (* @CC_LG 3004C @*)
                           end
                           (* @CC_LG 7009(8,8)(781,411) @*)
                        end;

                        section Stomach_Ache_Knowledge

                           declares

                              node_type StomachAche : SYMPTOM end;

                              node_type InUpperStomach : SYMPTOM end;

                              node_type InLowerStomach : SYMPTOM end;

                              node_type Diarrhoea : SYMPTOM end;

                              node_type ElectroCardioGramm : EXAMINATION end;

                              node_type ShowsAsphyxiaT : SYMPTOM end;

                              node_type CreatininkinaseFound : SYMPTOM end;

                              node_type CardiacInfarction : DISEASE end;

                              node_type LipaseAmylaseEnzymHigh : SYMPTOM end;

                              node_type TissueTypePlasminogenActivator : THERAPY end;

                              node_type Aspirin : THERAPY end;

                              transformation AddCardiacInfarction =
                                 (<OptPrecondDecl>)
                                 (* @CC_LG 7008(8,8)(291,33) @*)

                                 ::=
                                 begin
                                    1' : StomachAche;
                                    (* @CC_LG 6012(312,448)(97,29) @*)
                                    2' : InUpperStomach;
                                    (* @CC_LG 6012(168,350)(97,29) @*)
                                    3' : InLowerStomach;
                                    (* @CC_LG 6012(456,350)(97,29) @*)
                                    1' -> 2' : pro;
                                    (* @CC_LG 3004C @*)
                                    1' -> 3' : pro;
                                    (* @CC_LG 3004C @*)
                                    4' : Diarrhoea;
                                    (* @CC_LG 6012(456,252)(97,29) @*)
                                    3' -> 4' : pro;
                                    (* @CC_LG 3004C @*)
                                    2' -> 4' : pro;
                                    (* @CC_LG 3004C @*)
                                    5' : CardiacInfarction;
                                    (* @CC_LG 6012(384,154)(97,29) @*)
                                    4' -> 5' : contra (* @CC_LG 1002C,LC(10,0)(36,10) @*);
                                    (* @CC_LG
                                    3004C3
                                    432,196 @*)
                                    2' -> 5' : pro;
                                    (* @CC_LG
                                    3004C4
                                    192,280
                                    240,224 @*)
                                    6' : ElectroCardioGramm;
                                    (* @CC_LG 6012(312,350)(97,29) @*)
                                    7' : ShowsAsphyxiaT;
                                    (* @CC_LG 6012(312,252)(97,29) @*)
                                    6' -> 7' : pro;
                                    (* @CC_LG 3004C @*)
                                    7' -> 5' : pro;
                                    (* @CC_LG
                                    3004C3
                                    336,196 @*)
                                    8' : BloodTest;
                                    (* @CC_LG 6012(600,350)(97,29) @*)
                                    9' : CreatininkinaseFound;
                                    (* @CC_LG 6012(600,252)(97,29) @*)
                                    8' -> 9' : pro;
                                    (* @CC_LG 3004C @*)
                                    9' -> 5' : pro;
                                    (* @CC_LG 3004C @*)
                                    10' : Fibrinolyse;
                                    (* @CC_LG 6012(408,56)(97,29) @*)
                                    5' -> 10' : needs (* @CC_LG 1002C,BC(-21,-1)(30,10) @*);
                                    (* @CC_LG
                                    3004C3
                                    384,112 @*)
                                    11' : TissueTypePlasminogenActivator;
                                    (* @CC_LG 6012(264,56)(97,29) @*)
                                    5' -> 11' : needs (* @CC_LG 1002C,BC(-15,14)(30,10) @*);
                                    (* @CC_LG 3004C @*)
                                    12' : Aspirin;
                                    (* @CC_LG 6012(552,56)(97,29) @*)
                                    5' -> 12' : needs;
                                    (* @CC_LG
                                    3004C3
                                    480,112 @*)
                                 end
                                 (* @CC_LG 7009(8,8)(661,483) @*)
                              end;

                        end;

                        section Gastritis_Knowledge

                           declares

                              node_type Gastrosil : THERAPY end;

                              node_type Gastritis : SYMPTOM end;

                              node_type GastrosScopy : EXAMINATION end;

                              node_type ShowsInflammation : SYMPTOM end;

                              transformation AddGrastritis =

                                 begin
                                    obl_node `1 : Diarrhoea;
                                    (* @CC_LG 6007(132,28)(101,21) @*)
                                 end
                                 (* @CC_LG 7008(8,8)(291,47) @*)
                                 ::=
                                 begin
                                    obl_node 1' = `1;
                                    (* @CC_LG 6007(232,122)(57,21) @*)
                                    2' : GastrosScopy;
                                    (* @CC_LG 6012(108,224)(97,21) @*)
                                    3' : ShowsInflammation;
                                    (* @CC_LG 6012(108,126)(97,21) @*)
                                    2' -> 3' : pro;
                                    (* @CC_LG 3004C @*)
                                    4' : Gastritis;
                                    (* @CC_LG 6012(180,28)(97,21) @*)
                                    1' -> 4' : contra;
                                    (* @CC_LG
                                    3004C3
                                    204,84 @*)
                                    3' -> 4' : pro;
                                    (* @CC_LG
                                    3004C3
                                    156,84 @*)
                                 end
                                 (* @CC_LG 7009(8,8)(305,259) @*)
                              end;

                        end;

                        section Ulcer_Knowledge

                           declares

                              node_type ShowsMucousMembaneDefect : SYMPTOM end;

                              node_type BloodInExcrements : SYMPTOM end;

                              node_type GastricUlcer : SYMPTOM end;

                              transformation AddUlcus =

                                 begin
                                    obl_node `1 : GastrosScopy;
                                    (* @CC_LG 6007(72,28)(125,21) @*)
                                    obl_node `2 : Diarrhoea;
                                    (* @CC_LG 6007(240,28)(101,21) @*)
                                 end
                                 (* @CC_LG 7008(8,8)(303,47) @*)
                                 ::=
                                 begin
                                    obl_node 1' = `1;
                                    (* @CC_LG 6007(100,247)(57,19) @*)
                                    obl_node 2' = `2;
                                    (* @CC_LG 6007(364,149)(57,19) @*)
                                    4' : GastricUlcer;
                                    (* @CC_LG 6012(264,56)(97,29) @*)
                                    2' -> 4' : contra (* @CC_LG 1002C,BC(8,-10)(36,10) @*);
                                    (* @CC_LG 3004C @*)
                                    5' : ShowsMucousMembaneDefect;
                                    (* @CC_LG 6012(96,154)(97,29) @*)
                                    5' -> 4' : pro;
                                    (* @CC_LG
                                    3004C3
                                    192,112 @*)
                                    1' -> 5' : pro (* @CC_LG 1002C,BC(10,0)(18,10) @*);
                                    (* @CC_LG 3004C @*)
                                    3' : BloodInExcrements;
                                    (* @CC_LG 6012(240,154)(97,29) @*)
                                    3' -> 4' : pro;
                                    (* @CC_LG
                                    3004C3
                                    240,112 @*)
                                 end
                                 (* @CC_LG 7009(8,8)(405,277) @*)
                              end;

                        end;

                        section Pankreatitis_Knowledge

                           declares

                              node_type SuperSonics : EXAMINATION end;

                              node_type PancreasIsThick : SYMPTOM end;

                              node_type Pancreatitis : DISEASE end;

                              node_type FoodPoisoning : SYMPTOM end;

                              node_type LipaseAmylaseEnzymeHigh : SYMPTOM end;

                              transformation AddPankreatitis =

                                 begin
                                    obl_node `1 : BloodInExcrements;
                                    (* @CC_LG 6007(84,28)(149,21) @*)
                                    obl_node `2 : Diarrhoea;
                                    (* @CC_LG 6007(228,56)(101,21) @*)
                                    obl_node `3 : BloodTest;
                                    (* @CC_LG 6007(228,28)(101,21) @*)
                                    obl_node `4 : SuperSonics;
                                    (* @CC_LG 6007(96,56)(113,21) @*)
                                 end
                                 (* @CC_LG 7008(8,8)(303,75) @*)
                                 ::=
                                 begin
                                    obl_node 1' = `1;
                                    (* @CC_LG 6007(28,224)(57,19) @*)
                                    obl_node 2' = `2;
                                    (* @CC_LG 6007(364,149)(57,19) @*)
                                    5' : Pancreatitis;
                                    (* @CC_LG 6012(240,56)(97,29) @*)
                                    6' : FoodPoisoning;
                                    (* @CC_LG 6012(96,56)(97,29) @*)
                                    2' -> 6' : pro (* @CC_LG 1002C,TC(0,10)(18,10) @*);
                                    (* @CC_LG 3004C @*)
                                    obl_node 3' = `3;
                                    (* @CC_LG 6007(244,247)(57,19) @*)
                                    8' : LipaseAmylaseEnzymHigh;
                                    (* @CC_LG 6012(240,154)(97,29) @*)
                                    3' -> 8' : pro;
                                    (* @CC_LG 3004C @*)
                                    8' -> 5' : pro;
                                    (* @CC_LG 3004C @*)
                                    1' -> 6' : contra (* @CC_LG 1002C,LC(10,0)(36,10) @*);
                                    (* @CC_LG
                                    3004C4
                                    24,140
                                    72,112 @*)
                                    obl_node 4' = `4;
                                    (* @CC_LG 6007(100,247)(57,19) @*)
                                    7' : PancreasIsThick;
                                    (* @CC_LG 6012(96,154)(97,29) @*)
                                    4' -> 7' : pro;
                                    (* @CC_LG 3004C @*)
                                    7' -> 5' : pro;
                                    (* @CC_LG 3004C @*)
                                 end
                                 (* @CC_LG 7009(8,8)(405,277) @*)
                              end;

                        end;

                  end;

                  transformation CreatePractice ( practiceName : string ; doctorName : string ; out practice : PRACTICE ; out doctor : DOCTOR)
                     =
                     (<OptPrecondDecl>)
                     (* @CC_LG 7008(8,8)(291,33) @*)

                     ::=
                     begin
                        1' : LocalPractice;
                        (* @CC_LG 6012(96,154)(97,29) @*)
                        2' : Doctor;
                        (* @CC_LG 6012(96,56)(97,29) @*)
                        1' -> 2' : selem;
                        (* @CC_LG 3004C @*)
                     end
                     (* @CC_LG 7009(8,8)(291,189) @*)
                     transfer 1'.Name := practiceName;
                              2'.Name := doctorName;
                     return practice := 1';
                            doctor := 2';
                  end;

                  transformation AddPatient ( practice : PRACTICE ; patientName : string ; out patient : PATIENT)
                     =

                     begin
                        obl_node `1 = practice;
                        (* @CC_LG 6007(72,28)(97,21) @*)
                     end
                     (* @CC_LG 7008(8,8)(291,47) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(156,121)(57,21) @*)
                        2' : Patient;
                        (* @CC_LG 6012(156,28)(97,21) @*)
                        1' -> 2' : selem;
                        (* @CC_LG 3004C @*)
                     end
                     (* @CC_LG 7009(8,8)(291,151) @*)
                     transfer 2'.Name := patientName;
                     return patient := 2';
                  end;

                  transformation AddTreatment ( patient : PATIENT ; doctor : DOCTOR ; out examination : DIAGNOSIS)
                     =

                     begin
                        obl_node `1 = patient;
                        (* @CC_LG 6007(80,150)(89,21) @*)
                        obl_node `2 = doctor;
                        (* @CC_LG 6007(221,26)(83,21) @*)
                        opt_node `3 : Treatment;
                        (* @CC_LG 6009(84,56)(97,21) @*)
                        `1 -> `3 : llast;
                        (* @CC_LG 3004C @*)
                     end
                     (* @CC_LG 7008(8,8)(291,181) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(148,247)(57,19) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(52,247)(57,19) @*)
                        6' : Treatment;
                        (* @CC_LG 6012(168,154)(97,29) @*)
                        1' -> 6' : lelem (* @CC_LG 1002C,BC(-23,0)(24,10) @*);
                        (* @CC_LG
                        3004C3
                        144,196 @*)
                        2' -> 6' : does (* @CC_LG 1002C,BC(29,0)(24,10) @*);
                        (* @CC_LG 3004C @*)
                        4' : Diagnosis;
                        (* @CC_LG 6012(240,56)(97,29) @*)
                        5' : Prescription;
                        (* @CC_LG 6012(96,56)(97,29) @*)
                        6' -> 4' : toDiagnosis (* @CC_LG 1002C,BC(-21,-8)(66,10) @*);
                        (* @CC_LG 3004C @*)
                        6' -> 5' : toPrescription (* @CC_LG 1002C,BC(27,-10)(6,10) @*);
                        (* @CC_LG 3004C @*)
                        opt_node 3' = `3;
                        (* @CC_LG 6009(264,252)(97,29) @*)
                        3' -> 6' : lnext;
                        (* @CC_LG 3004C @*)
                        1' -> 6' : llast;
                        (* @CC_LG
                        3004C3
                        168,196 @*)
                     end
                     (* @CC_LG 7009(8,8)(325,287) @*)
                     return examination := 4';
                  end;

                  transformation SetCurrentTopic ( diagnosis : DIAGNOSIS ; evidence : type_in EVIDENCE)
                     =

                     begin
                        obl_node `1 = diagnosis;
                        (* @CC_LG 6007(72,28)(101,21) @*)
                        obl_node `2 : evidence;
                        (* @CC_LG 6007(228,70)(95,21) @*)
                        `1 #> `2 : Clear;
                        (* @CC_LG 1002C,BC(10,-8)(18,10) @*)
                     end
                     (* @CC_LG 7008(8,8)(291,89) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(168,214)(57,21) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(168,28)(57,21) @*)
                        3' : Current;
                        (* @CC_LG 6012(168,121)(97,21) @*)
                        1' -> 3' : f;
                        (* @CC_LG 3004C @*)
                        3' -> 2' : t;
                        (* @CC_LG 3004C @*)
                     end
                     (* @CC_LG 7009(8,8)(325,244) @*)
                  end;

                  transformation RecordEvidence ( examination : DIAGNOSIS ; result : string) =

                     choose
                        when (result = "yes")
                        then
                           RecordYesEvidence ( examination )
                     else
                        when (result = "no")
                        then
                           RecordNoEvidence ( examination )
                     end
                  end;

                  transformation RecordYesEvidence ( diagnosis : DIAGNOSIS) =

                     begin
                        obl_node `1 = diagnosis;
                        (* @CC_LG 6007(60,28)(101,21) @*)
                        obl_node `2 : EVIDENCE;
                        (* @CC_LG 6007(312,28)(95,21) @*)
                        obl_node `3 : Current;
                        (* @CC_LG 6007(192,28)(89,21) @*)
                        `1 -> `3 : f;
                        (* @CC_LG 3004C @*)
                        `3 -> `2 : t;
                        (* @CC_LG 3004C @*)
                     end
                     (* @CC_LG 7008(8,8)(375,47) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(60,28)(57,21) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(312,28)(57,21) @*)
                        1' -> 2' : yes;
                        (* @CC_LG 3004 @*)
                     end
                     (* @CC_LG 7009(8,8)(375,47) @*)
                  end;

                  transformation RecordNoEvidence ( diagnosis : DIAGNOSIS) =

                     begin
                        obl_node `1 = diagnosis;
                        (* @CC_LG 6007(60,28)(101,21) @*)
                        obl_node `2 : EVIDENCE;
                        (* @CC_LG 6007(312,28)(95,21) @*)
                        obl_node `3 : Current;
                        (* @CC_LG 6007(192,28)(89,21) @*)
                        `1 -> `3 : f;
                        (* @CC_LG 3004C @*)
                        `3 -> `2 : t;
                        (* @CC_LG 3004C @*)
                     end
                     (* @CC_LG 7008(8,8)(375,47) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(60,28)(57,21) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(312,28)(57,21) @*)
                        1' -> 2' : no;
                        (* @CC_LG 3004 @*)
                     end
                     (* @CC_LG 7009(8,8)(375,47) @*)
                  end;

                  section GuideDiagnosis

                     declares

                        path Clear : DIAGNOSIS -> EVIDENCE =
                               -yes->
                           or  -no->
                        end;

                        path Indicates : EVIDENCE -> EVIDENCE =
                               -pro->
                           or  -contra->
                        end;

                        path Preconditions : EVIDENCE -> EVIDENCE =
                           (     <-pro-
                             or  <-contra- )
                        end;

                        path Conclusion : DIAGNOSIS -> EVIDENCE =
                               (    -yes->
                                 &  -pro-> )
                           or  (    -no->
                                 &  -contra-> )
                        end;

                        path Exclusion : DIAGNOSIS -> EVIDENCE =
                               (    -yes->
                                 &  -contra-> )
                           or  (    -no->
                                 &  -pro-> )
                        end;

                        path Conjecture : DIAGNOSIS -> EVIDENCE =
                                    Conclusion
                           but_not  (     Exclusion
                                      or  Clear     )
                        end;

                        node_class MARKER is_a ITEM end;
                        (* @CC_LG 6007(144,84)(97,29) @*)

                        node_type Current : MARKER end;

                        edge_type f : DIAGNOSIS -> MARKER [0:1];

                        edge_type t : MARKER -> EVIDENCE [1:1];

                        transformation MarkCurrentTopic ( diagnosis : DIAGNOSIS) =

                           MarkConjecture ( diagnosis )
                           & loop
                                MarkPreCondition ( diagnosis )
                             end
                        end;

                        transformation MarkConjecture ( diagnosis : DIAGNOSIS) =

                           begin
                              obl_node `1 = diagnosis;
                              (* @CC_LG 6007(60,28)(101,21) @*)
                              obl_node `2 : EVIDENCE;
                              (* @CC_LG 6007(240,28)(101,21) @*)
                              `1 => `2 : Conjecture;
                              (* @CC_LG 3006 @*)
                           end
                           (* @CC_LG 7008(8,8)(303,47) @*)
                           ::=
                           begin
                              obl_node 1' = `1;
                              (* @CC_LG 6007(264,216)(59,21) @*)
                              obl_node 2' = `2;
                              (* @CC_LG 6007(264,28)(59,21) @*)
                              3' : Current;
                              (* @CC_LG 6012(264,122)(97,29) @*)
                              1' -> 3' : f;
                              (* @CC_LG 3004C @*)
                              3' -> 2' : t;
                              (* @CC_LG 3004C @*)
                           end
                           (* @CC_LG 7009(8,8)(325,247) @*)
                        end;

                        transformation MarkPreCondition ( diagnosis : DIAGNOSIS) =

                           begin
                              obl_node `1 = diagnosis;
                              (* @CC_LG 6007(108,28)(101,21) @*)
                              obl_node `2 : EVIDENCE;
                              (* @CC_LG 6007(168,126)(95,21) @*)
                              obl_node `3 : EVIDENCE;
                              (* @CC_LG 6007(60,182)(95,21) @*)
                              `1 #> `3 : Clear;
                              (* @CC_LG 3007C @*)
                              `2 => `3 : Preconditions;
                              (* @CC_LG 3006C @*)
                              obl_node `4 : Current;
                              (* @CC_LG 6007(168,70)(89,21) @*)
                              `1 -> `4 : f;
                              (* @CC_LG 3004C @*)
                              `4 -> `2 : t (* @CC_LG 1002C,RC(-10,0)(6,10) @*);
                              (* @CC_LG 3004C @*)
                           end
                           (* @CC_LG 7008(8,8)(224,204) @*)
                           ::=
                           begin
                              obl_node 1' = `1;
                              (* @CC_LG 6007(268,270)(57,19) @*)
                              obl_node 2' = `2;
                              (* @CC_LG 6007(124,112)(57,19) @*)
                              obl_node 3' = `3;
                              (* @CC_LG 6007(244,84)(57,19) @*)
                              4' : Current;
                              (* @CC_LG 6012(264,177)(97,29) @*)
                              1' -> 4' : f;
                              (* @CC_LG 3004C @*)
                              4' -> 3' : t (* @CC_LG 1002C,BC(-4,6)(6,10) @*);
                              (* @CC_LG 3004C @*)
                           end
                           (* @CC_LG 7009(8,8)(325,300) @*)
                        end;

                  end;

                  section Mark_Therapies

                     declares

                        path ToTherapies : DIAGNOSIS -> THERAPY =
                           `1 => `3 in
                           begin
                              obl_node `1 : DIAGNOSIS;
                              (* @CC_LG 6007(60,28)(101,21) @*)
                              obl_node `2 : EVIDENCE;
                              (* @CC_LG 6007(60,112)(95,21) @*)
                              obl_node `3 : THERAPY;
                              (* @CC_LG 6007(240,112)(89,21) @*)
                              `1 -> `2 : yes (* @CC_LG 1002(-23,4)(18,10) @*);
                              (* @CC_LG 3004C @*)
                              `2 -> `3 : needs (* @CC_LG 1002C,BC(-16,6)(30,10) @*);
                              (* @CC_LG 3004C @*)
                              `1 #> `3 : ToExcluded;
                           end
                           (* @CC_LG 7008(8,8)(291,131) @*)
                        end;

                        path ToExcluded : DIAGNOSIS -> THERAPY =
                              -yes->
                           &  -excludes->
                        end;

                        transformation MarkTherapies ( diagnosis : DIAGNOSIS) =

                           begin
                              obl_node `1 = diagnosis;
                              (* @CC_LG 6007(266,150)(101,21) @*)
                              opt_set `2 : THERAPY;
                              (* @CC_LG 6010(264,56)(97,29) @*)
                              `1 => `2 : ToTherapies;
                              (* @CC_LG 3008C @*)
                              obl_node `3 : TREATMENT;
                              (* @CC_LG 6007(218,240)(101,21) @*)
                              `3 -> `1 : toDiagnosis;
                              (* @CC_LG 3004C @*)
                              obl_node `4 : PRESCRIPTION;
                              (* @CC_LG 6007(203,28)(119,21) @*)
                              `3 -> `4 : toPrescription;
                              (* @CC_LG
                              3004C4
                              216,168
                              192,140 @*)
                           end
                           (* @CC_LG 7008(8,8)(375,271) @*)
                           ::=
                           begin
                              obl_node 1' = `1;
                              (* @CC_LG 6007(100,98)(57,19) @*)
                              opt_set 2' = `2;
                              (* @CC_LG 6010(144,56)(97,29) @*)
                              obl_node 3' = `3;
                              (* @CC_LG 6007(124,237)(57,19) @*)
                              3' -> 1' : toDiagnosis (* @CC_LG 1002C,BC(0,-10)(6,10) @*);
                              (* @CC_LG
                              3004C4
                              144,196
                              96,140 @*)
                              obl_node 4' = `4;
                              (* @CC_LG 6007(148,149)(57,19) @*)
                              3' -> 4' : toPrescription (* @CC_LG 1002C,BC(0,-10)(6,10) @*);
                              (* @CC_LG
                              3004C3
                              120,196 @*)
                              4' -> 2' : gets;
                              (* @CC_LG 3004C @*)
                           end
                           (* @CC_LG 7009(8,8)(327,267) @*)
                        end;

                        transformation YesAndMark ( diagnosis : DIAGNOSIS) =

                           RecordYesEvidence ( diagnosis )
                           & choose
                                MarkCurrentTopic ( diagnosis )
                             else
                                skip
                             end
                        end;

                        transformation NoAndMark ( diagnosis : DIAGNOSIS) =

                           RecordNoEvidence ( diagnosis )
                           & choose
                                MarkCurrentTopic ( diagnosis )
                             else
                                skip
                             end
                        end;

                  end;

            end;

      end;

      transformation Main =

         use practice : PRACTICE;
             patient : PATIENT;
             doctor : DOCTOR;
             diagnosis : DIAGNOSIS;
             evidence : type_in EVIDENCE
         do
            CreatePractice ( "General Hospital", "Dr. M. Welby", out practice, out doctor )
            & AddPatient ( practice, "W. Johnson", out patient )
            & AddTreatment ( patient, doctor, out diagnosis )
            & AddKnowledge
            & SetCurrentTopic ( diagnosis, SoreThroat )
            & YesAndMark ( diagnosis )
            & loop
                 begin
                       YesAndMark ( diagnosis )
                    or NoAndMark ( diagnosis )
                 end
              end
            & choose
                 MarkTherapies ( diagnosis )
              else
                 skip
              end
            & choose
                 AddPatient ( practice, "A. Smith", out patient )
                 & AddMoreKnowledge
              end
         end
      end;

end.
