spec PhascumCuspidatum (* The specification modells the first five *)
                       (* steps of the development process of the  *)
                       (* moss Phascum Cuspidatum. This is one of  *)
                       (* the popular examples in the literature   *)
                       (* about so-called L(indenmeyer) systems.   *)
                       (* L systems are closely related to graph   *)
                       (* grammars and usually used to model the   *)
                       (* development process of plants.           *)
                       (*                                          *)
                       (* Introduced node and edge types have the  *)
                       (* following meaning:                       *)
                       (*                                          *)
                       (* P = primary cell                         *)
                       (* S = secondary cell (six types of cells)  *)
                       (* T = tertiary cell                        *)
                       (*                                          *)
                       (* midv = middle vein (of leaf)             *)
                       (* branch = branch of middle vein           *)
                       (* peri = periphery (border) of leaf        *)

   declares

      node_class ANY end;

      node_type P : ANY end;

      node_type S : ANY end;

      node_type S1 : ANY end;

      node_type S2 : ANY end;

      node_type S3 : ANY end;

      node_type S4 : ANY end;

      node_type S5 : ANY end;

      node_type T : ANY end;

      edge_type midv : ANY -> ANY;

      edge_type peri : ANY -> ANY;

      edge_type branch : ANY -> ANY;

      transformation Create (* @CC_LG 7008(8,8)(405,45) @*)
                            (* @CC_LG 7008(8,8)(20,45) @*)
                            (* @CC_LG 7008(8,8)(20,45) @*)
                            (* @CC_LG 7008(8,8)(20,45) @*)
                            (* @CC_LG 7008(8,8)(20,45) @*)
                            (* @CC_LG 7008(8,8)(20,45) @*)  =
         (<OptPrecondDecl>)
         (* @CC_LG 7008(8,8)(36,38) @*)

         ::=
         begin
            1' : P;
            (* @CC_LG 6012(60,28)(97,21) @*)
         end
         (* @CC_LG 7009(8,8)(125,55) @*)
      end;

      transformation p1 * =

         begin
            obl_node `1 : P;
            (* @CC_LG 6007(269,20)(59,21) @*)
            opt_set `3 : S2;
            (* @CC_LG 6010(144,118)(97,29) @*)
            opt_set `4 : S1;
            (* @CC_LG 6010(288,118)(97,29) @*)
            `4 -> `1 : peri;
            (* @CC_LG 3004C @*)
            `3 -> `1 : peri;
            (* @CC_LG 3004C @*)
            opt_set `5 : ANY;
            (* @CC_LG 6010(432,118)(97,29) @*)
            `5 -> `1 : midv;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(498,153) @*)
         ::=
         begin
            opt_set 3' = `3;
            (* @CC_LG 6010(240,256)(97,29) @*)
            opt_set 4' = `4;
            (* @CC_LG 6010(240,158)(97,29) @*)
            1' : P;
            (* @CC_LG 6012(144,56)(97,29) @*)
            2' : S;
            (* @CC_LG 6012(96,154)(97,29) @*)
            3' -> 2' : peri;
            (* @CC_LG 3004C @*)
            4' -> 1' : peri;
            (* @CC_LG 3004C @*)
            2' -> 1' : midv;
            (* @CC_LG 3004C @*)
            2' -> 1' : peri (* @CC_LG 1002C,LC(27,0)(24,10) @*);
            (* @CC_LG
            3004C3
            96,112 @*)
            opt_set 5' = `5;
            (* @CC_LG 6010(96,256)(97,29) @*)
            5' -> 2' : midv;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(306,291) @*)
         folding { `4, `5 }; { `3, `5 };
      end;

      transformation p2 * =

         begin
            obl_node `1 : S;
            (* @CC_LG 6007(266,150)(53,21) @*)
            opt_set `2 : ANY;
            (* @CC_LG 6010(168,248)(97,29) @*)
            opt_set `3 : ANY;
            (* @CC_LG 6010(312,248)(97,29) @*)
            opt_set `4 : ANY;
            (* @CC_LG 6010(384,56)(97,29) @*)
            opt_set `5 : ANY;
            (* @CC_LG 6010(240,56)(97,29) @*)
            `2 -> `1 : midv;
            (* @CC_LG 3004C @*)
            `1 -> `4 : midv;
            (* @CC_LG
            3004C3
            336,112 @*)
            `3 -> `1 : peri;
            (* @CC_LG 3004C @*)
            `1 -> `5 : peri;
            (* @CC_LG
            3004C3
            264,112 @*)
         end
         (* @CC_LG 7008(8,8)(450,283) @*)
         ::=
         begin
            1' : S1;
            (* @CC_LG 6012(216,154)(97,29) @*)
            opt_set 2' = `2;
            (* @CC_LG 6010(168,256)(97,29) @*)
            opt_set 3' = `3;
            (* @CC_LG 6010(312,256)(97,29) @*)
            opt_set 4' = `4;
            (* @CC_LG 6010(96,56)(97,29) @*)
            opt_set 5' = `5;
            (* @CC_LG 6010(240,56)(97,29) @*)
            2' -> 1' : midv;
            (* @CC_LG 3004C @*)
            1' -> 4' : midv;
            (* @CC_LG 3004C @*)
            3' -> 1' : peri;
            (* @CC_LG 3004C @*)
            1' -> 5' : peri;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(378,291) @*)
         folding { `2, `3 }; { `4, `5 };
      end;

      transformation p3 * =

         begin
            obl_node `1 : S1;
            (* @CC_LG 6007(269,150)(59,21) @*)
            opt_set `2 : ANY;
            (* @CC_LG 6010(168,248)(97,29) @*)
            `2 -> `1 : peri;
            (* @CC_LG 3004C @*)
            opt_set `3 : ANY;
            (* @CC_LG 6010(384,56)(97,29) @*)
            `1 -> `3 : peri;
            (* @CC_LG
            3004C3
            336,112 @*)
            opt_set `4 : ANY;
            (* @CC_LG 6010(312,248)(97,29) @*)
            `4 -> `1 : midv;
            (* @CC_LG 3004C @*)
            opt_set `5 : ANY;
            (* @CC_LG 6010(240,56)(97,29) @*)
            `1 -> `5 : midv;
            (* @CC_LG
            3004C3
            264,112 @*)
         end
         (* @CC_LG 7008(8,8)(450,283) @*)
         ::=
         begin
            1' : S2;
            (* @CC_LG 6012(120,154)(97,29) @*)
            6' : T;
            (* @CC_LG 6012(240,252)(97,29) @*)
            6' -> 1' : branch (* @CC_LG 1002C,BC(14,16)(36,10) @*);
            (* @CC_LG
            3004C3
            192,196 @*)
            opt_set 2' = `2;
            (* @CC_LG 6010(96,256)(97,29) @*)
            2' -> 1' : peri;
            (* @CC_LG 3004C @*)
            opt_set 3' = `3;
            (* @CC_LG 6010(96,56)(97,29) @*)
            1' -> 3' : peri;
            (* @CC_LG 3004C @*)
            opt_set 4' = `4;
            (* @CC_LG 6010(240,354)(97,29) @*)
            4' -> 6' : midv;
            (* @CC_LG 3004C @*)
            opt_set 5' = `5;
            (* @CC_LG 6010(240,56)(97,29) @*)
            6' -> 5' : midv;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(340,389) @*)
         folding { `2, `4 }; { `3, `5 };
      end;

      transformation p4 * =

         begin
            obl_node `1 : S2;
            (* @CC_LG 6007(173,150)(59,21) @*)
            opt_set `2 : ANY;
            (* @CC_LG 6010(96,248)(97,29) @*)
            `2 -> `1 : branch (* @CC_LG 1002(-34,-25)(36,10) @*);
            (* @CC_LG 3004C @*)
            opt_set `3 : ANY;
            (* @CC_LG 6010(240,248)(97,29) @*)
            opt_set `4 : ANY;
            (* @CC_LG 6010(168,56)(97,29) @*)
            `3 -> `1 : peri;
            (* @CC_LG
            3004C3
            216,196 @*)
            `1 -> `4 : peri;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(306,283) @*)
         ::=
         begin
            1' : S3;
            (* @CC_LG 6012(288,154)(97,29) @*)
            5' : S3;
            (* @CC_LG 6012(240,252)(97,29) @*)
            5' -> 1' : peri;
            (* @CC_LG 3004C @*)
            opt_set 2' = `2;
            (* @CC_LG 6010(312,354)(97,29) @*)
            2' -> 1' : branch;
            (* @CC_LG
            3004C4
            336,308
            336,224 @*)
            2' -> 5' : branch;
            (* @CC_LG
            3004C3
            288,308 @*)
            opt_set 3' = `3;
            (* @CC_LG 6010(168,354)(97,29) @*)
            3' -> 5' : peri;
            (* @CC_LG
            3004C3
            216,308 @*)
            opt_set 4' = `4;
            (* @CC_LG 6010(288,56)(97,29) @*)
            1' -> 4' : peri (* @CC_LG 1002C,BC(0,-10)(24,10) @*);
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(378,389) @*)
      end;

      transformation p5 * =

         begin
            obl_node `1 : S3;
            (* @CC_LG 6007(173,150)(59,21) @*)
            opt_set `2 : ANY;
            (* @CC_LG 6010(96,248)(97,29) @*)
            `2 -> `1 : branch;
            (* @CC_LG 3004C @*)
            opt_set `3 : ANY;
            (* @CC_LG 6010(240,248)(97,29) @*)
            opt_set `4 : ANY;
            (* @CC_LG 6010(168,56)(97,29) @*)
            `3 -> `1 : peri;
            (* @CC_LG
            3004C3
            216,196 @*)
            `1 -> `4 : peri;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(306,283) @*)
         ::=
         begin
            1' : S4;
            (* @CC_LG 6012(168,154)(97,29) @*)
            opt_set 2' = `2;
            (* @CC_LG 6010(96,256)(97,29) @*)
            opt_set 3' = `3;
            (* @CC_LG 6010(240,256)(97,29) @*)
            opt_set 4' = `4;
            (* @CC_LG 6010(168,56)(97,29) @*)
            3' -> 1' : peri;
            (* @CC_LG 3004C @*)
            1' -> 4' : peri;
            (* @CC_LG 3004C @*)
            2' -> 1' : branch;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(306,291) @*)
      end;

      transformation p6 * =

         begin
            obl_node `1 : S4;
            (* @CC_LG 6007(197,150)(59,21) @*)
            opt_set `2 : ANY;
            (* @CC_LG 6010(264,248)(97,29) @*)
            `2 -> `1 : branch;
            (* @CC_LG
            3004C3
            240,196 @*)
            opt_set `3 : ANY;
            (* @CC_LG 6010(120,248)(97,29) @*)
            `3 -> `1 : peri;
            (* @CC_LG 3004C @*)
            opt_set `4 : ANY;
            (* @CC_LG 6010(192,56)(97,29) @*)
            `1 -> `4 : peri;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7008(8,8)(330,283) @*)
         ::=
         begin
            1' : S5;
            (* @CC_LG 6012(192,154)(97,29) @*)
            opt_set 2' = `2;
            (* @CC_LG 6010(96,354)(97,29) @*)
            opt_set 3' = `3;
            (* @CC_LG 6010(240,256)(97,29) @*)
            3' -> 1' : peri;
            (* @CC_LG 3004C @*)
            opt_set 4' = `4;
            (* @CC_LG 6010(192,56)(97,29) @*)
            1' -> 4' : peri;
            (* @CC_LG 3004C @*)
            5' : T;
            (* @CC_LG 6012(96,252)(97,29) @*)
            5' -> 1' : branch;
            (* @CC_LG 3004C @*)
            2' -> 5' : branch (* @CC_LG 1002C,TC(0,10)(36,10) @*);
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(342,389) @*)
      end;

      transformation MAIN (* The main transaction which regulates *)
                          (* the application of all productions.  *)
                          (* It's implementation is a little bit  *)
                          (* due to the following problem:        *)
                          (*                                      *)
                          (* The original version of the spec. in *)
                          (*                                      *)
                          (* Manfred Nagl: Graphgrammatiken       *)
                          (* Vieweg Verlag 1979                   *)
                          (*                                      *)
                          (* requires the parallel application of *)
                          (* all rules to all occurrences of left *)
                          (* hand sides. PROGRES supports the     *)
                          (* application of a single rule to all  *)
                          (* occurrences of its left hand side    *)
                          (* (via the * annotation in the rule's  *)
                          (* declaration), but not the parallel   *)
                          (* application of a set of rules.       *)
                          (* Therefore, we have to take care that *)
                          (* rule x rewrites in development step  *)
                          (* n only nodes created by rule y in    *)
                          (* development step n-1. The simplest   *)
                          (* way to guarantee such a behaviour of *)
                          (* rewrite rules is the reverse sequ.   *)
                          (* application of rules p1 through p6.  *) =

         Create
         & loop
              p6
              & p5
              & p4
              & p3
              & p2
              & p1
           end
      end;

end.
