spec RedBlackTrees

   declares

      section scheme

         declares

            node_class RBNODE
               intrinsic
                  key Value : integer;
                  GRAPH_BROWSER_LABEL : string;
               derived
                  Height : integer = max ( self.lHeight, self.rHeight );
                  lHeight = [ self.-bl->.Height + 1
                            | self.-rl->.Height
                            | 0 ]                  ;
                  rHeight = [ self.-br->.Height + 1
                            | self.-rr->.Height
                            | 0 ]                  ;
            end;
            (* @CC_LG 6007(205,77)(97,29) @*)

            edge_type bl : RBNODE [0:1] -> RBNODE [0:1];
            (* @CC_LG
            3004M5
            274,78
            274,112
            234,112 @*)

            edge_type br : RBNODE [0:1] -> RBNODE [0:1];
            (* @CC_LG
            3004M5
            274,78
            274,112
            234,112 @*)

            edge_type rl : RBNODE [0:1] -> RBNODE [0:1];
            (* @CC_LG
            3004M5
            274,78
            274,112
            234,112 @*)

            edge_type rr : RBNODE [0:1] -> RBNODE [0:1];
            (* @CC_LG
            3004M5
            274,78
            274,112
            234,112 @*)

            static path right : RBNODE [0:1] -> RBNODE [0:1] =
               [ -br->
               | -rr-> ]
            end;
            (* @CC_LG
            3006M5
            274,78
            274,112
            234,112 @*)

            static path left : RBNODE [0:1] -> RBNODE [0:1] =
               [ -bl->
               | -rl-> ]
            end;
            (* @CC_LG
            3006M5
            274,78
            274,112
            234,112 @*)

            path parent : RBNODE [0:1] -> RBNODE [0:1] =
               [ <=left=
               | <=right= ]
            end;
            (* @CC_LG
            3006M5
            274,78
            274,112
            234,112 @*)

            path down( Val : integer) : RBNODE -> RBNODE [0:1] =
               [ valid (Val < self.Value) :: =left=>
               | valid (Val > self.Value) :: =right=> ]
            end;
            (* @CC_LG
            3006M5
            274,78
            274,112
            234,112 @*)

            function df : ( Src : RBNODE ; Val : integer) -> RBNODE [0:1] =
               [ Val < Src.Value :: Src.=left=>
               | Val > Src.Value :: Src.=right=> ]
            end;

            restriction balanced : RBNODE =
               valid (self.lHeight = self.rHeight)
            end;
            (* @CC_LG
            3009XC2
            205,115 @*)

            restriction correctColoured : RBNODE =
               not with (    (     -rl->
                               or  -rr-> )
                          &  (     -rl->
                               or  -rr-> ) )
            end;
            (* @CC_LG
            3009XC2
            205,115 @*)

            node_type RBNode : RBNODE end;
            (* @CC_LG 6013(181,77)(47,29) @*)

            function max : ( x : integer ; y : integer) -> integer =
               [ x <= y :: y
               | x ]
            end;

            query GetRoot( out nn : RBNODE) =
               begin
                  obl_node `1 : RBNODE;
                  (* @CC_LG 6007(53,34)(83,21) @*)
                  => `1 : not with (     <-bl-
                                     or  <-rl-
                                     or  <-br-
                                     or  <-rr- );
                  (* @CC_LG
                  3011XC2
                  184,38 @*)
               end
               (* @CC_LG 7008(8,8)(336,79) @*)
               return nn := `1;
            end;

      end;
      (* @CC_LG 7013(12,27)(417,142) @*)

      section main

         declares

            transformation MAIN =

               use i := 1
               do
                  InsertFirst ( 20 )
                  & loop
                       Insert ( i )
                       & i := (i + 1)
                    end
               end
            end;

            transformation Rotate ( n : RBNODE ; out nn : RBNODE) =

               nn := n
               & loop
                       RotateLR ( n, out nn )
                    or RotateRL ( n, out nn )
                    or RotateLL ( n, out nn )
                    or RotateRR ( n, out nn )
                 end
            end;

            query FindInconsistencies( out S : RBNODE [1:n]) =
               begin
                  obl_node `X : RBNODE;
                  (* @CC_LG 6007(60,31)(83,21) @*)
                  => `X :     not balanced
                          or  not correctColoured;
                  (* @CC_LG
                  3011XC2
                  194,66 @*)
               end
               (* @CC_LG 7008(8,8)(354,94) @*)
               return S := `X;
            end;

            transformation Insert ( Val : integer) =

               use n : RBNODE
               do
                  GetRoot ( out n )
                  & loop
                       choose
                          Split ( n, out n )
                          & Rotate ( n, out n )
                       else
                          n := (n.down ( Val ))
                       end
                    end
                  & begin
                          InsertRight ( n, Val, out n )
                       or InsertLeft ( n, Val, out n )
                    end
                  & Rotate ( n, out n )
               end
            end;

            transformation InsertFirst ( Val : integer) =
               (<OptPrecondDecl>)
               (* @CC_LG 7008(8,8)(291,145) @*)

               (<OptLeftSideList>)
               (* @CC_LG 7008(8,8)(24,24) @*)
               ::=
               begin
                  x' : RBNode;
                  (* @CC_LG 6012(60,28)(97,21) @*)
               end
               (* @CC_LG 7009(8,8)(131,55) @*)
               transfer x'.Value := Val;
                        x'.GRAPH_BROWSER_LABEL := string ( Val );
            end;

            transformation Split ( n : RBNODE ; out nn : RBNODE) =

               begin
                  obl_node `x = n;
                  (* @CC_LG 6007(146,124)(53,21) @*)
                  obl_node `y : RBNODE;
                  (* @CC_LG 6007(65,34)(83,21) @*)
                  obl_node `z : RBNODE;
                  (* @CC_LG 6007(185,34)(83,21) @*)
                  `x -> `y : rl;
                  (* @CC_LG 3004C @*)
                  `x -> `z : rr;
                  (* @CC_LG 3004C @*)
                  opt_node `a : RBNODE;
                  (* @CC_LG 6009(120,218)(97,21) @*)
                  opt_node `b : RBNODE;
                  (* @CC_LG 6009(264,218)(97,21) @*)
                  `b -> `x : bl;
                  (* @CC_LG 3004C @*)
                  `a -> `x : br;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(325,253) @*)
               ::=
               begin
                  opt_node a' = `a;
                  (* @CC_LG 6009(252,218)(97,21) @*)
                  opt_node b' = `b;
                  (* @CC_LG 6009(108,218)(97,21) @*)
                  obl_node x' = `x;
                  (* @CC_LG 6007(137,124)(59,21) @*)
                  obl_node y' = `y;
                  (* @CC_LG 6007(161,34)(59,21) @*)
                  obl_node z' = `z;
                  (* @CC_LG 6007(65,34)(59,21) @*)
                  a' -> x' : rr;
                  (* @CC_LG 3004C @*)
                  b' -> x' : rl;
                  (* @CC_LG 3004C @*)
                  x' -> y' : bl;
                  (* @CC_LG 3004C @*)
                  x' -> z' : br;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(325,253) @*)
               return nn := n;
            end;

            transformation InsertRight ( n : RBNODE ; Val : integer ; out nn : RBNODE) =

               begin
                  obl_node `x = n;
                  (* @CC_LG 6007(98,34)(53,21) @*)
                  => `x : not with =right=>;
                  (* @CC_LG
                  3011XC2
                  131,122 @*)
               end
               (* @CC_LG 7008(8,8)(233,163) @*)
               ::=
               begin
                  obl_node x' = `x;
                  (* @CC_LG 6007(41,34)(59,21) @*)
                  y' : RBNode;
                  (* @CC_LG 6012(84,70)(97,21) @*)
                  x' -> y' : rr;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(155,97) @*)
               condition Val > `x.Value;
               transfer y'.Value := Val;
                        y'.GRAPH_BROWSER_LABEL := string ( Val );
               return nn := y';
            end;

            transformation InsertLeft ( n : RBNODE ; Val : integer ; out nn : RBNODE) =

               begin
                  obl_node `x = n;
                  (* @CC_LG 6007(47,33)(53,21) @*)
                  => `x : not with =left=>;
                  (* @CC_LG
                  3011XC2
                  194,80 @*)
               end
               (* @CC_LG 7008(8,8)(312,110) @*)
               ::=
               begin
                  obl_node x' = `x;
                  (* @CC_LG 6007(41,34)(59,21) @*)
                  y' : RBNode;
                  (* @CC_LG 6012(84,70)(97,21) @*)
                  x' -> y' : rl;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(155,97) @*)
               condition Val < `x.Value;
               transfer y'.Value := Val;
                        y'.GRAPH_BROWSER_LABEL := string ( Val );
               return nn := y';
            end;

            transformation RotateLL ( n : RBNODE ; out nn : RBNODE) =

               begin
                  obl_node `y : RBNode;
                  (* @CC_LG 6007(101,222)(83,21) @*)
                  obl_node `a = n;
                  (* @CC_LG 6007(62,132)(53,21) @*)
                  opt_node `b : RBNode;
                  (* @CC_LG 6009(132,28)(97,21) @*)
                  obl_node `x : RBNode;
                  (* @CC_LG 6007(101,312)(83,21) @*)
                  `x -> `y : rl;
                  (* @CC_LG 3004C @*)
                  `y -> `b : br;
                  (* @CC_LG
                  3004C4
                  108,168
                  132,140 @*)
                  `y -> `a : rl;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(200,343) @*)
               ::=
               begin
                  obl_node y' = `y;
                  (* @CC_LG 6007(161,212)(59,21) @*)
                  obl_node a' = `a;
                  (* @CC_LG 6007(60,84)(59,21) @*)
                  y' -> a' : rl;
                  (* @CC_LG 3004C @*)
                  obl_node x' = `x;
                  (* @CC_LG 6007(209,122)(59,21) @*)
                  opt_node b' = `b;
                  (* @CC_LG 6009(204,28)(97,21) @*)
                  x' -> b' : bl;
                  (* @CC_LG 3004C @*)
                  y' -> x' : rr;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(263,243) @*)
               embedding redirect <-br-, <-bl- from `x to y';
               return nn := n;
            end;

            transformation RotateRR ( n : RBNODE ; out nn : RBNODE) =

               begin
                  obl_node `y : RBNode;
                  (* @CC_LG 6007(137,250)(83,21) @*)
                  obl_node `x : RBNode;
                  (* @CC_LG 6007(137,340)(83,21) @*)
                  `x -> `y : rr;
                  (* @CC_LG 3004C @*)
                  opt_node `a : RBNode;
                  (* @CC_LG 6009(96,56)(97,29) @*)
                  obl_node `b = n;
                  (* @CC_LG 6007(146,160)(53,21) @*)
                  `y -> `a : bl;
                  (* @CC_LG
                  3004C5
                  120,196
                  120,168
                  96,140 @*)
                  `y -> `b : rr;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7008(8,8)(200,371) @*)
               ::=
               begin
                  obl_node y' = `y;
                  (* @CC_LG 6007(173,240)(59,21) @*)
                  obl_node x' = `x;
                  (* @CC_LG 6007(221,150)(59,21) @*)
                  y' -> x' : rl;
                  (* @CC_LG 3004C @*)
                  obl_node b' = `b;
                  (* @CC_LG 6007(108,112)(59,21) @*)
                  opt_node a' = `a;
                  (* @CC_LG 6009(96,56)(97,29) @*)
                  y' -> b' : rr;
                  (* @CC_LG 3004C @*)
                  x' -> a' : br;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(263,271) @*)
               embedding redirect <-br-, <-bl- from `x to y';
               return nn := n;
            end;

            transformation RotateLR ( n : RBNODE ; out nn : RBNODE) =

               begin
                  obl_node `x : RBNode;
                  (* @CC_LG 6007(185,340)(83,21) @*)
                  obl_node `y : RBNode;
                  (* @CC_LG 6007(185,250)(83,21) @*)
                  obl_node `z = n;
                  (* @CC_LG 6007(170,160)(53,21) @*)
                  opt_node `a : RBNode;
                  (* @CC_LG 6009(240,56)(97,29) @*)
                  opt_node `b : RBNode;
                  (* @CC_LG 6009(96,56)(97,29) @*)
                  `x -> `y : rl;
                  (* @CC_LG 3004C @*)
                  `y -> `z : rr;
                  (* @CC_LG 3004C @*)
                  `z -> `a : bl;
                  (* @CC_LG
                  3004C3
                  216,112 @*)
                  `z -> `b : br;
                  (* @CC_LG
                  3004C3
                  144,112 @*)
               end
               (* @CC_LG 7008(8,8)(301,371) @*)
               ::=
               begin
                  obl_node z' = `z;
                  (* @CC_LG 6007(173,240)(59,21) @*)
                  obl_node x' = `x;
                  (* @CC_LG 6007(101,150)(59,21) @*)
                  obl_node y' = `y;
                  (* @CC_LG 6007(197,150)(59,21) @*)
                  opt_node a' = `a;
                  (* @CC_LG 6009(240,56)(97,29) @*)
                  opt_node b' = `b;
                  (* @CC_LG 6009(96,56)(97,29) @*)
                  z' -> y' : rl;
                  (* @CC_LG
                  3004C3
                  192,196 @*)
                  x' -> b' : bl;
                  (* @CC_LG 3004C @*)
                  z' -> x' : rr;
                  (* @CC_LG 3004C @*)
                  y' -> a' : br;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(301,271) @*)
               embedding redirect <-br-, <-bl- from `x to z';
               return nn := y';
            end;

            transformation RotateRL ( n : RBNODE ; out nn : RBNODE) =

               begin
                  obl_node `x : RBNode;
                  (* @CC_LG 6007(185,340)(83,21) @*)
                  obl_node `y : RBNode;
                  (* @CC_LG 6007(185,250)(83,21) @*)
                  obl_node `z = n;
                  (* @CC_LG 6007(170,160)(53,21) @*)
                  opt_node `a : RBNode;
                  (* @CC_LG 6009(96,56)(97,29) @*)
                  opt_node `b : RBNode;
                  (* @CC_LG 6009(240,56)(97,29) @*)
                  `x -> `y : rr;
                  (* @CC_LG 3004C @*)
                  `y -> `z : rl;
                  (* @CC_LG 3004C @*)
                  `z -> `a : bl;
                  (* @CC_LG
                  3004C3
                  144,112 @*)
                  `z -> `b : br;
                  (* @CC_LG
                  3004C3
                  216,112 @*)
               end
               (* @CC_LG 7008(8,8)(301,371) @*)
               ::=
               begin
                  obl_node z' = `z;
                  (* @CC_LG 6007(173,240)(59,21) @*)
                  obl_node y' = `y;
                  (* @CC_LG 6007(101,150)(59,21) @*)
                  obl_node x' = `x;
                  (* @CC_LG 6007(197,150)(59,21) @*)
                  opt_node a' = `a;
                  (* @CC_LG 6009(240,56)(97,29) @*)
                  opt_node b' = `b;
                  (* @CC_LG 6009(96,56)(97,29) @*)
                  z' -> x' : rl;
                  (* @CC_LG
                  3004C3
                  192,196 @*)
                  x' -> a' : br;
                  (* @CC_LG 3004C @*)
                  z' -> y' : rr;
                  (* @CC_LG 3004C @*)
                  y' -> b' : bl;
                  (* @CC_LG 3004C @*)
               end
               (* @CC_LG 7009(8,8)(301,271) @*)
               embedding redirect <-br-, <-bl- from `x to z';
               return nn := y';
            end;

      end;
      (* @CC_LG 7013(141,36)(96,30) @*)

end.
(* @CC_LG 7012(8,8)(441,179) @*)
