spec SampleImport (* This specification demonstrates the use of imported types and imported
                  functions. The type text is an extension to the built-in type string (since
                  string is limited in length) and therefore some functions combining texts
                  and strings are natural. The && operator concatenates texts and strings (as
                  does & for strings only), == compares the contents of a text with a string.
                  Text(x) is a short-cut for value(x,text). These functions are samples for
                  the use of text, the user can easily define new functions within
                  the PROGRES specification that enable more complex operations. *)
                  (* @CC_LT HoriWithIndent @*)

   declares

      external_package LongStrings

         types text ;

         functions
            EmptyText : -> text; Text : ( string) -> text; && : ( text, string) -> text; == : ( text, string) -> boolean;
            ConcatText : ( text, text) -> text; IsEqualText : ( text, text) -> boolean;

      end;

      package MainPackage

         imports
            package LongStrings
               use
                  function ConcatText;
                  type text;
                  function Text;
                  function &&;
                  function EmptyText;
                  function IsEqualText;
            end;

         declares

            section Schema

               declares

                  node_class ANY
                     derived
                        GRAPH_BROWSER_LABEL : string;
                  end;

                  node_class EXPR is_a ANY
                     derived
                        Prefix : text;
                        Infix : text;
                        Postfix : text;
                     redef_derived
                        GRAPH_BROWSER_LABEL = string ( self.Infix );
                  end;

                  node_class OPERATION is_a EXPR
                     intrinsic
                        Op : string;
                  end;

                  node_class BINARY_OPERATION is_a OPERATION
                     redef_derived
                        Prefix = ConcatText ( ConcatText ( Text ( self.Op ) && " ", self.-Operand1->.Prefix && " " ), self.-Operand2->.Prefix && " "
                                            )                                                                                                       ;
                        Infix = ConcatText ( self.-Operand1->.Infix && " " && self.Op && " ", self.-Operand2->.Infix );
                        Postfix = ConcatText ( self.-Operand1->.Postfix && " ", self.-Operand2->.Postfix && " " && self.Op );
                  end;

                  node_type Plus : BINARY_OPERATION
                     redef_intrinsic
                        Op := "+";
                  end;

                  node_type Minus : BINARY_OPERATION
                     redef_intrinsic
                        Op := "-";
                  end;

                  node_type Mult : BINARY_OPERATION
                     redef_intrinsic
                        Op := "*";
                  end;

                  node_type Div : BINARY_OPERATION
                     redef_intrinsic
                        Op := "/";
                  end;

                  node_class UNARY_OPERATION is_a OPERATION
                     redef_derived
                        Prefix = ConcatText ( Text ( self.Op ) && " ", self.-Operand1->.Prefix && " " );
                        Infix = ConcatText ( Text ( self.Op ) && " ", self.-Operand1->.Infix );
                        Postfix = self.-Operand1->.Postfix && " " && self.Op;
                  end;

                  node_type MinusSign : UNARY_OPERATION
                     redef_intrinsic
                        Op := "-";
                  end;

                  node_type PlusSign : UNARY_OPERATION
                     redef_intrinsic
                        Op := "+";
                  end;

                  node_class VALUE is_a EXPR
                     intrinsic
                        Literal : string;
                     derived
                        Value : integer = 0;
                     redef_derived
                        Prefix = Text ( self.Literal );
                        Infix = Text ( self.Literal );
                        Postfix = Text ( self.Literal );
                  end;

                  node_type Variable : VALUE end;

                  node_type Const : VALUE end;

                  node_type Number : VALUE
                     redef_derived
                        Value = value ( self.Literal, integer );
                  end;

                  edge_type Operand1 : OPERATION [0:n] -> EXPR [1:1];

                  edge_type Operand2 : BINARY_OPERATION [0:n] -> EXPR [1:1];

            end;

            section Operations

               declares

                  transformation ExprValue ( valueType : type_in VALUE ; lit : string ; out outExpr : VALUE)
                     =

                     (<OptLeftSideList>)
                     (* @CC_LG 7008C(39,33) @*)
                     ::=
                     begin
                        1' : valueType;
                        (* @CC_LG 6012(72,28)(107,21) @*)
                     end
                     (* @CC_LG 7009C(147,61) @*)
                     transfer 1'.Literal := lit;
                     return outExpr := 1';
                  end;

                  transformation ExprUnary ( unaryOpType : type_in UNARY_OPERATION ; inExpr : EXPR ; out outExpr : UNARY_OPERATION)
                     =

                     begin
                        obl_node `1 = inExpr;
                        (* @CC_LG 6007(60,28)(89,21) @*)
                     end
                     (* @CC_LG 7008C(123,61) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(72,28)(65,21) @*)
                        2' : unaryOpType;
                        (* @CC_LG 6012(72,126)(119,21) @*)
                        2' -> 1' : Operand1;
                        (* @CC_LG 3004C @*)
                     end
                     (* @CC_LG 7009C(147,161) @*)
                     return outExpr := 2';
                  end;

                  transformation ExprBinary ( binaryOpType : type_in BINARY_OPERATION ; inExpr1, inExpr2 : EXPR ; out outExpr : BINARY_OPERATION)
                     =

                     begin
                        obl_node `1 = inExpr1;
                        (* @CC_LG 6007(60,28)(95,21) @*)
                        obl_node `2 = inExpr2;
                        (* @CC_LG 6007(204,28)(95,21) @*)
                     end
                     (* @CC_LG 7008C(265,61) @*)
                     ::=
                     begin
                        obl_node 1' = `1;
                        (* @CC_LG 6007(60,28)(65,21) @*)
                        obl_node 2' = `2;
                        (* @CC_LG 6007(204,28)(65,21) @*)
                        3' : binaryOpType;
                        (* @CC_LG 6012(132,126)(125,21) @*)
                        3' -> 1' : Operand1;
                        (* @CC_LG
                        3004C3
                        84,84 @*)
                        3' -> 2' : Operand2;
                        (* @CC_LG
                        3004C3
                        180,84 @*)
                     end
                     (* @CC_LG 7009C(253,161) @*)
                     return outExpr := 3';
                  end;

            end;

            transformation Main =

               use e1, e2, e3, e4, e5 : EXPR
               do
                  ExprValue ( Number, "5", out e1 )
                  & ExprUnary ( MinusSign, e1, out e2 )
                  & ExprValue ( Variable, "c", out e3 )
                  & ExprBinary ( Mult, e3, e2, out e4 )
                  & ExprBinary ( Plus, e4, e1, out e5 )
                               (* Finally, force evaluation of derived attributes *)
                  & not IsEqualText ( e5.Prefix, EmptyText )
                  & not IsEqualText ( e5.Infix, EmptyText )
                  & not IsEqualText ( e5.Postfix, EmptyText )
               end
            end;

      end;

end.
(* @CC_LG 7012C(1101,806) @*)
