spec nDimCube (* This specification shows how to create a
              (n+1)-dimensional cube from an n-dimensional
              cube. It uses the new '+'-production to apply
              one production to all of its matches in the
              given graph. *)
              (* @CC_LT HoriWithSpace @*)

   declares

      node_class VERTEX end;
      (* @CC_LG 6013(96,56)(97,87) @*)

      node_type v : VERTEX
         (<OptAttDeclList>)
         (* @CC_LG 3006C @*)
      end;
      (* @CC_LG 6015(96,212)(97,61) @*)

      edge_type e : VERTEX -> VERTEX;
      (* @CC_LG
      3004M5
      120,112
      192,112
      192,56 @*)

   transformation Create0DimCube (* This production creates a 0-dimensional cube
                                 with one node.
                                 *)                                              =
      (<OptPrecondDecl>)
      (* @CC_LG 7008(8,8)(24,31) @*)

      (<OptLeftSideList>)
      (* @CC_LG 7008(8,8)(24,24) @*)
      ::=
      begin
         1' : v;
         (* @CC_LG 6012(48,28)(59,21) @*)
      end
      (* @CC_LG 7009(8,8)(101,61) @*)
   end;
   (* @CC_LT VertWithSpace @*)

      transformation AddDim ( n : integer) =

         use new : v [1:n]
         do
            AddNewDimNode ( out new )
            & choose
                 when (n > 1)
                 then
                    ConnectNewDim ( new )
              else
                 skip
              end
         end
         (* Here the cube is extended to the dimension n. This is
         done by appending a new edge with a new node to all
         nodes of the cube. Then each of these new nodes is
         connected with its 'brother' at the neighbour-node.
         Exception : For the dimension 2 this last step is not
         needed, because a 2-dimensional cube consits of two
         nodes and one edge   *)
      end;

      transformation AddNewDimNode ( out new : v [0:n]) +
                                                        (* Append a new edge with a new node to all nodes of the cube.
                                                        To do so, the specified production must be applied to all
                                                        matches in the graph.
                                                        NOTE : This is done by the new '+'-production. All new nodes
                                                        are collected in the set 'new' and then returned.
                                                        *)
         =

         begin
            obl_node `1 : v;
            (* @CC_LG 6007(48,26)(59,21) @*)
         end
         (* @CC_LG 7008(8,8)(93,57) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(104,132)(65,21) @*)
            2' : v;
            (* @CC_LG 6012(106,42)(59,21) @*)
            1' -> 2' : e (* @CC_LG 1002C,BC(0,-9)(6,10) @*);
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(8,8)(149,163) @*)
         return new := 2';
      end;

      transformation ConnectNewDim ( new_nodes : VERTEX [1:n]) +
                                                               (* Connects the new nodes, so that the cube has a new dimension.
                                                               Therefore two new nodes are searched with their father-nodes
                                                               connected by an edge, but not themeselves.
                                                               NOTE :To do this with all new nodes, the production is applied
                                                               to all matches in the graph. This is specified with the new
                                                               '+'-operator.
                                                               *)
         =

         begin
            obl_node `1 : VERTEX;
            (* @CC_LG 6007(60,40)(83,21) @*)
            obl_node `2 = elem ( new_nodes );
            (* @CC_LG 6007(251,40)(155,21) @*)
            obl_node `3 : VERTEX;
            (* @CC_LG 6007(60,160)(83,21) @*)
            obl_node `4 = elem ( new_nodes );
            (* @CC_LG 6007(251,160)(155,21) @*)
            `1 -> `3 : e;
            (* @CC_LG 3004C @*)
            `1 -> `2 : e;
            (* @CC_LG 3004C @*)
            `3 -> `4 : e;
            (* @CC_LG 3004C @*)
            `2 +> `4 : e;
            (* @CC_LG 3008C @*)
         end
         (* @CC_LG 7008(341,187) @*)
         ::=
         begin
            obl_node 1' = `1;
            (* @CC_LG 6007(48,40)(59,21) @*)
            obl_node 2' = `2;
            (* @CC_LG 6007(198,40)(59,21) @*)
            obl_node 3' = `3;
            (* @CC_LG 6007(48,160)(59,21) @*)
            obl_node 4' = `4;
            (* @CC_LG 6007(198,160)(59,21) @*)
            1' -> 3' : e (* @CC_LG 1002C,BC(0,-9)(6,10) @*);
            (* @CC_LG 3004C @*)
            1' -> 2' : e (* @CC_LG 1002C,BC(0,-9)(6,10) @*);
            (* @CC_LG 3004C @*)
            3' -> 4' : e (* @CC_LG 1002C,BC(0,-9)(6,10) @*);
            (* @CC_LG 3004C @*)
            2' -> 4' : e;
            (* @CC_LG 3004C @*)
         end
         (* @CC_LG 7009(245,187) @*)
      end;

   transformation MAIN (* Here a 0-dimensional cube is created
                       first. Then the dimension is incremented
                       step by step.
                       *)
                       (* @CC_LT HoriWithSpace @*)              =

      Create0DimCube
      & use n : integer := 1
        do
           loop
              AddDim ( n )
              & n := (n + 1)
           end
        end
   end;
   (* @CC_LT VertWithSpace @*)

end.
(* @CC_LG 7012C(222,258) @*)
