/*********************************************/
/* With kind permission from Curtis Hrischuk */
/*********************************************/

/*************************************************************************
** $Source: /home/projects/prom3/CVS/develop/ImportedFunctions/AngioUtils.c,v $
** $Revision: 1.1 $
** $Date: 1998/01/16 13:59:46 $
** $State: Exp $
*************************************************************************/
/*
** AuStoreNodeBegin -		Stores a begin node for later retrieval
** AuStoreNodeExternal -	Stores an external node for later retrieval
** AuStoreNodeEnd -		Storead an end node
** AuStoreNodeActivity -	Stores an activity node
** AuStoreNodeJoin -		Stores a join node
** AuStoreNodeFork -		Stores a fork node
** AuStoreNodeEnty -		stores an entry node
** AuStoreNodeRequest -		Stores a request node
** AuStoreNodePhase -		Stores a phase node
** AuStoreNodeReply -		Stores a reply node
** AuStoreEdgeStart -		Stores a start application edge
** AuStoreEdgeNextTask -	Stores a next task edge
** AuStoreEdgeNextApp -		Stores a next applition thread_s node
** AuStoreEdgeNextPeriod -	Stores a next task period edge
** AuStoreEdgeFork -		Stores a fork edge
** AuStoreEdgeFwd -		stores a forward edge
** AuStoreEdgeRpc -		Stores an RPC edge
** AuStoreEdgeAsy -		Stores an asynchronous edge
** AuStoreEdgeReply -		Stores a reply edge
** AuDebugSet -			turns debugging on or off based and returns the set value
**
*************************************************************************/
/*
**
**	ATTENTION: please SET environment Variable "IMPORTLIB" according       
**	           to the library containing the object code of the            
**	           imported types and functions.                               
**	
**	
**	Note further that an imported type needs to be provided with at 
**	least three pre-defined functions that are needed for PROGRES built-in 
**	functionality.
**	    (let USERTYPE be the type to be imported)
**	     - USERTYPE_Size returns the size of the type in the parameter
**	     - USERTYPE_ValueToString writes the contents of the imported type to a string, 
**	       that will be returned in the first parameter
**	     - USERTYPE_StringToValue converts a string to USERTYPE and returns it by
**	       parameter
**	
**	Remark: The formerly needed procedures WriteConst and ReadConst are obsolete!
**	
**	Further user-defined functions for an imported type have the following rules: 
**	Assume that you want to import a function in the specification:
**	        MyImportedFunction : (T1, ..., Tn) -> Tr
**	        where types T1, ..., Tn, Tr are imported or built-in types.
**	The procedure implementing the function above must then look like:
**	        PROCEDURE MyImportedFunction (Tr *pr, T1 *p1, ..., Tn *pn);
**	        p1, ..., pn are the in-paramters and pr is the result of the function.
**	   
**	Note: All parameters must be passed by address (for technical reasons)
**	
*/
/***********************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "FileUtils.h"
#include "AngioUtils.h"


/***********************************************************/
#ifndef lint
static char *RCSid = "$Header: /home/projects/prom3/CVS/develop/ImportedFunctions/AngioUtils.c,v 1.1 1998/01/16 13:59:46 prom3 Exp $";
#endif

static int debug = 1;                                           /* Set to 0 to not print out information to stderr */

/*
**********************************************************************
** @NAME: AuStoreNodeBegin -	Stores a begin node for later retrieval
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/

void
AuStoreNodeBegin(int* ret_val, FUFile* fh, PGCMString* id, PGCMString* task_id, int* task_per)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t\"%-s\"\t%-4d\n",
                                  "Be",
                                        *id, 
                                               *task_id,
                                                       *task_per);
        }       

        ret = fprintf(*fh, "%-4s\t%-4s\t\"%-s\"\t%-4d\n",
                            "Be",
                                   *id, 
                                               *task_id,
                                                       *task_per);

        if (ret < 0) 
                fprintf(stderr, "Problem storing begin node to file\n");

        *ret_val = ret;
}

/*
**********************************************************************
** @NAME: AuStoreNodeExternal -	Stores an external node for later retrieval
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/

void
AuStoreNodeExternal(int* ret_val, FUFile* fh, PGCMString* id, PGCMString* app_name )
{
                                                                /* check to see if " is in the app_name */
        int   ret;
        char* format;

        if (strchr((*app_name).s , "\"") != NULL) {
                                                                /* The double quote is already there so don't add it */
          format = "%-4s\t%-4s\t%-s\n";
        } else {
          format = "%-4s\t%-4s\t\"%-s\"\n";
        }
          
        if (debug) {
                fprintf(stderr, format,
                                  "Ex",
                                             *id, 
                                               (*app_name).s);


        }
        ret = fprintf(*fh, format,
                          "Ex",
                                     *id,
                                         (*app_name).s);

        if (ret < 0) 
                fprintf(stderr, "Problem storing external node to file\n");

        *ret_val = ret;
}

/*
**********************************************************************
** @NAME: AuStoreNodeEnd -	Storead an end node
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreNodeEnd(int* ret_val, FUFile* fh, PGCMString* id, PGCMString* task_name, int* task_per)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t\"%-s\"\t%-4d\n",
                                  "En",
                                     *id, 
                                               *task_name,
                                                       *task_per);

        }
        ret = fprintf(*fh, "%-4s\t%-4s\t\"%-s\"\t%-4d\n",
                                  "En",
                                     *id, 
                                               *task_name,
                                                       *task_per);

        if (ret < 0) 
                fprintf(stderr, "Problem storing end node to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreNodeActivity -	Stores an activity node
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreNodeActivity(int* ret_val, FUFile* fh, PGCMString* id, PGCMString* task_name, int* task_per, PGCMString* outString)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t\"%-s\"\t%-4d\t%s\n",
                                          "Ac",
                                             *id, 
                                               *task_name,
                                                       *task_per,
                                                            (*outString).s);

        }
        ret = fprintf(*fh, "%-4s\t%-4s\t\"%-s\"\t%-4d\t%s\n",
                                  "Ac",
                                     *id, 
                                               *task_name,
                                                       *task_per,
                                                         (*outString).s);

        if (ret < 0) 
                fprintf(stderr, "Problem storing activity node to file\n");

        if (!*fh) 
                fprintf(stderr, "Null file pointer\n");


        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreNodeActivity -	Stores an activity node
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreNodeActivityProper(int* ret_val, FUFile* fh, PGCMString* id, PGCMString* outString)
{
        int ret;
        if (debug) {
                fprintf(stderr, "Ac\t%s\t%s\n",
                                      (*id).s, 
                                            (*outString).s);

        }
        ret = fprintf(*fh, "Ac\t%s\t%s\n",
                                  (*id).s, 
                                        (*outString).s);

        if (ret < 0) 
                fprintf(stderr, "Problem storing activity node to file\n");

        if (!*fh) 
                fprintf(stderr, "Null file pointer\n");


        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreNodeJoin -	Stores a join node
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreNodeJoin(int* ret_val, FUFile* fh, PGCMString* id, PGCMString* task_name, int* task_per)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t\"%-s\"\t%-4d\n",
                                  "Jo",
                                     *id, 
                                               *task_name,
                                                       *task_per);

        }
        ret = fprintf(*fh, "%-4s\t%-4s\t\"%-s\"\t%-4d\n",
                                  "Jo",
                                     *id, 
                                               *task_name,
                                                       *task_per);

        if (ret < 0) 
                fprintf(stderr, "Problem storing and-join node to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreNodeFork -	Stores a fork node
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreNodeFork(int* ret_val, FUFile* fh, PGCMString* id, PGCMString* task_name, int* task_per)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t\"%-s\"\t%-4d\n",
                                    "Fo",
                                             *id, 
                                               *task_name,
                                                       *task_per);
        }
        ret = fprintf(*fh, "%-4s\t%-4s\t\"%-s\"\t%-4d\n",
                                  "Fo",
                                     *id, 
                                               *task_name,
                                                       *task_per);

        if (ret < 0) 
                fprintf(stderr, "Problem storing and-fork node to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreNodeEntry -	stores an entry node
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/

void
AuStoreNodeProper(int* ret_val, FUFile* fh, PGCMString* type, PGCMString* id)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-s\t%-s\n",
                                   (*type).s,
                                             (*id).s);
        }
        ret = fprintf(*fh, "%-s\t%-s\n",
                             (*type).s,
                                     (*id).s);

        if (ret < 0) 
                fprintf(stderr, "Problem storing proper time node %s to file\n", *type);

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreEdgeI -	Stores an edge type when passed integer parameters
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/

void	
AuStoreEdgeProper(int* ret_val, FUFile* fh, PGCMString* fromid, PGCMString* toid, PGCMString* type)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t%-4s\t\n",
                                   *type,       
                                          *fromid,
                                             *toid);
        }
        ret = fprintf(*fh, "%-4s\t%-4s\t%-4s\t\n",
                             *type,
                                     *fromid,
                                        *toid);

        if (ret < 0) 
                fprintf(stderr, "Problem storing start edge to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreEdgeStart -	Stores a start application edge
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreEdgeStart(int* ret_val, FUFile* fh, PGCMString* fromid, PGCMString* toid)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t%-4s\n",
                                          "ESt",
                                             *fromid,
                                                *toid);
        }
        ret = fprintf(*fh, "%-4s\t%-4s\t%-4s\n",
                                  "ESt",
                                     *fromid,
                                        *toid);

        if (ret < 0) 
                fprintf(stderr, "Problem storing start edge to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreEdgeNextTask -	Stores a next task edge
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreEdgeNextTask(int* ret_val, FUFile* fh, PGCMString* fromid, PGCMString* toid)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t%-4s\n",
                                          "ENt",
                                             *fromid,
                                                *toid);
        }
        ret = fprintf(*fh, "%-4s\t%-4s\t%-4s\n",
                                  "ENt",
                                     *fromid,
                                        *toid);

        if (ret < 0) 
                fprintf(stderr, "Problem storing next task edge to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreEdgeNextApp -	Stores a next applition thread_s node
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreEdgeNextApp(int* ret_val, FUFile* fh, PGCMString* fromid, PGCMString* toid)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t%-4s\n",
                                          "ENa",
                                             *fromid,
                                                *toid);
        }
        ret = fprintf(*fh, "%-4s\t%-4s\t%-4s\n",
                                  "ENa",
                                     *fromid,
                                        *toid);

        if (ret < 0) 
                fprintf(stderr, "Problem storing next application edge to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreEdgeNextPeriod -	Stores a next task period edge
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreEdgeNextPeriod(int* ret_val, FUFile* fh, PGCMString* fromid, PGCMString* toid)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t%-4s\n",
                                  "ENp",
                                     *fromid,
                                        *toid);
        }
        ret = fprintf(*fh, "%-4s\t%-4s\t%-4s\n",
                                  "ENp",
                                     *fromid,
                                        *toid);

        if (ret < 0) 
                fprintf(stderr, "Problem storing next task period edge to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @NAME: AuStoreEdgeFork -	Stores a fork edge
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/


void
AuStoreEdgeFork(int* ret_val, FUFile* fh, PGCMString* fromid, PGCMString* toid)
{
        int ret;
        if (debug) {
                fprintf(stderr, "%-4s\t%-4s\t%-4s\n",
                                          "EFk",
                                             *fromid,
                                                *toid);
        }
        ret = fprintf(*fh, "%-4s\t%-4s\t%-4s\n",
                                  "EFk",
                                     *fromid,
                                        *toid);

        if (ret < 0) 
                fprintf(stderr, "Problem storing fork edge to file\n");

        *ret_val = ret;
}
/*
**********************************************************************
** @Name: AuDebugSet -	turns debugging on or off based and returns the set value
** 
** @DESCRIPTION:
** 
** 
** @CLASS:	Public System
** 
** @SYNOPSIS:
** 
** 
** @RETURN:
** 
** 
*/

void
AuDebugSet(BOOLEAN * set, BOOLEAN * ret ) 
{
        if (debug) fprintf(stderr, "AUDebugSet assigning debug value to %d\n", *set);
        *ret  = debug = *set;
}
/*************************************************************************
** $Header: /home/projects/prom3/CVS/develop/ImportedFunctions/AngioUtils.c,v 1.1 1998/01/16 13:59:46 prom3 Exp $
** $Log: AngioUtils.c,v $
** Revision 1.1  1998/01/16 13:59:46  prom3
** Neue importierte Funktionen aus Ottawa. -ajw
**
** Revision 1.7  1997/10/31 15:37:49  ceh
** backup
**
** Revision 1.6  1997/10/31 15:33:19  ceh
** backup
**
** Revision 1.5  1997/10/20 13:42:46  ceh
** backup
**
** Revision 1.4  1997/10/07 15:31:29  ceh
** backup
**
** Revision 1.3  1997/10/07 15:14:34  ceh
** backup
**
** Revision 1.2  1997/10/07 14:34:47  ceh
** backup
**
** Revision 1.1  1997/09/17 14:39:50  ceh
** backup
**
** Revision 1.1  1997/04/03 19:23:50  ceh
** Initial revision
**
**
**
*/

