/***************/
/* FileUtils.h */
/***************/

/************************************************/
/* header file for imported types and functions */
/************************************************/

/***************************************************************************/
/*                                                                         */
/*  ATTENTION: please SET environment Variable "IMPORTLIB" according       */
/*             to the library containing the object code of the            */
/*             imported types and functions.                               */
/*                                                                         */
/***************************************************************************/

/* Note further that an imported type needs to be provided with at 
   least three pre-defined functions that are needed for PROGRES built-in 
   functionality.
   * (let USERTYPE be the type to be imported)
   *  - USERTYPE_Size returns the size of the type in the parameter
   *  - USERTYPE_ValueToString writes the contents of the imported type
   *    to a string, that will be returned in the first parameter
   *  - USERTYPE_StringToValue converts a string to USERTYPE and returns it by
   *    parameter
   * Remark: The formerly needed procedures WriteConst and ReadConst are obsolete!

   * Further user-defined functions for an imported type have the following rules: 
   * Assume that you want to import a function in the specification:
   *   MyImportedFunction : (T1, ..., Tn) -> Tr
   * where types T1, ..., Tn, Tr are imported or built-in types.
   * The procedure implementing the function above must then look like:
   *   PROCEDURE MyImportedFunction (Tr *pr, T1 *p1, ..., Tn *pn);
   * p1, ..., pn are the in-paramters and pr is the result of the function.
   *
   * Note: All parameters must be passed by address (for technical reasons)
 */


#include "C_PGC.h"

#define MAXLINELENGTH 100
#define MAXNAMELENGTH 20
#define CharToInt(c) (c - '0')


/****************************************/
/* type definition of the imported type */
/****************************************/

typedef FILE *FUFile;


/******************************************************/
/* interface of standard functions for imported types */
/******************************************************/

/* The following three procedures are obligatory in connection
 * with an imported type. They must not be used in the specification
 * explicitely, but they are needed for internal use (e.g. conversion
 * from text to string and vice versa. */
/* Note: The procedure names and the parameters (type and order) are important.
 * The convention is, that in procedures results are returned as the first  
 * parameter. */

void FUFile_Size (int *);

void FUFile_ValueToString (PGCMString *, FUFile *);

void FUFile_StringToValue (FUFile *, PGCMString *);


/************************************************/
/* interface of user-defined imported functions */
/************************************************/

void FUOpenFile (FUFile *, PGCMString *);

void FUCreateFile (FUFile *, PGCMString *);

void FUIsFileOpen (BOOLEAN *, FUFile *);

void FUCloseFile (BOOLEAN *, FUFile *);

void FUReadLine (PGCMString *, FUFile *);

void FUIsEOF (BOOLEAN *, FUFile *);

void FUWriteLine (BOOLEAN *, FUFile *, PGCMString *);

void FUWriteText (BOOLEAN *, FUFile *, PGCMString *);
