/*****************/
/* LongStrings.h */
/*****************/

/************************************************/
/* header file for imported types and functions */
/************************************************/

/***************************************************************************/
/*                                                                         */
/*  ATTENTION: please SET environment Variable "IMPORTLIB" according       */
/*             to the library containing the object code of the            */
/*             imported types and functions.                               */
/*                                                                         */
/***************************************************************************/

/* Pay even more attention to the comment below, concerning type_size 
 * and curLength!
 * Note further that an imported type needs to be provided with at 
 * least three pre-defined functions that are needed for PROGRES built-in 
 * functionality.
 * (let USERTYPE be the type to be imported)
 *  - USERTYPE_Size returns the size of the type in the parameter
 *  - USERTYPE_ValueToString writes the contents of the imported type
 *    to a string, that will be returned in the first parameter
 *  - USERTYPE_StringToValue converts a string to USERTYPE and returns it by
 *    parameter
 * Remark: The formerly needed procedures WriteConst and ReadConst 
 * are obsolete!

 * Further user-defined functions for an imported type have the following 
 * rules.
 * Assume that you want to import a function in the specification like
 *   MyImportedFunction : (T1, ..., Tn) -> Tr
 * where types T1, ..., Tn, Tr are imported or built-in types.
 * The procedure implementing the function above must then look like:
 *   PROCEDURE MyImportedFunction (Tr *pr, T1 *p1, ..., Tn *pn);
 * p1, ..., pn are the in-paramters and pr is the result of the function.
 *
 * Note: All parameters must be passed by address (for technical reasons)
 */


#include "C_PGC.h"


/****************************************/
/* type definition of the imported type */
/****************************************/

#define MAXTEXTLENGTH 2044
 	                                      
typedef char BufStr[MAXTEXTLENGTH];

/* REALLY IMPORTANT: 
 * When using types with type_size > 251 bytes, the first 
 * 4 bytes MUST contain a 32-bit-integer with the currently 
 * used size of the regarded type instance (not counting
 * the length information itself). 
 * In the given example of the text type the component 
 * curLength needs to hold the current size of contents. 
 * For the empty text initialized with '\0' curLength will 
 * be 1 and for a text using the full length available it 
 * will be 2044 (which is not to be confused with the result 
 * of the text_Size procedure yielding the maximum size of 
 * any instance of type text).
 */ 

typedef struct _text
  {
    int curLength;    /* current size of contents */  
    BufStr contents;
  } text;


/******************************************************/
/* interface of standard functions for imported types */
/******************************************************/

/* The following three procedures are obligatory in connection
 * with an imported type. They must not be used in the specification
 * explicitely, but they are needed for internal use (e.g. conversion
 * from text to string and vice versa. 
 */
/* Note: The procedure names and the parameters (type and order) 
 * are important. The convention is, that in procedures results are 
 * returned as the first parameter. 
 */

void text_Size (int *size);

void text_ValueToString (PGCMString *outString, text *value);

void text_StringToValue (text *value, PGCMString *inString);


/************************************************/
/* interface of user-defined imported functions */
/************************************************/

void Text (text * value, PGCMString *inString);

void EmptyText (text *outText);

void AmpersandAmpersand (text *outText, text *inText, PGCMString *inString);

void ConcatText (text *outText, text *inText1, text *inText2);

void EqualEqual (BOOLEAN *result, text *inText, PGCMString *inString);

void IsEqualText (BOOLEAN *result, text *inText1, text *inText2);
