/*****************/
/* RealNumbers.c */
/*****************/

/**************************************************/
/* implementation of imported types and functions */
/**************************************************/

/***************************************************************************/
/*                                                                         */
/*  ATTENTION: please SET environment Variable "IMPORTLIB" according       */
/*             to the library containing the object code of the            */
/*             imported types and functions.                               */
/*                                                                         */
/***************************************************************************/

/* Note further that an imported type needs to be provided with at 
 * least three pre-defined functions that are needed for PROGRES built-in 
 * functionality.
 * (let USERTYPE be the type to be imported)
 *  - USERTYPE_Size returns the size of the type in the parameter
 *  - USERTYPE_ValueToString writes the contents of the imported type 
 *    to a string, that will be returned in the first parameter
 *  - USERTYPE_StringToValue converts a string to USERTYPE and returns it by
 *    parameter
 * Remark: The formerly needed procedures WriteConst and ReadConst 
 * are obsolete!

 * Further user-defined functions for an imported type have the following 
 * rules: 
 * Assume that you want to import a function in the specification like
 *   MyImportedFunction : (T1, ..., Tn) -> Tr
 * where types T1, ..., Tn, Tr are imported or built-in types.
 * The procedure implementing the function above must then look like:
 *   PROCEDURE MyImportedFunction (Tr *pr, T1 *p1, ..., Tn *pn);
 * p1, ..., pn are the in-paramters and pr is the result of the function.
 *
 * Note: All parameters must be passed by address (for technical reasons)
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* use type definition of reals in the header file */
#include "RealNumbers.h"


/***********************************************************/
/* implementation of standard functions for imported types */
/***********************************************************/

/* The following three procedures are obligatory in connection 
 * with an imported type. They must not be used in the specification
 * explicitely, but they are needed for internal use (e.g. conversion
 * from text to string and vice versa. 
 */
/* Note: The procedure names and the parameters (type and order) 
 * are important. The convention is, that in procedures results are 
 * returned as the first parameter. 
 */

void Real_Size (int *size)
/* tell only the size of the real type */
{
  *size = sizeof (Real);
}

void Real_ValueToString (PGCMString *outString, Real *value) 
{
  sprintf((*outString).s, "%.8g", *value);
}

void Real_StringToValue (Real *value, PGCMString *inString) 
{
  *value = (Real) atof((*inString).s);
}


/*****************************************************/
/* implementation of user-defined imported functions */
/*****************************************************/

void R_0 (Real *outvalue)
{
  *outvalue = 0.0;
}

void R_Pi (Real *outvalue)
{
  *outvalue = 3.141592654;
}

void R_e (Real *outvalue)
{
  *outvalue = 2.718281828;
}

void R_Minus (Real *outvalue, Real *invalue)
{
  *outvalue = - *invalue;
}

void BQPlusQuote (Real *outvalue, Real *invalue1, Real *invalue2)
{
  *outvalue = *invalue1 + *invalue2;
}

void BQMinusQuote (Real *outvalue, Real *invalue1, Real *invalue2)
{
  *outvalue = *invalue1 - *invalue2;
}

void BQStarQuote (Real *outvalue, Real *invalue1, Real *invalue2)
{
  *outvalue = *invalue1 * *invalue2;
}

void BQSlashQuote (Real *outvalue, Real *invalue1, Real *invalue2)
{
  *outvalue = *invalue1 / *invalue2;
}

void BQEqualQuote (BOOLEAN *outvalue, Real *invalue1, Real *invalue2)
{
  *outvalue = (*invalue1 == *invalue2);
}

void R_RealToInt (int *outvalue, Real *invalue)
{
  *outvalue = (int) *invalue;
}

void R_Ceil (int *outvalue, Real *invalue)
{
  *outvalue = ceil(*invalue);
}

void R_Floor (int *outvalue, Real *invalue)
{
  *outvalue = floor(*invalue);
}

void R_IntToReal(Real *outvalue, int *invalue)
{
  *outvalue = (Real) *invalue;
}

void R_InitReal(Real *outvalue, int *in1, int *in2, int *in3)
{
  char buf[20];

  sprintf(buf,"%d.%de%d", *in1, *in2, *in3);
  *outvalue = atof(buf);
}

void R_Square (Real *outvalue, Real *invalue)
{
  *outvalue = *invalue * *invalue;
}

void R_Sqrt (Real *outvalue, Real *invalue)
{
  *outvalue = sqrt(*invalue);
}

void R_Log (Real *outvalue, Real *invalue)
{
  *outvalue = log(*invalue);
}

void R_Pow (Real *outvalue, Real *invalue1, Real *invalue2)
{
  *outvalue = pow(*invalue1, *invalue2);
}

void R_Exp (Real *outvalue, Real *invalue)
{
  *outvalue = exp(*invalue);
}

void R_Sin (Real *outvalue, Real *invalue)
{
  *outvalue = sin(*invalue);
}

void R_Cos (Real *outvalue, Real *invalue)
{
  *outvalue = cos(*invalue);
}

void R_Tan (Real *outvalue, Real *invalue)
{
  *outvalue = tan(*invalue);
}

