/*****************/
/* RealNumbers.h */
/*****************/

/************************************************/
/* header file for imported types and functions */
/************************************************/

/***************************************************************************/
/*                                                                         */
/*  ATTENTION: please SET environment Variable "IMPORTLIB" according       */
/*             to the library containing the object code of the            */
/*             imported types and functions.                               */
/*                                                                         */
/***************************************************************************/

/* Pay even more attention to the comment below, concerning type_size 
 * and curLength!
 * Note further that an imported type needs to be provided with at 
 * least three pre-defined functions that are needed for PROGRES built-in 
 * functionality.
 * (let USERTYPE be the type to be imported)
 *  - USERTYPE_Size returns the size of the type in the parameter
 *  - USERTYPE_ValueToString writes the contents of the imported type
 *    to a string, that will be returned in the first parameter
 *  - USERTYPE_StringToValue converts a string to USERTYPE and returns it by
 *    parameter
 * Remark: The formerly needed procedures WriteConst and ReadConst 
 * are obsolete!

 * Further user-defined functions for an imported type have the following 
 * rules.
 * Assume that you want to import a function in the specification like
 *   MyImportedFunction : (T1, ..., Tn) -> Tr
 * where types T1, ..., Tn, Tr are imported or built-in types.
 * The procedure implementing the function above must then look like:
 *   PROCEDURE MyImportedFunction (Tr *pr, T1 *p1, ..., Tn *pn);
 * p1, ..., pn are the in-paramters and pr is the result of the function.
 *
 * Note: All parameters must be passed by address (for technical reasons)
 */


#include "C_PGC.h"


/****************************************/
/* type definition of the imported type */
/****************************************/

/* REALLY IMPORTANT: 
 * When using types with type_size > 251 bytes, the first 
 * 4 bytes MUST contain a 32-bit-integer with the currently 
 * used size of the regarded type instance (not counting
 * the length information itself). 
 * In the given example of the text type the component 
 * curLength needs to hold the current size of contents. 
 * For the empty text initialized with '\0' curLength will 
 * be 1 and for a text using the full length available it 
 * will be 2044 (which is not to be confused with the result 
 * of the text_Size procedure yielding the maximum size of 
 * any instance of type text).
 */ 

typedef double Real;


/******************************************************/
/* interface of standard functions for imported types */
/******************************************************/

/* The following three procedures are obligatory in connection
 * with an imported type. They must not be used in the specification
 * explicitely, but they are needed for internal use (e.g. conversion
 * from text to string and vice versa. 
 */
/* Note: The procedure names and the parameters (type and order) 
 * are important. The convention is, that in procedures results are 
 * returned as the first parameter. 
 */

void Real_Size (int *size);

void Real_ValueToString (PGCMString *outString, Real *value);

void Real_StringToValue (Real *value, PGCMString *inString);


/************************************************/
/* interface of user-defined imported functions */
/************************************************/

void R_0 (Real *outvalue);

void R_Pi (Real *outvalue);

void R_e (Real *outvalue);

void R_Minus (Real *outvalue, Real *invalue);

void BQPlusQuote (Real *outvalue, Real *invalue1, Real *invalue2);

void BQMinusQuote (Real *outvalue, Real *invalue1, Real *invalue2);

void BQStarQuote (Real *outvalue, Real *invalue1, Real *invalue2);

void BQSlashQuote (Real *outvalue, Real *invalue1, Real *invalue2);

void BQEqualQuote (BOOLEAN *outvalue, Real *invalue1, Real *invalue2);

void R_RealToInt (int *outvalue, Real *invalue);

void R_IntToReal(Real *outvalue, int *invalue);

void R_InitReal(Real *outvalue, int *in1, int *in2, int *in3);

void R_Square (Real *outvalue, Real *invalue);

void R_Sqrt (Real *outvalue, Real *invalue);

void R_Log (Real *outvalue, Real *invalue);

void R_Sin (Real *outvalue, Real *invalue);

void R_Cos (Real *outvalue, Real *invalue);

void R_Tan (Real *outvalue, Real *invalue);
