/*****************/
/* StringUtils.c */
/*****************/

/**************************************************/
/* implementation of imported types and functions */
/**************************************************/

/***************************************************************************/
/*                                                                         */
/*  ATTENTION: please SET environment Variable "IMPORTLIB" according       */
/*             to the library containing the object code of the            */
/*             imported types and functions.                               */
/*                                                                         */
/***************************************************************************/

/* Note further that an imported type needs to be provided with at 
   least three pre-defined functions that are needed for PROGRES built-in 
   functionality.
   * (let USERTYPE be the type to be imported)
   *  - USERTYPE_Size returns the size of the type in the parameter
   *  - USERTYPE_ValueToString writes the contents of the imported type to a string, 
   *    that will be returned in the first parameter
   *  - USERTYPE_StringToValue converts a string to USERTYPE and returns it by
   *    parameter
   * Remark: The formerly needed procedures WriteConst and ReadConst are obsolete!

   * Further user-defined functions for an imported type have the following rules: 
   * Assume that you want to import a function in the specification:
   *   MyImportedFunction : (T1, ..., Tn) -> Tr
   * where types T1, ..., Tn, Tr are imported or built-in types.
   * The procedure implementing the function above must then look like:
   *   PROCEDURE MyImportedFunction (Tr *pr, T1 *p1, ..., Tn *pn);
   * p1, ..., pn are the in-paramters and pr is the result of the function.
   *
   * Note: All parameters must be passed by address (for technical reasons)
 */


#include <stdio.h>
#include <stdlib.h>

/* use type definition of text in the header file */
#include "StringUtils.h"

#define MAX_STR_SIZE 200

/* header of local functions */


/***********************************************************/
/* implementation of standard functions for imported types */
/***********************************************************/

/* The following three procedures are obligatory in connection 
 * with an imported type. They must not be used in the specification
 * explicitely, but they are needed for internal use (e.g. conversion
 * from text to string and vice versa. */
/* Note: The procedure names and the parameters (type and order) are important.
 * The convention is, that in procedures results are returned as the first  
 * parameter. */

void StringList_Size (int *size)
{
	*size = sizeof (StringList);
}


void StringList_ValueToString (PGCMString *outString, StringList *sl)
/* string representation for type StringList -its not save due to
   the use of sprintf - we can't guarantee to prevent a buffer overrun */
{
	char	copy[MAX_STR_SIZE];
	int     i, pos;

	if (*sl) {
		pos = sprintf ( &(copy[0]), "< ");
		for (i=0; i<(*sl)->numberOfElements; i++) {

			if ( (pos + strlen((*sl)->list[i]) +3) >= MAX_STR_SIZE) {
				pos = sprintf(&copy[0], "< too long ");
				break;
			}

			pos += sprintf ( &(copy[pos]), "%s ", (*sl)->list[i] );

		}
		pos += sprintf (&copy[pos], ">");
	}
	else 
		sprintf (&copy[0], "< >");

	strcpy( (*outString).s, copy);
}


void StringList_StringToValue (StringList *sl, PGCMString *inString)
/* Make list out of single string - use init or split instead */

{
	*sl = NULL;
}


/*****************************************************/
/* implementation of user-defined imported functions */
/*****************************************************/

void SLinit(StringList* sl)

{
	(*sl) = malloc(sizeof(struct _StringList));
	(*sl)->numberOfElements = (*sl)->iterator = 0;
}

void SLsplit(StringList* sl, PGCMString* disc, PGCMString* pgcm_str)

{
	char	copy[MAX_STR_SIZE];
	char	discriminator = (*disc).s[0];
	int		i=0, j=0, n=0;
	char*	str = (*pgcm_str).s;

	(*sl) = malloc(sizeof(struct _StringList));
	(*sl)->iterator = 0;

	if (str[0] == 0) {
		(*sl)->numberOfElements = 0;
		return;
	}

	for (;;) {

		if ( (str[i] == discriminator) || (str[i] == 0) ) {
			copy[j++] = 0;
			(*sl)->list[n] = malloc(sizeof(char) * j);
			strcpy((*sl)->list[n], copy);
			j = 0;
			n++;
			if (str[i] == 0) break;
			i++;
		}
		else
			copy[j++] = str[i++];

		if ( (n >= MAX_LIST_SIZE) || (j >= MAX_STR_SIZE) ) {
			fprintf(stderr, "StringList: String too long or maximum number of list entries exceeded");
			free (*sl);
			*sl = NULL;
			break;
		}

	}

	(*sl)->numberOfElements = n;
}
	
void SLappend(BOOLEAN* ok, StringList* sl, PGCMString* str)

{
	int n;
	
	*ok = FALSE;
	if (*sl) {
	
		n = (*sl)->numberOfElements;

		if (n < MAX_LIST_SIZE) {

			(*sl)->list[n] = malloc (sizeof(char) * strlen ((*str).s));
			strcpy( (*sl)->list[n], (*str).s);

			(*sl)->numberOfElements++;
			*ok = TRUE;
		}
	}
}


void SLdeleteList(BOOLEAN* ok, StringList* sl)

{
	int		i;

	if (*sl) {

		for (i = 0; i < (*sl)->numberOfElements; i++)
			if ( (*sl)->list[i]) free( (*sl)->list[i]);

/* the variable *sl is a value variable and can't be set to nil. Not
   releasing the memory is very bad, but simple alternative ...
*/
		// free(*sl);
		(*sl)->numberOfElements = (*sl)->iterator = 0;
		*ok = TRUE;
	}
	else
		*ok = FALSE;
}


void SLresetIterator(BOOLEAN* ok, StringList* sl)

{
	if (*sl) {
		(*sl)->iterator = 0;
		*ok = TRUE;
	}
	else
		*ok = FALSE;
}


void SLisLast (BOOLEAN* ok, StringList* sl)

{
	if (*sl)
		*ok = ( (*sl)->iterator == (*sl)->numberOfElements );
	else
		*ok = FALSE;
}

void SLgetNext (PGCMString* str, StringList* sl)

{
	if (*sl) {
		if ( (*sl)->iterator < (*sl)->numberOfElements ) 
			strcpy((*str).s, (*sl)->list[(*sl)->iterator++]);
	}
	else
			strcpy((*str).s, "");
}
