/*****************/
/* StringUtils.h */
/*****************/

/************************************************/
/* header file for imported types and functions */
/************************************************/

/***************************************************************************/
/*                                                                         */
/*  ATTENTION: please SET environment Variable "IMPORTLIB" according       */
/*             to the library containing the object code of the            */
/*             imported types and functions.                               */
/*                                                                         */
/***************************************************************************/

/* Note further that an imported type needs to be provided with at 
   least three pre-defined functions that are needed for PROGRES built-in 
   functionality.
   * (let USERTYPE be the type to be imported)
   *  - USERTYPE_Size returns the size of the type in the parameter
   *  - USERTYPE_ValueToString writes the contents of the imported type
   *    to a string, that will be returned in the first parameter
   *  - USERTYPE_StringToValue converts a string to USERTYPE and returns it by
   *    parameter
   * Remark: The formerly needed procedures WriteConst and ReadConst are obsolete!

   * Further user-defined functions for an imported type have the following rules: 
   * Assume that you want to import a function in the specification:
   *   MyImportedFunction : (T1, ..., Tn) -> Tr
   * where types T1, ..., Tn, Tr are imported or built-in types.
   * The procedure implementing the function above must then look like:
   *   PROCEDURE MyImportedFunction (Tr *pr, T1 *p1, ..., Tn *pn);
   * p1, ..., pn are the in-paramters and pr is the result of the function.
   *
   * Note: All parameters must be passed by address (for technical reasons)
 */


#include "C_PGC.h"

#define MAX_LIST_SIZE 100

/****************************************/
/* type definition of the imported type */
/****************************************/

struct _StringList {
	int		numberOfElements;
	int		iterator;
	char*	list[MAX_LIST_SIZE];
};
typedef struct _StringList* StringList;


/******************************************************/
/* interface of standard functions for imported types */
/******************************************************/

/* The following three procedures are obligatory in connection
 * with an imported type. They must not be used in the specification
 * explicitely, but they are needed for internal use (e.g. conversion
 * from text to string and vice versa. */
/* Note: The procedure names and the parameters (type and order) are important.
 * The convention is, that in procedures results are returned as the first  
 * parameter. */

void StringList_Size (int *);

void StringList_ValueToString (PGCMString *, StringList *);

void StringList_StringToValue (StringList *, PGCMString *);


/************************************************/
/* interface of user-defined imported functions */
/************************************************/

/* Constructors */
void SLinit(StringList* sl);
void SLsplit(StringList* sl, PGCMString* disc, PGCMString* pgcm_str);

/* Destructor */
void SLdeleteList(BOOLEAN* ok, StringList* sl);

/* Append a string to the List */
void SLappend(BOOLEAN* ok, StringList* sl, PGCMString* str);

/* Iterate through the list */ 

void SLresetIterator(BOOLEAN* ok, StringList* sl);
/* true *after* the last element has been retrieved via SLgetNext */
void SLisLast (BOOLEAN* ok, StringList* sl);
void SLgetNext (PGCMString* str, StringList* sl);
