#!/bin/sh

# Script to remove common witespace prefix from miftext lines.

me=`basename $0`

if [ $# != 1 ] ; then
    echo "${me} ERROR: Unexpected argument count $#"
    echo "${me} Usage: $0 <filename>.tex"
    exit 1
else
    inFile="${1}" ; shift
fi

if [ ! -r "${inFile}" ] ; then
    echo "${me} ERROR: Input file '${inFile}' nonexistant or unreadable."
    exit 2
fi

outFile=`echo "${inFile}" | sed -n 's/\.tex$/.deindented.tex/p'`
if [ ! "${outFile}" ] ; then
    echo "${me} ERROR: Input file name '${inFile}' does not end in .tex."
    exit 3
fi

# Determine space prefixed to all miftext lines.

# The prefix of the lines indented least is the prefix shared by all
# lines, so that is determined using sort and head.

# The lines are picked with an address range, within which lines
# beginning with a space are reduced to their space prefix.  Since the
# environment-delimiting lines begin in the first column in the
# current version of the export, they do not have to be discarded
# specially.

prefix=`sed -n '/\\\\begin{miftext}/,/\\\\end{miftext}/ s/^\(  *\)[^ ].*$/\1/p' "${inFile}" \
    | sort -u \
    | head -1`

# Next, chip off the prefix.
sed '/\\begin{miftext}/,/\\end{miftext}/ s/^'"${prefix}"'//' \
    "${inFile}" \
    > "${outFile}"