/*********************************************************************/
/*                                                                   */
/* Copyright 1994/95/96 Carsten Friedrich, University of Passau.     */
/*                                                                   */
/* For details see the copyright file included in the distribution.  */
/* This notice must not be removed from the file.                    */
/*                                                                   */
/*********************************************************************/
// This may look like C code, but it is really -*- C++ -*-
/* 
Copyright (C) 1988 Free Software Foundation
    written by Doug Lea (dl@rocky.oswego.edu)

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef _VHSet_h
#ifdef __GNUG__
#pragma interface
#endif
#define _VHSet_h 1

#include "MFSet.h"

#ifndef DEFAULT_INITIAL_CAPACITY
#define DEFAULT_INITIAL_CAPACITY 5
#endif


template <class T1>
class VHSet : public MFSet<T1>
{
protected:
  T1*          tab;
  char*         status;
  unsigned int  size;

public:
                VHSet(unsigned int sz = DEFAULT_INITIAL_CAPACITY);
                VHSet(const VHSet& a);
                ~VHSet();

  Pix           add(T1  item);
  void          del(T1  item);
  int           contains(T1  item) const;

  void          clear();

  Pix           first() const;
  void          next(Pix& i) const;
  T1&          operator () (Pix i) const ;
  Pix           seek(T1  item) const;

  void          operator |= (const VHSet<T1>& b);
  void          operator -= (const VHSet<T1>& b);
  void          operator &= (const VHSet<T1>& b);

  int           operator == (const VHSet<T1>& b);
  int           operator != (const VHSet<T1>& b);
  int           operator <= (const VHSet<T1>& b); 

  int           capacity();
  void          resize(int newsize = 0);

  int           OK();
};

template <class T1>
inline VHSet<T1>::~VHSet()
{
  delete [] tab;
  delete status;
}

template<class T1>
inline int VHSet<T1>::capacity()
{
  return size;
}

template<class T1>
inline int VHSet<T1>::contains(T1  key) const
{
  return seek(key) != 0;
}

template<class T1>
inline T1& VHSet<T1>::operator () (Pix i) const
{
  if (i == 0) error("null Pix");
  return *((T1*)i);
}

template<class T1>
inline int VHSet<T1>::operator != (const VHSet<T1>& b)
{
  return ! ((*this) == b);
}

#endif
