%/*
% *
% *  Filter fuer die Management-Funktionen von GRAS.
% *
% */


%#include "xdr/prim.h"
%#include "xdr/other.h"

%/*
% * Erzeugen eines Graphpools.
% */
struct C_OpenGraphPool
{
  PoolDir   Poolpath;
  PoolName  Pool;
  GraphPoolMode mode;/* new, old, get*/
};

struct C_CloseGraphPool
{
  PoolDir  Poolpath;
  PoolName Pool;
};

%/*
% * Loeschen eines Graphpools.
% */
struct C_DeleteGraphPool
{
  PoolDir  Poolpath;
  PoolName Pool;
};

%/*
% * Graphpool reorganisieren.
% */
struct C_ReorganizeGraphPool
{
  PoolDir  Poolpath;
  PoolName Pool;
  PoolDir  execpath;
};


%/*
% * Ueberpruefung des Zustandes eines Graphen.
% */
struct C_CheckGraph
{
  PoolDir   Poolpath;
  PoolName  Pool;
  GraphName Graph;
};


%/*
% * Auslesen der im Pool vorhandenen Graphen.
% */
struct C_GetGraphsInGraphPool
{
  PoolDir  Poolpath;
  PoolName Pool;
};


%/*
% * Delta-Kopie eines Graphen.
% */
struct C_DeltaCopyGraph
{
  PoolDir   Poolpath;
  PoolName  Pool;
  GraphName SourceGraph;
  GraphName TargetGraph;
  BOOLEAN   Forward;
};

%/*
% * Informationen anhand der externen Graphnummer.
% */
struct C_GetInfoByExtNo
{
/*  ApplNumber   Appl;*/
  PoolDir      Poolpath;
  PoolName     Pool;
  ExternNumber ExtNo;
};

struct S_GetInfoByExtNo
{
  BOOLEAN     Existent;
  GraphName   Graphname;
  GraphType   Type;
  BOOLEAN     IsOpen;
  GraphNumber Graphnr;
  HostName    host;
};

%/*
% * Informationen anhand des Graphnamens.
% */
struct C_GetInfoByName
{
/*  ApplNumber Appl;*/
  PoolDir    Poolpath;
  PoolName   Pool;
  GraphName  Graph;
};

struct S_GetInfoByName
{
  BOOLEAN      Existent;
  GraphType    Type;
  ExternNumber ExtNo;
  BOOLEAN      IsOpen;
  GraphNumber  Graphnr;
  HostName     host;
};

#if 0
!!!
struct C_FullGraphName
{
  PoolDir   poolpath;
  PoolName  pool;
  GraphName graph;
};
#endif
%/*
% * Loeschen eines Graphen.
% */
struct C_DeleteGraph
{
  PoolDir   Poolpath;
  PoolName  Pool;
  GraphName Graph;
  BOOLEAN   Force;
};

%/*
% * GRAS-Server auf Signal des Oeffnungs-Servers terminieren.
% */

%/*
% * Erzeugen und Oeffnen von Graphen im neuen Zugriffsmodell.
% *
% * Das Oeffnen (und damit auch das Erzeugen) von Graphen wird ueber
% * ganimed serialisiert, d.h. alle Oeffnungsoperationen gehen nicht
% * direkt vom csci zum ssci, sondern ueber ganimed. Ob eine Anfrage
% * erfolgreich ist, entscheidet sich an mehreren Stellen:
% *
% * 1) Das csci stellt die erforderlichen Parameter zusammen
% *
% * 2) ganimed ueberprueft, ob der Graph schon einem gexed zugeordnet
% *    ist, ansonsten wird ein existierender gexed ausgewaehlt oder ein
% *    neuer gestartet. An diesen wird der Aufruf weitergereicht.
% *
% * 3) gexed ueberprueft, ob die Anfrage den Restriktionen des
% *    Zugriffsmodells genuegt.
% *
% * Das folgende Diagramm zeigt den Kontroll- und Datenfluss zwischen
% * den einzelnen Komponenten (IN-Parameter oben, OUT-Parameter unten;
% * p=einzelne Parameter, [CSG]=Praefix des Namens der Datenstruktur)
% *
% *        p          C             G          p
% * appl <---> csci <---> ganimed <---> ssci <---> GRAS
% *        p          S             S          p
% */


%/*
% * Informationen anhand des Graphnamens ueber Gruppen des Graphen.
% */
struct C_GetGroupInfo
{
  PoolDir   Poolpath;
  PoolName  Pool;
  GraphName Graph;
};

struct GroupInfo
{
  GroupName group;
  GroupType lockmode;
  BOOLEAN   onegroup;		/* Ist in jedem Item der Liste enthalten, */
};
				/* obwohl das nur 1 mal noetig waere. */

struct rg_list
{
  GroupInfo       group;
  struct rg_list *rg_next;
};

struct al_list
{
  ApplNumber     num;
  ApplName       name;
  HostName       host;
  struct al_list *next;
};



