%/*
% * Prototypen der 'primitiven XDR-Filter'.
% */

%#include <sys/param.h>
%#include <netdb.h>
%#include "xdr/base.h"

typedef CARDINAL LogicalNumberType;
typedef SHORTCARD IntDatabaseNumberType;
typedef CARDINAL LSETNumberType;
typedef CARDINAL LRELNumberType;


#ifdef RPC_HDR
%#define MAXCLIENTNUM   256

/* aus csci/global.h*/

%#define MaxRecordLength 251
%#define NameLength      MaxRecordLength
%#define MDBNLength      MaxRecordLength
%#define DBNameLength    MaxRecordLength

%typedef CHAR            NameStringType[NameLength];
%typedef char            HostName[MAXHOSTNAMELEN];


#endif

#define MaxRecordLength 251

#define NameLength      MaxRecordLength
#define MDBNLength      MaxRecordLength
#define DBNameLength    MaxRecordLength


/* aus agglobal.h */

/* If this list is changed, this *MUST* be reflected in RGGlobal.def */
/* Always append before the internally used enums (because they don't */
/* exist in RGGlobal.def and the correct interaction between M2 and C */
/* relies on the identity of the positions). */
enum TStatus
{
  NoError=0,
  AlreadyExistent,
  AlreadyOpen,
  NotExistent,
  Overflow,
  NotEmpty,
  Locked,
  NoDump,
  NoOriginal,
  NoDumpNoOriginal,
  IndexExistent,
  IncompatibleGraph,
  OldVersion,
  AttributeSizeError,
  NoEvalFunction,
  StillPendingTransaction,
  NoPendingTransaction,
  ActionsBlocked,
  ActionsNotBlocked,
  NoActionPerformer,
  RangeError,
  NoWriteAccess,
  InternalError,
  GroupModeError,
  RequestWouldBeDelayed,
  StillOpen,
  PoolNotOpen, 
  RequestDelayedNoWrite,	/* Wird nur intern verwendet */
  RequestDelayed,		/* Wird nur intern verwendet */
  NoServer 
};


typedef NameStringType  MDBNameType;
typedef NameStringType  DBNameType;



typedef NameStringType           AGString;

/*typedef CHAR            PoolDir[NameLength];*/

typedef AGString       PoolDir;

typedef AGString       ApplName;
typedef CARDINAL       t_clientid;


typedef MDBNameType		 PoolName;
typedef IntDatabaseNumberType	 GraphNumber;
typedef DBNameType		 GraphName;

typedef LogicalNumberType	 Surrogate;
typedef Surrogate		 ExternNumber;
typedef Surrogate		 GraphType;
typedef Surrogate		 NodeNumber;
typedef Surrogate		 SchemeNumber;
typedef Surrogate		 TypeNumber;
typedef Surrogate		 ApplNumber;
typedef LSETNumberType		 SimpleSet;
typedef LRELNumberType		 RelSet;
typedef Surrogate		 AttributeNumber;

/*
 * Deklarationen fuer das neue Graphzugriffsmodell.
 */
enum GraphPoolMode
{
  NewPool=0,			/* Error if already existent */
  OldPool,			/* Error if not existent */
  GetPool			/* open or create */
};

enum GraphMode
{
  NewGraph=0,			/* Error if already existent */
  OldGraph,			/* Error if not existent */
  GetGraph			/* open or create */
};

enum GroupMode
{
  NewGroup=0,
  OldGroup,
  GetGroup
};

enum GroupType
{
  Session=0,
  Transaction,
  Operation
};

typedef AGString GroupName;

enum AttributeKind
{
  Derived=0,
  Intrinsic,
  Dummy,
  Meta
};

enum IndexProperties
{
  Normal=0,
  Index,
  Key
};

enum DependencyKind
{
  SelfDependent=0,
  IncomingDependent,
  OutgoingDependent
};

typedef Surrogate ValueTypeNumber;

enum Cardinality
{
  OptUnique=0,
  OblUnique,
  OptSet,
  OblSet
};

#ifdef RPC_HDR
%typedef void (*EvalFunction)(GraphNumber, NodeNumber, AttributeNumber);
#endif

struct UniqueID
{
  Surrogate ID1;
  Surrogate ID2;
};

typedef Surrogate ActionPriority;

enum EventKind
{
    EVNewNode=0, 
    EVDeleteNode, 
    EVNewEdge, 
    EVDeleteEdge,
    EVModifyAttribute,
    EVIndexKeyModified,
    EVTransactionStart, 
    EVTransactionEnd, 
    EVSetCheckpoint,
    EVUndo, 
    EVRedo, 
    EVOpenGraph, 
    EVCloseGraph,
    EVUserDefined
};

enum EventMode
{
  EMRegular = 0,
  EMDuringUndoRedo,
  EMBoth
};

struct GraphEvent
{
  EventKind  Kind;
  EventMode  Mode;
  NodeNumber SNode;
  BOOLEAN    SNodeExists;
  NodeNumber TNode;
  BOOLEAN    TNodeExists;
  TypeNumber TNo;
  BOOLEAN    Self;
};

#ifdef RPC_HDR
%typedef void (*ActionProc)(GraphNumber, GraphEvent);
#endif

struct ActionProcExt
{
  ActionProc Action;
  t_clientid ClientID;
};

struct EvalFunctionExt
{
  EvalFunction Function;
  t_clientid   ClientID;
};

#ifdef RPC_HDR
%#include "xdr/special.h"
#endif

%#define _PRIM_H_RPCGEN
