%/*
% *
% * Header fuer die Schema-Funktionen von GRAS.
% *
% */
%

%#include "xdr/prim.h"
%#include "xdr/special.h"

typedef NameStringType SchemeString;


%
struct C_DeclareScheme
{
  SchemeString          Name;
};
%
struct C_CommitScheme
{
  SchemeNumber Scheme;
};
%
%/*
% * Liefert Nummer des Schemas.
% */
struct C_TestAndShowSchemeNumber
{
  SchemeString          Name;
};
%
struct S_TestAndShowSchemeNumber
{
  BOOLEAN      Existent;
  SchemeNumber Scheme;
};
%
struct C_ShowSchemeName
{
  SchemeNumber Scheme;
};
%
%/*
% * Setzen der Versionsnummer des Schemas.
% */
struct C_SetSchemeVersion
{
  SchemeNumber Scheme;
  SchemeString          Version;
};
%
struct C_ShowSchemeVersion
{
  SchemeNumber Scheme;
};
%
%/*
% * Deklariert Knotenklasse fuer Schema.
% */
struct C_DeclareNodeClass
{
  SchemeNumber Scheme;
  SchemeString          Name;
  TypeNumber   NodeClass;
};
%
%/*
% * Liefert Nummer der Knotenklasse.
% */
struct C_TestAndShowNodeClassNumber
{
  SchemeString          Name;
};
%
struct S_TestAndShowNodeClassNumber
{
  BOOLEAN    Existent;
  TypeNumber Nodeclass;
};
%
%/*
% * Liefert externen Knotenklassen-Bezeichner.
% */
struct C_ShowNodeClassName
{
  TypeNumber   Nodeclass;
};
%
struct S_ShowNodeClassName
{
  SchemeString     Name;
  BOOLEAN Existent;
};
%
%/*
% * Liefert alle Kontentypen von Nodeclass und dessen Nachfahren.
% */
struct C_ShowAllNodeTypes
{
  TypeNumber   Nodeclass;
};
%
%/*
% * Ordnet eine Klasse einer anderen unter.
% */
struct C_AppendNodeClass
{
  SchemeNumber Scheme;
  SchemeString          Nodeclass;
  SchemeString          Superclass;
};
%
%/*
% * Liefert alle direkten Superklassen.
% */
struct C_ShowDirectSuperclasses
{
  TypeNumber   NodeclassOrType;
};
%
%/*
% * Liefert alle direkten Subklassen.
% */
struct C_ShowDirectSubclasses
{
  TypeNumber   Nodeclass;
};
%
%/*
% * Liefert alle direkten Subtypen.
% */
struct C_ShowDirectSubtypes
{
  TypeNumber   Nodeclass;
};
%
%/*
% * Deklariert Knotentyp fuer ein Schema.
% */
struct C_DeclareNodeType
{
  SchemeNumber Scheme;
  BOOLEAN      UseMembership;
  SchemeString          Name;
  TypeNumber   NType;
};
%
%/*
% * Liefert Knotentypnummer.
% */
struct C_TestAndShowNodeTypeNumber
{
  SchemeString          Name;
};
%
struct S_TestAndShowNodeTypeNumber
{
  BOOLEAN    Existent;
  TypeNumber NType;
};
%
%/*
% * Liefert externen Bezeicher des Knotentypen.
% */
struct C_ShowNodeTypeName
{
  TypeNumber   NType;
};
%
struct S_ShowNodeTypeName
{
  SchemeString     Name;
  BOOLEAN Existent;
};
%
%/*
% * Ordnet einen Knotentypen einer Knotenklasse unter.
% */
struct C_AppendNodeType
{
  SchemeNumber Scheme;
  SchemeString          NType;
  SchemeString          NodeClass;
};
%
%/*
% * Deklariert Kantentypen fuer ein Schema.
% */
struct C_DeclareEdgeType
{
  SchemeNumber Scheme;
  SchemeString          Name;
  SchemeString          SourceNodeCT;
  Cardinality  SourceCardinality;
  SchemeString          TargetNodeCT;
  Cardinality  TargetCardinality;
  TypeNumber   Edge;
};
%
%/*
% * Liefert interne Nummer des Kantentypen zurueck.
% */
struct C_TestAndShowEdgeTypeNumber
{
  SchemeString          Name;
};
%
struct S_TestAndShowEdgeTypeNumber
{
  BOOLEAN    Existent;
  TypeNumber Edge;
};
%
%/*
% * Liefert externen Namen des Kantentypen.
% */
struct C_ShowEdgeTypeName
{
  TypeNumber   Edge;
};
%
struct S_ShowEdgeTypeName
{
  SchemeString     Name;
  BOOLEAN Existent;
};
%
struct C_ShowEdgeTypeProps
{
  TypeNumber   Edge;
};
%
struct S_ShowEdgeTypeProps
{
  TypeNumber  SourceCT;
  Cardinality SourceCardinality;
  TypeNumber  TargetCT;
  Cardinality TargetCardinality;
  BOOLEAN     Existent;
};
%
%/*
% * Deklariert Attribut fuer Schema.
% */
struct C_DeclareAttribute
{
  SchemeNumber    Scheme;
  SchemeString             DeclareForCT;
  SchemeString             Name;
  AttributeKind   Kind;
  IndexProperties Props;
  ValueTypeNumber ValueType;
  Cardinality	  ValueCardinality;
  BOOLEAN         ConstantLength;
  CARDINAL        Length;
  AttributeNumber AttNo;
};
%
%/*
% * Liefert interne Nummer des Attributs.
% */
struct C_TestAndShowAttributeNumber
{
  SchemeString          Name;
};
%
struct S_TestAndShowAttributeNumber
{
  BOOLEAN         Existent;
  AttributeNumber AttNo;
};
%
%/*
% * Liefert externen Bezeichner des Attributs.
% */
struct C_ShowAttributeName
{
  AttributeNumber AttNo;
};
%
struct S_ShowAttributeName
{
  SchemeString     Name;
  BOOLEAN Existent;
};
%
struct C_ShowAttributeProps
{
  AttributeNumber AttNo;
};
%
struct S_ShowAttributeProps
{
  AttributeKind   Kind;
  IndexProperties Props;
  ValueTypeNumber ValueType;
  Cardinality     AttCard;
  BOOLEAN	  ConstLength;
  CARDINAL	  Length;
  BOOLEAN	  Existent;
};
%
struct C_PutMetaAttribute
{
  TypeNumber	  NType;
  AttributeNumber AttNo;
  CARDINAL	  AttBegin;
  AOBP            Val;
  BOOLEAN	  Truncate;
};
%
struct C_GetMetaAttribute
{
  TypeNumber	  NType;
  AttributeNumber AttNo;
  CARDINAL	  AttBegin;
  CARDINAL	  AttLength;
};
%
%/*
% * Legt eine Berechnungsfunktion fest.
% */
struct C_SetEvaluationFunction
{
  SchemeNumber Scheme;
  SchemeString          SetForCT;
  SchemeString          SetForAttribute;
  SchemeString          Name;
};
%
%/*
% * Liefert externen Namen der Berechnungsfunktion.
% */
struct C_ShowEvaluationFunction
{
  AttributeNumber AttNo;
  TypeNumber	  NodeClassOrType;
};
%
struct S_ShowEvaluationFunction
{
  SchemeString     Evaluationfunction;
  BOOLEAN Existent;
};
%
struct C_BindEvaluationFunction
{
  SchemeString          Name;
  EvalFunction Function;
};
%
%/*
% * Liefert Adresse der Berechnungsfunktion.
% */
struct C_ShowBindedEvaluationFunction
{
  SchemeString     Evaluationfunction;
};
%
struct S_ShowBindedEvaluationFunction
{
  EvalFunction     Function;
  BOOLEAN Existent;
};
%
%/*
% * Definiert eine funktionale Attributabhaengigkeit.
% */
struct C_DeclareDependency
{
  SchemeNumber   Scheme;
  SchemeString            NodeClassOrType;
  SchemeString            Dependent;
  SchemeString            DependsOn;
  SchemeString            NameOfEdge;
  DependencyKind KindOfDependency;
};
%
%/*
% * Loescht eine deklarierte Abhaengigkeit.
% */
struct C_DeleteDependencies
{
  SchemeString          NodeClassOrType;
  SchemeString          Dependent;
  BOOLEAN      RestoreInheritance;
};
%
%/*
% * Liefert Menge der fuer Nodetype deklarierten Attribute.
% */
struct C_ShowAllAttributesOfNodeType
{
  TypeNumber   NType;
};
%
%/*
% * Liefert alle Knoten einer Knotenklasse.
% */
struct C_GetAllNodesOfNodeClass
{
  TypeNumber   NodeClass;
};
%
%/*
% * Liefert alle Knoten des Typs 'nodetype'.
% */
struct C_GetAllNodesOfNodeType
{
  TypeNumber   NType;
};


%
%/*
% * Prueft, ob ein Knoten in einer Knotenklasse liegt.
% */
struct C_TestIsNodeOfNodeClass
{
  TypeNumber   NodeClass;
  TypeNumber   Node;
};
%
%/*
% * Schaltet Legalitaetstests ein oder aus.
% */
struct C_SetErrorCheckMode
{
  BOOLEAN     ErrorcheckOn;
};
%
